local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
local _dbg = require "coaf.debug"
local __flg, FEATURES = pcall( require, "features" )
if not __flg or type(FEATURES) ~= "table" then
	FEATURES = {
	}
end

local dset = {}
--todo
------------------------------------------------------视频源/平台源 发现-------------------------------------
--发现外部设备/平台的信息
local function API_externalInfo(red,args)
	local ret = {}

	if FEATURES.FEATURE_NDI_DISCOVERY then
		local val = 		
		{
			status = "ready",
			manual = true,
			id = "0", --id 如何设置
			name = "NDI Sources",
			readonly = false,
			username = "NDI",
			type = "NDI"
		}
		table.insert( ret, val )
	end

	if FEATURES.FEATURE_DEV_DISCOVERY then
		local proxy = CO.GetProxy("/","CoManager")
		if not proxy then
			MSG.ERROR("coManagerError")
		else
			local r,data,err = proxy.GetAllInfos()
			proxy:Destroy()
			if not r or not data then
				MSG.ERROR("getexternalInfoError")
				return true
			else
				for k, val in pairs(data) do
					if val then
						table.insert( ret, val )
					end
				end
			end
		end
	end

	MSG.OK("", { data = ret })
	return true
end


--手动添加连接---------------------
local function API_externalLink(red,args)
	if not FEATURES.FEATURE_DEV_DISCOVERY then
		MSG.OK("")
		return true
	end
	
	if not  args.address or not args.port then
		MSG.ERROR( "argumentError" )
		return true
	end
	-- if not args.name then
	-- 	MSG.ERROR( "无效的提交参数" )
	-- 	return true
	-- end
	local proxy = CO.GetProxy("/","CoManager")
	if not proxy then
		MSG.ERROR("coManagerError")
	else
		local tabs = {}
		tabs.address=args.address
		tabs.port=args.port
		tabs.web_port=args.web_port
		tabs.username=args.username 
		tabs.password=args.password 
		tabs.departmentName=args.departmentName 
		tabs.link_token=args.link_token 
		tabs.type = args.type
		tabs.readonly = args.readonly
		local r,data,err = proxy.Add(tabs)
		proxy:Destroy()
		if not r or not data then
			-- MSG.ERROR("externalLinkError"..err)
			MSG.ERROR("externalLinkError")
		else
			MSG.OK("")
		end
	end	
	return true
end

--修改外部列表中的 外部的平台/终端
local function API_externalModify(red,args)
	if not FEATURES.FEATURE_DEV_DISCOVERY then
		MSG.OK("")
		return true
	end

	if not args.address or not args.port or not args.LINK_TOKEN  then
		MSG.ERROR( "argumentError" )
		return true
	end

	local proxy = CO.GetProxy("/","CoManager")
	if not proxy then
		MSG.ERROR("coManagerError")
	else
		local tabs ={}
		-- tabs.name=args.name
		tabs.id=args.id
		tabs.address=args.address
		--当修改平台信息的时候，会将port 替换为 平台返回的信息 --规避前端传值异常的问题
		tabs.port = args.COAF_PORT or args.port
		tabs.web_port=args.web_port
		tabs.username=args.username
		tabs.password=args.password
		tabs.department=args.department
		tabs.link_token=args.LINK_TOKEN or args.link_token
		tabs.type = args.type
		local r,data,err = proxy.Modify(tabs.id,tabs)
		proxy:Destroy()
		if not r or not data then
			MSG.ERROR("externalModifyError")
		else
			MSG.OK("")
		end
	end
	return true
end

--删除外部列表中的 外部的平台/终端
local function API_externalRemove(red,args)
	if not FEATURES.FEATURE_DEV_DISCOVERY then
		MSG.OK("")
		return true
	end
	-- todo 处理删除id=NDI的发现
	if not args.id then
		MSG.ERROR( "argumentError" )
		return true
	end

	local proxy = CO.GetProxy("/","CoManager")
	if not proxy then
		MSG.ERROR("coManagerError")
	else
		local r,data,err = proxy.Remove(args.id)
		proxy:Destroy()
		if not r or not data then
			MSG.ERROR("externalRemoveError")
		else
			MSG.OK("")
		end
	end
	return true
end

--[[
	--连接/断开  与外部（平台/终端）连接
	local function API_externalControl(red,args)
		if not args.id or not args.status then
			MSG.ERROR( "无效的提交参数" )
			return true
		end

		MSG.OK()
		return true
	end
]]


---查询 ‘指定’ 外部的平台/终端中的 流资源
local function API_externalSourceList(red,args)
	if not args.platId then
		MSG.ERROR( "argumentError" )
		return true
	end

	if FEATURES.FEATURE_NDI_DISCOVERY then
		local proxy = CO.GetProxy("/discovery/ndi","codec")
		if not proxy then
			MSG.ERROR("ndiDiscoveryError")
		else
			if tonumber(args.platId) == 0 then
				local r,more = proxy.GetNDISources()
				proxy:Destroy()
				if not r or type(more) ~= "table" then
					MSG.ERROR( "Failure to get discovery list" ) --TODO: Error code?
				else
					local rslt = {}
					local k,v
					for k,v in pairs(more) do
						v.group_name = v.group
						local dname, cname = v.name:match("(.-)%s+%((.+)%)")
						if not dname then
							v.device_name = v.name
							v.channel_name = "unknown"
						else
							v.device_name = dname
							v.channel_name = cname
						end
						v.series = "NDI" --TODO
						v.enable = 1
						v.url = v.original_url
						table.insert(rslt,v)
					end

					table.sort( rslt, function(a,b)
						if a.device_name ~= b.device_name then
							return a.device_name < b.device_name
						else
							return a.channel_name < b.channel_name
						end
					end)

					local group_rslt={}
					local prev_group = nil
					local prev_groupname = nil

					for k=1, #rslt do
						rslt[k].id = tostring(k)

						if not prev_group or prev_groupname ~= rslt[k].device_name then
							prev_group = {name=rslt[k].device_name}
							prev_group.children = {rslt[k]}
							prev_group.children_num = 1
							prev_groupname = rslt[k].device_name
							table.insert(group_rslt, prev_group)
						else
							prev_group.children_num = prev_group.children_num + 1
							rslt[k].device_name = nil --Remove the device name.
							table.insert( prev_group.children, rslt[k] )
						end
					end

					for i, v in ipairs(group_rslt) do
						if v.children_num == 1 then
							group_rslt[i] = v.children[1]
						end
						group_rslt[i].idx = i
						group_rslt[i].platId = args.platId
						group_rslt[i].tree_id = tostring(args.platId) .. "-" .. tostring(i)
					end
					MSG.OK("", {
						data = group_rslt, data_size = #group_rslt
					})
					return true
				end
			end
		end
	end

	if FEATURES.FEATURE_DEV_DISCOVERY then
		local proxy = CO.GetProxy("/","CoManager")
		if not proxy then
			MSG.ERROR("coManagerError")
		else
			local r,data,err
			if tonumber(args.platId) == 1 then
				r,data,err = proxy.GetPlatStreamerInfo("1",args.username)
			elseif (tonumber(args.platId) ~= 0 and  tonumber(args.platId) ~= 1) then
				r,data,err = proxy.GetStreamerInfo(args.platId)
			else
				r = true
				data = {{}}
			end
			proxy:Destroy()
			if not r or not data then
				MSG.ERROR("externalFindError")
			else
				MSG.OK("",{data=data})
			end
		end
	end

	return true
end

--简单的URL检查 必须包含-> protocol://orig_url_tail 文件类型除外
local function CheckURL(url)
	local protocol, orig_url_tail
	if type(url) == "table"  then --file type
		if #(url or {}) == 0 then
			return false
		else
			return true
		end
	end

	protocol, orig_url_tail = url:match( "(%w+)://(.+)$" )
	if not protocol or not orig_url_tail then
		return false
	end

	return true
end

--将外部平台/终端 选择的源 添加到当前设备的视频源
--将数组中的流 直接调用AddNetworkStream 新增流
local function API_externalSourceAdd(red,args)
	if not args.data or not args.platId then
		MSG.ERROR( "argumentError" )
		return true
	end

	local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	else
		for k,v in pairs(args.data) do
			-- local file_,stream_id = string.match(v.id,"(%a+).(%a+)")
			local r, done, id 
			if args.type and args.type == "NDI" then
				local main_options = 
				{			
					type = "ndi",
					url = "ndi://"..v.url,
					ndi_name =  v.name,
					preview_mode =  "0",
					buffer_option = "live:200:200:500:0:0"
				}

				local sub_options = 
				{			
					type = "ndi",
					url = "ndi://"..v.url,
					ndi_name =  v.name,
					preview_mode =  "1",
					buffer_option = "live:200:200:500:0:0"
				}
				--ngx.log(ngx.ERR, string.format("main_options len = %s", main_options.url))
				r, done = proxy.AddNetworkStream( "MANUAL.", v.name, main_options, false, true, sub_options)
				if r and done then
					some_ok = true
				end
			else
				local sub_enable = true
				local origin_name = args.origin_name
				if not args.origin_name or args.origin_name == "平台" then
					origin_name = v.origin_name
				end 

				if v.type == "local" then
					v.type = "rtsp"
				end

				if not CheckURL(v.sub_url) then
					sub_enable = false
				end

				local main_options =
				{
					type = v.type or "rtsp",
					url = v.manual_url,
					buffer_option = v.buffer or "live:200:200:500:0:0",
					username = v.user or nil,
					password = v.password or nil,
					bind =  v.bind_net or nil,
					transport = v.trans_mode or nil,
					rtp_video_port = (v.video_port and tonumber(v.video_port)) or nil,
					rtp_audio_port = (v.audio_port and tonumber(v.audio_port)) or nil,
					rtp_video_codec = v.video_codec or nil,
					rtp_audio_codec = v.audio_codec or nil,
					connect_speed = (v.connect_speed and tonumber(v.connect_speed)) or 2,
					imgUrl = v.PlatIP .. "/convert/"..v.imgUrl  or nil
				}

				local sub_options =
				{
					type = v.type or "rtsp",
					url = v.sub_url,
					buffer_option = v.buffer or "live:200:200:500:0:0",
					username = v.user or nil,
					password = v.password or nil,
					bind =  v.bind_net or nil,
					transport = v.trans_mode or nil,
					rtp_video_port = (v.video_port and tonumber(v.video_port)) or nil,
					rtp_audio_port = (v.audio_port and tonumber(v.audio_port)) or nil,
					rtp_video_codec = v.video_codec or nil,
					rtp_audio_codec = v.audio_codec or nil,
					connect_speed = (v.connect_speed and tonumber(v.connect_speed)) or 2,
					imgUrl = v.PlatIP .. "/convert/"..v.imgUrl  or nil
				}

				r, done = proxy.AddNetworkStream("M.".. v.id ..".".. v.platId, origin_name..":"..v.name, main_options, v.islocal, sub_enable, sub_options)
			end

			if not r or not done then
				MSG.ERROR( "externalAddError" )
				proxy:Destroy()
				return true
			else
			end
		end
	end
	proxy:Destroy()
	MSG.OK()
	return true
end

local function API_GetAuthcode(red,args)
	if not FEATURES.FEATURE_DEV_DISCOVERY then
		MSG.OK("")
		return true
	end

	local proxy = CO.GetProxy( "/", "CoManager" )
	if not proxy then
		MSG.ERROR( "codecError" )
	else
		local r,link_token,err = proxy.link_token()
		proxy:Destroy()
		if not r or not link_token then
			MSG.ERROR( "linkTokenError" )
		else
			local data={authCode=link_token}
			MSG.OK("",{data=data})
		end
	end
	return true
end

local function API_SetAuthcode(red,args)
	if not FEATURES.FEATURE_DEV_DISCOVERY then
		MSG.OK("")
		return true
	end

	if not args or not args.authCode then
		MSG.ERROR( "argumentError" )
		return true
	end
	local proxy = CO.GetProxy( "/", "CoManager" )
	if not proxy then
		MSG.ERROR( "codecError" )
	else
		local r,data,err = proxy.Modify_link_token(args.authCode)
		proxy:Destroy()
		if not r or not  data then
			MSG.ERROR( "setLinkTokenError" )
		else
			MSG.OK("")
		end
	end
	return true
end


local function API_Modify_link_name(red, args)
	if not FEATURES.FEATURE_DEV_DISCOVERY then
		MSG.OK("")
		return true
	end

	local proxy = CO.GetProxy( "/", "CoManager" )
	if proxy then
		local r, done = proxy.Modify_link_name(args.name)
		proxy:Destroy()
		if r and done then
			MSG.OK( "")
			return true
		else
			MSG.ERROR( "setLinkNameError")
			return true
		end
	else
		return false, "codecError" 
	end
end

local function API_getDiscoveryServer(red,args)
	local proxy = CO.GetProxy( "/discovery", "codec" )
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	else
		local result = {}
		local r, enable, server = proxy.NDI_GetDiscoveryServer()
		if not r then
			MSG.ERROR( "Get NDI Discovery Server ERROR!")
		else
			result.enable = enable or false
			result.address = server or ""
			MSG.OK( "", {
				data = result
			})
		end
		proxy:Destroy()
		return true
	end
end

local function API_setDiscoveryServer(red, args)
	if args and args.address then
		local proxy = CO.GetProxy( "/discovery", "codec" )
		if not proxy then
			MSG.ERROR( "codecError" )
			return true
		else
			local r, done = proxy.NDI_SetDiscoveryServer(args.enable , args.address)
			if not r or not done then
				MSG.ERROR( "Set NDI Discovery Server ERROR!")
			else
				MSG.OK()
			end
			proxy:Destroy()
			return true
		end
	
	else
		MSG.ERROR( "无效的NDI discovery server参数" )
	end
	return true
end

--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["externalLink"] = API_externalLink
dset.APIS["externalModify"] = API_externalModify
dset.APIS["externalInfo"] = API_externalInfo
dset.APIS["externalRemove"] = API_externalRemove
dset.APIS["externalSourceAdd"] = API_externalSourceAdd
dset.APIS["externalSourceList"] = API_externalSourceList

dset.APIS["SetAuthcode"] = API_SetAuthcode
dset.APIS["GetAuthcode"] = API_GetAuthcode

dset.APIS["get_discovery_server"] = API_getDiscoveryServer
dset.APIS["set_discovery_server"] = API_setDiscoveryServer

dset.APIS["Modify_link_name"] = API_Modify_link_name


--------------------------------------
return dset

