local msg = require "api_msg"
local co = require "api_coservice"
local cjson = require "cjson"
local upload = require "resty.upload"

local dset = {}

local function API_get()
	local proxy = co.GetProxy("/preset", "systemctrl", nil, 3)
    if not proxy then
        msg.ERROR( "systemError" )
    end

    local r, result = proxy.GetPresetList()
    proxy:Destroy()
    if r and result then
        msg.OK("", {data = result})
    end
    return true
end

local function API_add( _, args)
	local proxy = co.GetProxy("/preset", "systemctrl", nil, 3)
    if not proxy then
        msg.ERROR( "systemError" )
    end

    local r, result = proxy.AddPreset(args.name, args.desc)
    proxy:Destroy()
    if r and result then
        msg.OK("", {data = result})
    else
        msg.ERROR( "添加预设失败！" )
    end
    return true
end

local function API_delete( _, args)
	local proxy = co.GetProxy("/preset", "systemctrl", nil, 3)
    if not proxy then
        msg.ERROR( "systemError" )
    end

    local r, result = proxy.DeletePreset(args.id)
    proxy:Destroy()
    if r and result then
        msg.OK("", {})
    else
        msg.ERROR( "删除预设失败！" )
    end
    return true
end

local function API_modify( _, args)
	local proxy = co.GetProxy("/preset", "systemctrl", nil, 3)
    if not proxy then
        msg.ERROR( "systemError" )
    end

    local r, result = proxy.ModifyPreset(args.id, args.name, args.desc)
    proxy:Destroy()
    if r and result then
        msg.OK("", {})
    else
        msg.ERROR( "修改预设失败！" )
    end
    return true
end

local function API_export( _, args)
	local proxy = co.GetProxy("/preset", "systemctrl", nil, 3)
    if not proxy then
        msg.ERROR( "systemError" )
    end

    local r, result = proxy.ExportPreset(args.id)
    proxy:Destroy()
    if r and result then
        msg.OK("", {result})
    else
        msg.ERROR( "导出预设失败！" )
    end
    return true
end

local function checkPartType(res)
	if type(res) == "table" then
		local idr = res[1]:lower()
		if idr == "content-disposition" then
			local fieldname, filename = res[2]:match('name=%"(.*)%".*filename=%"(.*)%"')
			if not fieldname then
				fieldname = res[2]:match('name=%"(.*)%"')
			end
			if not fieldname then
				return nil
			end
			if not filename then
				return "field", fieldname
			else
				return "file", fieldname, filename
			end
		elseif idr == "content-type" then
			return idr, res[2] or ""
		else
			return nil
		end
	else
		return nil
	end
end

local STORE_PATH = "/tmp"
local function getUploadFile(args)
	args = args or {}
	local form,err = upload:new(8192)
	if not form then
		return nil, "无法开启上传文件:" .. (err or "")
	end
	form:set_timeout(5*1000)        --5secs timeout

	local fieldType, fieldName
	local srcFileName
	local curFile

	while true do
		local typ, res, err = form:read()
		if not typ then
			if curFile then
				curFile:close()
			end
			return nil, "读取上传内容失败"
		end

		if typ == "header" then
			local ftype,fname,filename = checkPartType(res)
			if ftype == "field" then
				fieldType = ftype
				fieldName = fname
				args[fieldName] = ""
			elseif ftype == "file" then
				fieldType = ftype
				fieldName = fname
				srcFileName = filename
				if srcFileName == "" then
					return nil, "无效的上传文件(检查文件名失败)"
				end
				-- if not srcFileName:match("%.[Zz][Ii][Pp]$") then
				-- 	srcFileName = srcFileName .. ".zip"
				-- end

			elseif ftype == "content-type" then
				if fieldType ~= "file" then
					if curFile then
						curFile:close()
					end
					return nil, "无效的上传内容：错误的Content-Type出现位置"
				elseif fname ~= "application/gzip" and fname ~= "application/x-gzip" and fname ~= "application/x-gtar" and fname~="application/x-tgz" and fname ~= "application/octet-stream"  then
					if curFile then
						curFile:close()
					end
					return nil, "上传文件必须是.tar.gz文件" .. fname
				else
					local openerrormsg
					curFile, openerrormsg = io.open( STORE_PATH .. "/" .. srcFileName, "wb" )
					if not curFile then
						return nil, "无法创建上传文件"
					end
				end
			else
				filedType = nil
				fieldName = nil
				if curFile then
					curFile:close()
					curFile = nil
				end
			end
		elseif typ == "body" then
			if fieldType == "field" then
				args[fieldName] = args[fieldName] .. res
			elseif fieldType == "file" then
				if not curFile then
					return nil, "上传文件失败"
				end
				curFile:write( res )
			end
		elseif typ == "part_end" then
			filedType = nil
			fieldName = nil
			if curFile then
				curFile:close()
				curFile = nil
			end
		elseif typ == "eof" then
			break
		end
	end

	if curFile then
		curFile:close()
	end

	if not srcFileName then
		return nil, "没有上传文件"
	end

	return srcFileName
end

local function API_upload_preset( _, args)
	local srcFileName, err = getUploadFile(args)
	if not srcFileName then
		return false, err or "文件上传错误"
	end
    local proxy = co.GetProxy("/preset", "systemctrl", nil, 3)
    if not proxy then
        msg.ERROR( "systemError" )
	else
		local r, result = proxy.ImportPreset(STORE_PATH .. "/" .. srcFileName)
		proxy:Destroy()
		if r and result then
            msg.OK("", {})
        else
            msg.ERROR( "修改预设失败！" )
        end
        return true
	end

	return true
end

local function API_load_preset( _, args)
	local proxy = co.GetProxy("/preset", "systemctrl", nil, 3)
    if not proxy then
        msg.ERROR( "systemError" )
    end

    local r, result = proxy.LoadPreset(args.id)
    proxy:Destroy()
    if r and result then
        msg.OK("", {})
    else
        msg.ERROR( "应用预设失败！" )
    end
    return true
end





--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get        --获取预设列表
dset.APIS["add"] = API_add        --添加预设
dset.APIS["delete"] = API_delete  --添加预设
dset.APIS["modify"] = API_modify  --修改预设
dset.APIS["export"] = API_export  --导出预设
dset.APIS["upload_preset"] = API_upload_preset  --导入预设
dset.APIS["load_preset"] = API_load_preset  --应用预设


--------------------------------------
return dset
