local CO = require "api_coservice"
local GetProxy = CO.GetProxy
local MSG = require "api_msg"
local cjson = require "cjson"

local role = {}
role.APIS = role

function role.get(_, args)
    local proxy = GetProxy("/roles", "systemctrl")
    if not proxy then
        MSG.ERROR( "systemError" )
        return true
    end

    local is_ok, data = proxy.Get()
    proxy:Destroy()
    if not is_ok or not data then
        MSG.ERROR("requestError")
        return true
    end

    local new = {}
    for k, v  in pairs(data) do
        table.insert(new, v)
    end

    table.sort(new, function(a, b)
        return a.id < b.id
    end)

    local page = tonumber(args.page) or 1
    local limit = tonumber(args.limit) or 10

    local result = {}
    for i = (page - 1) * limit + 1, page * limit do
        table.insert(result, new[i])
    end

    MSG.OK("", {data = result, count = #new})
    return true
end

function role.getRoleItem()
    MSG.OK("", {data = 
        {
            {
                id = 1,
                name = "systemManagement",
                checked = 1,
                children = {
                    { checked = 1, name = "usersManagement", value = "users" }, 
                    { checked = 1, name = "roleManagement", value = "role" }, 
                    { checked = 1, name = "systime", value = "systime" }, 
                    { checked = 1, name = "firmwareManagement", value = "firmware" }, 
                    { checked = 1, name = "reset", value = "reset" }, 
                    { checked = 1, name = "reboot", value = "reboot" }, 
                    { checked = 1, name = "restore", value = "restore" }, 
                    { checked = 1, name = "network", value = "network" }, 
                    { checked = 1, name = "platform", value = "platform" }, 
                    { checked = 1, name = "authorization", value = "authorization" }, 
                    { checked = 1, name = "setDeviceName", value = "setDeviceName" }
                }
            }, 
            {
                id = 2,
                name = "sourceOutput",
                checked = 1,
                children = {
                    { checked = 1, name = "sourceManage", value = "sourceManage" }, 
                    { checked = 1, name = "sourceSettings", value = "setSource" }, 
                    { checked = 1, name = "terminal", value = "terminal" }, 
                    { checked = 1, name = "pattern", value = "pattern" }
                }
            }, 
            {
                id = 3,
                name = "streamingService",
                checked = 1,
                children = {
                    { checked = 1, name = "streamingService", value = "streamService" }, 
                    { checked = 1, name = "streamServiceControl", value = "streamServiceControl" }, 
                    { checked = 1, name = "publishing", value = "publishing" }, 
                    { checked = 1, name = "sip", value = "sip" }
                }
            }, 
            {
                id = 4,
                name = "audioVideoSet",
                checked = 1,
                children = {
                        { checked = 1, name = "outputFormate", value = "outputFormate" }, 
                        { checked = 1, name = "coding", value = "coding" }, 
                        { checked = 1, name = "overlay", value = "overlay" }, 
                        { checked = 1, name = "adjustment", value = "adjustment" }
                }
            },
            {
                id = 5,
                name = "recordManagement",
                checked = 1,
                children = {
                    { checked = 1, name = "recordControl", value = "recordControl" }, 
                    { checked = 1, name = "recordFile", value = "recordFile"}
                }
            }, 
            {
                id = 6,
                name = "pictureManagement",
                checked = 1,
                children = {
                    { checked = 1, name = "pictureManagement", value = "pictures" }
                }
            }
        }
    })
    return true
end

function role.add(_, args)
    local proxy = GetProxy("/roles", "systemctrl")
    if not proxy then
        MSG.ERROR( "systemError" )
        return true
    end

    if type(args) ~= "table" or not args.alias then
        MSG.ERROR("argumentError")
        return true
    end

    local is_ok, skip, err = proxy.Add({
        users = args.users,
        role = args.role, 
        systime = args.systime, 
        firmware = args.firmware,
        reset = args.reset, 
        reboot = args.reboot, 
        restore = args.restore, 
        network = args.network,
        platform = args.platform, 
        authorization = args.authorization, 
        setDeviceName = args.setDeviceName, 
        sourceManage = args.sourceManage,
        setSource = args.setSource, 
        terminal = args.terminal, 
        pattern = args.pattern, 
        streamService = args.streamService,
        streamServiceControl = args.streamServiceControl, 
        publishing = args.publishing, 
        sip = args.sip, 
        outputFormate = args.outputFormate,
        coding = args.coding, 
        overlay = args.overlay, 
        adjustment = args.adjustment, 
        recordControl = args.recordControl,
        recordFile = args.recordFile, 
        pictures = args.pictures, 
        alias = args.alias,
    })
    proxy:Destroy()
    if not is_ok or not skip then
        -- MSG.ERROR("添加失败！ " .. tostring(err or ""))
        MSG.ERROR("addFailed")
        return true
    end
    MSG.OK()
    return true
end

function role.modify(_, args)
    local proxy = GetProxy("/roles", "systemctrl")
    if not proxy then
        MSG.ERROR( "systemError" )
        return true
    end

    local _, data = proxy.Get(args.alias)
    if not data then
        MSG.ERROR("requestError")
    else
        for k in pairs(data) do
            data[k] = args[k]
        end

        local is_ok, skip, err = proxy.Modify(data)
        if not is_ok or not skip then
            -- MSG.ERROR("修改失败！" .. tostring(err))
            MSG.ERROR("modifyUserError")
        else
            MSG.OK("")
        end
    end
    proxy:Destroy()
    return true
end

function role.remove(_, args)
    local proxy = GetProxy("/roles", "systemctrl")
    if not proxy then
        MSG.ERROR( "systemError" )
        return true
    end

    local is_ok, skip, err = proxy.Remove(args.ids)
    if not is_ok or not skip then
        -- MSG.ERROR(err or "请求失败！")
        MSG.ERROR("requestError")
    else
        MSG.OK("")
    end
    return true
end

function role.getUserRole(_, user)
    local proxy = GetProxy("/roles", "systemctrl")
    if not proxy then
        return {}
    end

    local is_ok, data = proxy.Get(user)
    proxy:Destroy()
    return is_ok and data or {}
end

return role