local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
local _dbg = require "coaf.debug"

local dset = {}
--------------------------------------
local function API_get(red, args)
	local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	else
		local r, streams = proxy.GetStreams({"local","manual","remote"})
		proxy:Destroy()

		if not r or not streams then
			MSG.ERROR( "getStreamsError" )
			-- MSG.ERROR( "无法获取媒体源列表" .. (streams or "ERROR") )
			return true
		end

		local result = {
		}

		local i, item
		for i=1, #streams do
			if not args or not args.id or args.id == streams[i].id then
				if not streams[i].sourceType or streams[i].sourceType == "main" then
					local originIp,imgUrl
					if streams[i].origin =="remote" then
						local num = string.find(streams[i].imgUrl,"%/")
						originIp= string.sub(streams[i].imgUrl,1,num-1)
						imgUrl = string.sub(streams[i].imgUrl,num)
					else
						imgUrl = streams[i].imgUrl
					end
					
					--只返回主码流的动态数据
					item = {
						id = streams[i].id,
						name = streams[i].name,
						type = streams[i].type,
						src = streams[i].address,
						status = streams[i].origin == "local" and "on" or "off",
						resolution = "分辨率未知",
						name_position = "left_top", --TODO
						time_position = "left_top",
						icons = streams[i].icons,
						manual_url = streams[i].manual_url or "",
						buffer = streams[i].buffer_option or "live:200:200:500:0:0",
						bitrate = 0,
						origin= streams[i].origin,
						address  = streams[i].address or  streams[i].manual_url,
						imgUrl = imgUrl,
						originIp = originIp
					}
					
					if streams[i].stream_info then
						if streams[i].stream_info.live == "connected" then
							if streams[i].stream_info.stream_start and streams[i].stream_info.output_start then
								item.status = "view_pull"
							elseif streams[i].stream_info.stream_start then
								item.status = "pull"
							elseif streams[i].stream_info.output_start then
								item.status = "view"
							else
								item.status = "on"
							end
						else
							if (streams[i].type == "manual" and streams[i].stream_info.live and streams[i].stream_info.live ~= "") or (streams[i].stream_info.live ~= "closed" and streams[i].stream_info.live ~= "") then
								item.status = streams[i].stream_info.live
							end
						end

						item.resolution = streams[i].stream_info.resolution or "分辨率未知"
						item.bitrate = streams[i].stream_info.bitrate or 0
					end

					if streams[i].srt_status then
						item.srtStatus = streams[i].srt_status
					end

					if streams[i].rtmp_server_status then
						item.rtmp_server_status =streams[i].rtmp_server_status
					end
				end
				table.insert( result, item )
			end
		end

		MSG.OK( "", {
			data = result
		})

		return true
	end
end

--------------------------------------
--简单的URL检查 必须包含-> protocol://orig_url_tail 文件类型除外
local function CheckURL(url)
	local protocol, orig_url_tail
	if type(url) == "table"  then --file type
		if #(url or {}) == 0 then
			return false
		else
			return true
		end
	end

	protocol, orig_url_tail = url:match( "(%w+)://(.+)$" )
	if not protocol or not orig_url_tail then
		return false
	end

	return true
end

local function CreateOptions( args )
	local options = {}
	if args.type and args.type:lower() == "ndi" then
		options.url = args.manual_url
		options.type = args.type
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.ndi_name =  args.ndi_name or ""
		options.preview_mode =  args.preview_mode or 0	
		options.audio_sync_compst = args.audio_sync_compst or 0  --音频同步补偿
	elseif args.type and args.type:lower() == "srt" then
		options.type = args.type
		options.url = args.manual_url
		options.address = args.address or nil
		options.listenerPort = args.listenerPort or nil
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.connectionMode =  args.connectionMode or nil
		options.passphrase =  args.passphrase or nil
		options.encryption =  args.encryption or nil
		options.payloadSize =  args.payloadSize or 1316
		options.bandwidth =  args.bandwidth or 1316
		options.latency =  args.latency or 125
		options.connect_speed = (args.connect_speed and tonumber(args.connect_speed)) or 2
		options.showAdvanced = args.showAdvanced or 0
		options.srt_streamid = args.srt_streamid or ""
		options.audio_sync_compst = args.audio_sync_compst or 0  --音频同步补偿
	elseif args.type and args.type:lower() == "file" then
		--ngx.log(ngx.ERR, string.format("file_list len = %d", #args.file_list))
		for _, v in pairs(args.file_list) do
			v.disk = nil
			v.id = nil
			--v.name = nil
			v.totalDuration = nil
			v.work = nil
		end

		options.type = args.type
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.loop = args.loop or false
		options.file_list = args.file_list
		options.audio_sync_compst = args.audio_sync_compst or 0  --音频同步补偿
	elseif args.type and args.type:lower() == "rtmp_server" then
		options.type = args.type
		options.url = args.manual_url
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.rtmp_server_port = args.rtmp_server_port or 1935
		options.rtmp_server_enable = args.rtmp_server_enable or false
		options.rtmp_server_security = args.rtmp_server_security or "none"
		options.rtmp_server_session = args.rtmp_server_session
		options.rtmp_server_token = args.rtmp_server_token
		options.rtmp_server_application = args.rtmp_server_application
		options.username = args.user or nil
		options.password = args.password or nil
		options.connect_speed = (args.connect_speed and tonumber(args.connect_speed)) or 2
		options.audio_sync_compst = args.audio_sync_compst or 0  --音频同步补偿
	else
		options.type = args.type
		options.url = args.manual_url
		options.buffer_option = args.buffer or "live:200:200:500:0:0"
		options.username = args.user or nil
		options.password = args.password or nil
		options.bind =  args.bind_net or nil
		options.transport = args.trans_mode or nil
		options.rtp_video_port = (args.video_port and tonumber(args.video_port)) or nil
		options.rtp_audio_port = (args.audio_port and tonumber(args.audio_port)) or nil
		options.rtp_video_codec = args.video_codec or nil
		options.rtp_audio_codec = args.audio_codec or nil
		options.connect_speed = (args.connect_speed and tonumber(args.connect_speed)) or 2
		options.imgUrl = args.imgUrl or ""
		options.audio_sync_compst = args.audio_sync_compst or 0  --音频同步补偿
	end

	return options
end


local function API_modify(red, args)
	if args and args.id and args.name and args.main and args.sub then
		local proxy = CO.GetProxy( "/streams", "codec" )
		if not proxy then
			MSG.ERROR( "codecError" )
			return true
		else
			local r, done 
			if args.id:match("^LOCAL-") or args.id:match("^OUTPUT") then
				--Only change name for local or output stream.
				r, done = proxy.ChangeStreamDetail( args.id, {
					name = args.name
				})
			else
				local is_sub = true
				local main, sub

				if not CheckURL(args.main.file_list or args.main.manual_url) then
					MSG.ERROR( "Invalid main URL！" )
					proxy:Destroy()
					return true
				end

				if not CheckURL(args.sub.file_list or args.sub.manual_url) then
					is_sub = false
				end

				main = CreateOptions(args.main)
				sub = CreateOptions(args.sub)

				r, done = proxy.ChangeStreamDetail( args.id, {
					name = args.name,
					with_sub = is_sub,
					options = main,
					sub_options = sub,
					audio_enables = args.audio_enables or {}
				})		
			end

			if not r or not done then
				MSG.ERROR( "modifyStreamArgError")
				-- MSG.ERROR( "无法修改流参数:" .. (done or "未知错误") )
			else
				MSG.OK()
			end

			proxy:Destroy()
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

--------------------------------------

local function API_add(red, args)
	local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	end

	local r, done

	if args  and  args.name and  args.main and  args.sub then
		local sub_enable = true
		local islocal = false
		local main_options, sub_options

		if not CheckURL(args.main.file_list or args.main.manual_url) then
			MSG.ERROR( "Invalid main URL！" )
			proxy:Destroy()
			return true
		end

		if not CheckURL(args.sub.file_list or args.sub.manual_url) then
			sub_enable = false
		end

		main_options = CreateOptions(args.main)
		sub_options = CreateOptions(args.sub)
		r, done = proxy.AddNetworkStream( "MANUAL.", args.name, main_options, islocal, sub_enable, sub_options)
	else
		MSG.ERROR( "argumentError" )
		proxy:Destroy()
		return true
	end

	if not r or not done then
		MSG.ERROR( "addNetStreamError" )
	else
		MSG.OK()
	end
	
	proxy:Destroy()
	return true
end

--------------------------------------

local function API_remove(red, args)
	if args and args.id then
		local proxy = CO.GetProxy( "/streams", "codec" )
		if not proxy then
			MSG.ERROR( "codecError" )
			return true
		else
			local r,done = proxy.RemoveNetworkStream( args.id )
			proxy:Destroy()
			if not r or not done then
				MSG.ERROR( "delNetStreamError")
			else
				MSG.OK()
			end
			return true
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end


local function API_getSourceConfig(red, args)
	local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	else
		local r, config = proxy.GetNetSourceConfig(args.id)
		proxy:Destroy()

		if not r or not config then
			MSG.ERROR( "getSourceConfigError" )
			return true
		end
		
		if config.options then 
			config.options.user = config.options.username or ""
			config.options.username = nil
			config.options.manual_url = config.options.url or ""
			config.options.url = nil
			config.options.buffer = config.options.buffer_option or "live:200:200:500:0:0"
			config.options.buffer_option = nil
			config.options.trans_mode = config.options.transport or ""
			config.options.transport = nil
		end

		if config.sub_options then 
			config.sub_options.user = config.sub_options.username or ""
			config.sub_options.username = nil
			config.sub_options.manual_url = config.sub_options.url or ""
			config.sub_options.url = nil
			config.sub_options.buffer = config.sub_options.buffer_option or "live:200:200:500:0:0"
			config.sub_options.buffer_option = nil
			config.sub_options.trans_mode = config.sub_options.transport or ""
			config.sub_options.transport = nil
		end

		local result = {
			id = args.id,
			name = config.name or "",
			audio_enables = config.audio_enables or false,
			manual_type = config.manual_type or "manual",
			manual = config.manual or true,
			main = config.options or nil,
			sub = config.sub_options or nil
		}

		MSG.OK( "", {
			data = result
		})

		return true
	end
end

-------------------------------------------------------------------------

local function API_search(red, args)
	--TODO:
	MSG.OK( "", {
		data = {
			ONVIF = {
				{
					id = "invalid",
					name = "暂未支持发现Onvif设备<br/>",
					ip = "0.0.0.0",
					type = "IPCAM",
					flag = "ONVIF"
				}
			},
			KILOVIEW = {
				{
					id = "invalid",
					name = "暂未支持发现KILOVIEW设备<br/>",
					ip = "0.0.0.0",
					type = "ENCODER",
					flag = "KILOVIEW"
				}
			},
			["NDI-HX"] = {
				{
					id = "invalid",
					name = "暂未支持发现NDI-HX设备<br/>",
					ip = "0.0.0.0",
					type = "NDI",
					flag = "NDI-HX"
				}
			}
		}
	})
	return true
end


--一键清空所有网络源
local function API_empty(red,args)
	local proxy = CO.GetProxy( "/streams", "codec" ,"local", 10)
	if not proxy then
		MSG.ERROR( "codecError" )
		return true
	else
		local r, done,err = proxy.RemoveAllNetworkStream()
		if not r or not done then
			-- MSG.ERROR( "一键清空所有网络源 失败:",err )
			MSG.ERROR("removeAllNetStreamError")
		else
			MSG.OK("")
		end
		proxy:Destroy()

	end
	return true
end


local function API_test(red,args)
    local headers =ngx.req.get_headers()
    MSG.OK("",{data=headers})
    
    return true

end

local function ConvertDB(num, skip)
	if skip then
		return math.floor((num + 121) / (121 + 6) * 200)
	else
		return math.floor(num / 200 * (121 + 6)) - 121
	end
end

local function API_remix()
	local info = {}
	local proxy_3 = CO.GetProxy( "/audioEncoder", "codec" )
	local proxy_1 = CO.GetProxy( "/output/hd0", "codec" )
	if proxy_1 then
		local _, emb_dis, emb_gain, localEn = proxy_1.GetAudioGain("embedded")
		local _, mute = proxy_3.GetEmbedMute("hd:0")
		proxy_1:Destroy()
		info[1] = {	localOutput = localEn, checked = 0, enable = 1 - mute, volumn = tonumber(ConvertDB(emb_gain, true)), name = "neiqian1", layout = 1, id = "neiqian1"}
	end

	local proxy_2 = CO.GetProxy( "/output/hd1", "codec" )
	if proxy_2 then
		local _, emb_dis, emb_gain, localEn = proxy_2.GetAudioGain("embedded")
		local _, dis, gain, localenable = proxy_2.GetAudioGain("analog")
		local _, mute = proxy_3.GetEmbedMute("hd:1")
		proxy_2:Destroy()
		info[2] = {	localOutput = localEn, checked = 0, enable = 1 - mute, volumn = tonumber(ConvertDB(emb_gain, true)), name = "neiqian2", layout = 2, id = "neiqian2"}
		info[3] = {
			checked = 0,
			localOutput = localenable,
			name = "channel",
			id = "channel",
			device = "channel"
		}
	end

	if proxy_3 then
		local _, data = proxy_3.GetAudioInfo()
		proxy_3:Destroy()
		for _, v in pairs(data) do
			if v.name ~= "channel" then
				table.insert(info, {
					checked = 0,
					enable = 1-tonumber(v.mute),
					volumn = tonumber(v.volume),
					name = v.name,
					id = v.device,
					device = v.device,
					localOutput = v.localoutput
				})
			else
				info[3].enable = 1 - tonumber(v.mute)
				info[3].volumn = ConvertDB(tonumber(v.volume), true)
			end
		end
	end

	MSG.OK("", {data = info})
	return true
end

local function API_remixSet(_, args)
	if not args.device and not args.enable then
		if args.volumn then
			args.value = ConvertDB(args.volumn)
		end
		args.audio = "embedded"
		args.output = args.layout
		return require "api_output".APIS.audio_set(_, args)
	end

	local proxy = CO.GetProxy("/audioEncoder", "codec")
	if not proxy then
		MSG.ERROR("codecError")
		return true	
	end

	if not args.device and args.enable then
		local is_ok, skip = proxy.ModifyMuteEmbed("hd:" .. (tonumber(args.output or args.layout) - 1), tonumber(args.enable) == 0 )
		if not is_ok or not skip then
			MSG.ERROR("请求失败！")
		else
			MSG.OK("")
		end
		return true
	end

	if args.device == "channel" and args.volumn then
		args.volumn = ConvertDB(tonumber(args.volumn))
	end

	local is_ok, skip = proxy.ModifyAduioInfo(args.device, {
		mute = args.enable and (1 - args.enable) or nil,
		volume = args.volumn,
		localoutput = args.localOutput
	})
	proxy:Destroy()
	if not is_ok or not skip then
		MSG.ERROR("modifyMuteError")
	end
	MSG.OK("")
	return true
end

local function API_remixOutput(_, args)
	local output = tonumber(args.output) or 1
	local proxy = CO.GetProxy("/output/hd" .. (output-1), "codec" )
	if not proxy then
		MSG.ERROR("codecError")
		return true
	end
	
	local is_ok, data = proxy.GetAudioVolume()
	if not is_ok then
		MSG.ERROR("getRemixError")
		return true
	end
	
	local dict = {}
	for _, v in ipairs(data) do
		dict[v.stream_id] = {
			name = v.name, checked = 0, enable = tonumber(1 - v.mute), volumn = tonumber(v.volume), id = v.idx
		}
	end

	local list = {}
	for _, v in pairs(dict) do
		table.insert(list, v)
	end

	MSG.OK("", {data = list})
	return true
end

local function API_remixOutputSet(_, args)
	if not tonumber(args.output)  then
		MSG.ERROR("argumentError")
		return true
	end

	local proxy = CO.GetProxy("/output/hd" .. (tonumber(args.output) - 1), "codec")
	if not proxy then
		MSG.ERROR("codecError")
		return true
	end

	local is_ok, skip, err = proxy.EnableAudio(args.id , tonumber(args.enable) == 1)
	if not is_ok or not skip then
		-- MSG.ERROR("修改失败！" .. tostring(err))
		MSG.ERROR("modifyMuteError")
		return true
	end

	is_ok, skip, err = proxy.ModifyAudioVolume(args.id, args.volumn, 1 - args.enable)
	proxy:Destroy()
	if not is_ok or not skip then
		-- MSG.ERROR("修改失败！" .. tostring(err))
		MSG.ERROR("modifyMuteError")
		return true
	end
	MSG.OK("")
	return true
end

local function API_getRtmpServicePort(red,args)
	local proxy = CO.GetProxy("/rtmp_service", "rtmp_service")
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, done = proxy.GetServerPort()
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "getServerPort" )
		else
			MSG.OK("",{data= {rtmp_server_port = done}})
		end
		return true
	end
end

local function API_setRtmpServicePort(red,args)
	if not args.rtmp_server_port  then
		MSG.ERROR( "argumentError" )
		return true
	end

	local proxy = CO.GetProxy("/rtmp_service", "rtmp_service")
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, done = proxy.UpdateServerPort(args.rtmp_server_port)
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "setServerPort" )
			return true
		--else
		--	MSG.OK("")
		end
	end

	local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	else
		local r, done = proxy.UpdateRtmpSourcePort(args.rtmp_server_port)
		proxy:Destroy()
		if not r or not done then
			MSG.ERROR( "setServerPort" )
		else
			MSG.OK("")
		end
	end

	return true		
end

--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get
dset.APIS["add"] = API_add
dset.APIS["remove"] = API_remove
dset.APIS["modify"] = API_modify
dset.APIS["search"] = API_search
dset.APIS["getSourceConfig"] = API_getSourceConfig

dset.APIS["empty"] = API_empty
dset.APIS["test"] = API_test
dset.APIS["remix"] = API_remix
dset.APIS["remixSet"] = API_remixSet
dset.APIS["remixOutput"] = API_remixOutput
dset.APIS["remixOutputSet"] = API_remixOutputSet

dset.APIS["getRtmpServicePort"] = API_getRtmpServicePort
dset.APIS["setRtmpServicePort"] = API_setRtmpServicePort
--------------------------------------
return dset

