local sysinfo = require "api_sysinfo"
local MSG = require "api_msg"
local CO = require "api_coservice"
local cjson = require "cjson"
pcall( require, "features" )
local __flg, FEATURES = pcall( require, "features" )
if not __flg or type(FEATURES) ~= "table" then
	FEATURES = {
	}
end

local dset = {}

local function API_server_info(red, args)
	local cpu_c = sysinfo.GetCpuCores()
	local cpu_p, cpu_i = sysinfo.GetCpuLoad()
	local mem_f, mem_t = sysinfo.GetMemUse()
	local pers_m, pers_t = sysinfo.GetPersisTime()

	local info = {
		addr = ngx.var.server_addr,
		name = ngx.var.scheme .. "://" .. ngx.var.host,
		port = ngx.var.server_port,
		persis = "" .. math.floor(pers_m/3600) .. " 小时 " .. math.floor((pers_m%3600)/60) .. " 分 " .. math.floor(pers_m%60) .. "秒",
		start_time = string.format( "%04d-%02d-%02d %02d:%02d:%02d", pers_t.year, pers_t.month, pers_t.day, pers_t.hour, pers_t.min,  pers_t.sec ),
		cpu_cores = cpu_c,
		cpu_payload = math.floor(cpu_p*100)/100,
		mem_used = math.floor( ((mem_t - mem_f)/1024)*100 )/100,
		mem_total = math.floor( (mem_t/1024)*100 ) / 100
	}

	MSG.OK( "", { data = info } )
	return true
end

local function API_reset(red,args)
	local proxy = CO.GetProxy( "/", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		proxy.Reset()
		proxy:Destroy()
		MSG.OK()
	end

	return true
end

local function API_reboot(red,args)
	if FEATURES.FEATURE_LOCAL_STREAM then
		local proxy = CO.GetProxy( "/local", "CoManager" )
		if proxy then
			local r, data,err = proxy.GetLocalEnInfo()
			proxy:Destroy()
			if r and data then
				for k,v in pairs(data) do
					if v.id then
						local amba_proxy = CO.GetProxy( "/", "systemctrl",v.id )
						if not amba_proxy then
							-- MSG.ERROR( "无法连接系统服务以控制系统重启！编码模块：" .. v.id )
							MSG.ERROR("systemError")
						else
							amba_proxy.Reboot(1)
							amba_proxy:Destroy()
						end
					end
				end
			end
		else
			MSG.ERROR( "coManagerError" )
		end
	end

	local proxy = CO.GetProxy( "/", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		proxy.Reboot(3)
		proxy:Destroy()
	end

	MSG.OK()
	return true
end

local function API_restore(red,args)

	local proxy = CO.GetProxy( "/", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		proxy.RestoreFactory()
		proxy:Destroy()
		MSG.OK()
	end
	return true
end

local function API_check_master(red,args)
	if _G.FEATURE_WORK == "master" then
		MSG.OK(_G.FEATURE_WORK)
		return true
	end

	local proxy = CO.GetProxy( "/", "coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
		return true
	end
	local r, ret = proxy.GetLocations( "MASTER", "online" )
	if not r or not ret or not ret["MASTER"] then
		MSG.ERROR( "locationError" )
	else
		MSG.OK()
	end
	proxy:Destroy()
	return true
end

local cached_versions
local cached_sn

local function API_status(red, args)
	if not cached_versions or not cached_sn then
		local proxy = CO.GetProxy( "/", "systemctrl" )
		if proxy then
			local r
			r, cached_versions = proxy.GetVersion()
			r, cached_sn = proxy.GetSerialNo()
			proxy:Destroy()
		end
	end

	local cpu_c = sysinfo.GetCpuCores()
	local cpu_p, cpu_i = sysinfo.GetCpuLoad()
	local mem_f, mem_t = sysinfo.GetMemUse()
	local pers_m, pers_t = sysinfo.GetPersisTime()
	-- software_version  hardware_version firmware_version
	local info = {
		softwareVersion = cached_versions and cached_versions.SOFTWARE_VERSION or "1.00.0000",
		hardwareVersion = cached_versions and cached_versions.HARDWARE_VERSION or "1.0",
        firmwareVersion  = cached_versions and cached_versions.FIRMWARE_VERSION or "0.00",

		mem_total = math.floor((mem_t/1024)*100)/100,
		mem_free = math.floor((mem_f/1024)*100)/100,
		cpu_payload = math.floor(cpu_p*100)/100,
		persis_time = pers_m,
		serial_no = cached_sn or "00000000",
		cpu_cores = cpu_c
	}

	MSG.OK( "", { data = {info} } )
	return true
end

local function API_advanced(red, args)
	if not args or not args.adminpass then
		MSG.ERROR( "argumentError" )
	else
		local proxy = CO.GetProxy( "/", "systemctrl" )
		if proxy then
			local r,sn = proxy.GetSerialNo()
			if not r or not sn or tonumber(sn) == 0 then
				local reqSn = args.adminpass:match("99[0-9][0-9](.*)5$")
				if reqSn then
					cached_sn = nil
					local done
					r, done = proxy.CreateSerialNo(reqSn)
					proxy:Destroy()
					if r and done then
						MSG.ERROR( "createSerialNoSucc" )
					else
						MSG.ERROR( "createSerialNoError" )
					end
					return true
				end
			end
			proxy:Destroy()

			--Next, verify password.
			--TODO:
			if args.adminpass == "QsAbZg180706" then
				MSG.OK()
			else
				MSG.ERROR( "checkError" )
			end
		else
			MSG.ERROR( "systemError" )
		end
	end
	return true
end

local function API_newsn(red, args)
	if not args or not args.serial_no then
		MSG.ERROR( "argumentError" )
	else
		local proxy = CO.GetProxy( "/", "systemctrl" )
		if proxy then
			cached_sn = nil
			local r, done = proxy.CreateSerialNo(args.serial_no)
			proxy:Destroy()
			if r and done then
				MSG.OK( "" )
			else
				MSG.ERROR( "createSerialNoError" )
			end
		else
			MSG.ERROR( "systemError" )
		end
	end
	return true
end

local function API_ipList(red, args)
	MSG.OK( "", {
		data = {
			"192.168.0.1",
			"192.168.0.254",
			"192.168.1.1",
			"192.168.1.254",
			"192.168.2.1",
			"192.168.2.254",
			"www.baidu.com"
		}
	})
	return true
end

local function API_ping(red, args)
	if not args or not args.ip then
		MSG.ERROR( "argumentError" )
	else
		local proxy = CO.GetProxy( "/", "systemctrl", nil, 15 )
		if proxy then
			local r, done, arrival, lossRate = proxy.Ping(args.ip)
			proxy:Destroy()
			if r and done then
				if arrival then
					MSG.OK("", {data = {
						lossRate = lossRate
					}})
				else
					MSG.ERROR("IPUnreachable")
				end
			else
				MSG.ERROR( "requestError" )
			end
		else
			MSG.ERROR( "systemError" )
		end
	end
	return true
end

local function API_iperf3(red, args)
	if not args or not args.ip then
		MSG.ERROR( "argumentError" )
	else
		local proxy = CO.GetProxy( "/", "systemctrl", nil, 15 )
		if proxy then
			local r, done, arrival, ret = proxy.iperf3(args.ip)
			proxy:Destroy()
			if r and done then
				if arrival then
					MSG.OK( ret )
				else
					MSG.ERROR( "iperf3Error" )
				end
			else
				MSG.ERROR( "requestError" )
			end
		else
			MSG.ERROR( "systemError" )
		end
	end
	return true
end

local function API_ledtest(red, args)
	if not args or not args.id or not args.color then
		MSG.ERROR( "argumentError" )
	else
		local proxy = CO.GetProxy( "/diskmanage", "codec")
		if proxy then
			local r = proxy.change_disk_status(tonumber(args.id),tonumber(args.color))
			proxy:Destroy()

			if r then
				MSG.OK( "ok" )
			else
				MSG.ERROR( "requestError" )
			end
		else
			MSG.ERROR( "systemError" )
		end
	end
	return true
end

local function API_superMode(red,args)
	if not args or not args.user or not args.password then
		MSG.ERROR( "argumentError" )
	elseif args.user == "root" and args.password == "Kilo25070View03653" then
		MSG.OK("",{user='root'})
	elseif args.user == "admin" then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else 
			local r,done,err = proxy.Auth("web", args.user, args.password)
			if not r or not done then
				MSG.ERROR( "requestError")
				-- MSG.ERROR( "登录失败！错误:" .. (err or "系统错误") )
			else
				MSG.OK("",{user='admin'})
			end
		end
		proxy:Destroy()
	else
		MSG.ERROR( "requestError" )
	end
	return true
end

local function API_setAuthEnable(_, args)
	local proxy = CO.GetProxy("/", "systemctrl")
	if not proxy then
		MSG.ERROR( "requestError" )
		return
	end
	local is_ok = proxy.SetHTTPAPIAuthorization(args.enable)
	if not is_ok then
		MSG.ERROR( "requestError" )
		return
	end
	MSG.OK("")
	return true
end

local function API_getAuthEnable(_, args)
	local proxy = CO.GetProxy("/", "systemctrl")
	if not proxy then
		MSG.ERROR( "requestError" )
		return
	end
	local is_ok, enable = proxy.GetHTTPAPIAuthorization(args.enable)
	if not is_ok then
		MSG.ERROR( "requestError" )
		return
	end
	MSG.OK("", {
		data = {enable = enable}
	})
	return true
end

dset.APIS = dset.APIS or {}

dset.APIS["server_info"] = API_server_info
dset.APIS["reset"] = API_reset
dset.APIS["reboot"] = API_reboot
dset.APIS["restore"] = API_restore
dset.APIS["check_master"] = API_check_master
dset.APIS["status"] = API_status
dset.APIS["advanced"] = API_advanced
dset.APIS["newsn"] = API_newsn
dset.APIS["ipList"] = API_ipList
dset.APIS["ping"] = API_ping
dset.APIS["iperf3"] = API_iperf3
dset.APIS["ledtest"] = API_ledtest
dset.APIS["superMode"] = API_superMode
dset.APIS["setAuthEnable"] = API_setAuthEnable
dset.APIS["getAuthEnable"] = API_getAuthEnable

return dset

