local CO = require "api_coservice"
local MSG = require "api_msg"
local cjson = require "cjson"

local dset = {}
--------------------------------------
local function API_get(red, args)
	--TODO: Note, no used.
	MSG.OK()
	return true
end

local function API_list(red, args)
	local proxy = CO.GetProxy( "/auth", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	else
		local r,users,err = proxy.GetUsers("web")
		proxy:Destroy()
		if not r or not users then
			-- MSG.ERROR( "获取用户列表失败！错误:" .. (err or "系统错误") )
			MSG.ERROR("getUsersError")
		else
			local rTable = {}
			local k,v
			for k,v in pairs(users) do
				table.insert( rTable, {
					id = k,
					alias = v.name or k,
					create_time = v.createTime or "1970-01-01 00:00:00",
					role = v.role
				})
			end
			table.sort( rTable, function(a,b)
				return a.create_time <= b.create_time
			end)

			local page = tonumber(args.page) or 1
			local limit = tonumber(args.limit) or 10

			local result = {}
			for i = (page - 1) * limit + 1, page * limit do
				table.insert(result, rTable[i])
			end


			MSG.OK("", {
				data = result, count = #rTable
			})
		end
	end
	return true
end

local checksum = "s2GA!s"
local function API_login(red, args)
	local proxy = CO.GetProxy( "/auth", "systemctrl" )
	if not proxy then
		MSG.ERROR( "systemError" )
	elseif args then
		local r,done,err = proxy.Auth("web", args.user, args.password)
		if not r or not done then
			-- MSG.ERROR( "登录失败！错误:" .. (err or "系统错误") )
			MSG.ERROR("loginError")
		else
			r, done, err = proxy.GetUserInfo( "web", args.user )
			local now = os.time()
			MSG.OK( "", {
				data = {
					token = ngx.md5( args.password ),
					time = now,
					tokenNew = ngx.md5( checksum .. now ),
					changed = done.changed,
					alias = done and done.name or args.user
				}
			})
		end
		proxy:Destroy()
	else
		proxy:Destroy()
		MSG.ERROR( "authCheckError" )
	end
	return true
end

local function API_session(red, args)
	local headers = ngx.req.get_headers()
	if headers and headers["platform"] == "this-is-made-by-kiloview-for-platform-login" then
		MSG.OK("",{isProxy = 1})
		-- MSG.OK()
		return true
	end

	local cookie_user = ngx.var["cookie_user"]
	local cookie_token = ngx.var["cookie_token"]
	if not cookie_user or not cookie_token or cookie_user == "" or cookie_token == "" then
		MSG.ERROR( "验证失败:1" )
	else
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "无法连接系统服务！" )
		else
			local r, done, err = proxy.GetUserInfo( "web", cookie_user )
			if not r or not done then
				MSG.ERROR( "验证失败:2" )
			else
				if ngx.md5( done.password ) == cookie_token then
					local checked = require "api_roles".getUserRole(_, done.role)
					MSG.OK("", {data = checked})
				else
					MSG.ERROR( "验证失败:3" )
				end
			end
			proxy:Destroy()
		end
	end
	return true
end
--------------------------------------

local function API_modify(red, args)
	if args and (args.id or args.username) then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r,done,err = proxy.ChangeUser( "web", args.id or args.username, {
				password = args.password,
				name = args.alias,
				role = args.role
			})
			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "修改失败！错误：" .. (err or "系统错误") )
				MSG.ERROR("modifyUserError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_add(red, args)
	if args and args.id then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r,done,err = proxy.CreateUser( "web", args.id, {
				password = args.password,
				name = args.alias,
				role = args.role,
			})
			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "创建用户失败！错误：" .. (err or "系统错误") )
				MSG.ERROR("createUserError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end

local function API_remove(red, args)
	if args and args.ids then
		local proxy = CO.GetProxy( "/auth", "systemctrl" )
		if not proxy then
			MSG.ERROR( "systemError" )
		else
			local r,done,err
			if type(args.ids) == "string" then
				r,done,err = proxy.RemoveUser("web", args.ids )
			elseif type(args.ids) == "table" then
				local i,v
				for i,v in pairs(args.ids) do
					r,done,err = proxy.RemoveUser("web", v )
				end
			else
				r = false
				done = false
				err = "无效的用户ID"
			end

			proxy:Destroy()
			if r and done then
				MSG.OK()
			else
				-- MSG.ERROR( "删除用户失败！错误：" .. (err or "系统错误") )
				MSG.ERROR("delUserError")
			end
		end
	else
		MSG.ERROR( "argumentError" )
	end
	return true
end
--------------------------------------
dset.APIS = dset.APIS or {}

dset.APIS["get"] = API_get
dset.APIS["modify"] = API_modify
dset.APIS["login"] = API_login
dset.APIS["session"] = API_session
dset.APIS["list"] = API_list
dset.APIS["add"] = API_add
dset.APIS["remove"] = API_remove

--------------------------------------
return dset

