#!/usr/bin/env lua
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local withLog = false
local _x
for _x=1, #arg do
        if arg[_x] == "--log" then
                withLog = true
                break
        end
end

if withLog then
        -- ** initialize the log file **
        local roll = require "coaf.log.rollfile"
        os.execute( "/bin/mkdir -p /var/log/" )
        local logF = roll.Create("/var/log/CoManager.log")
        if not logF or not logF:init() then
                _dbg.Error("CoManager", "Fail to open log file: /var/log/CoManager.log, so the log messages should be output to console.")
                withLog = false
        else
                _dbg.AddLogger("logfile", logF, nil, nil)
        end
        -- ** log **
end

coaf.Init( "CoManager" )
if withLog then
    local stdio = require "coaf.log.stdio"
    stdio.SetDefaultFilterName( "CoManager" )
    stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

_G.cfg_CoDevice = require("configs.CoManager_config").Load()

--todo


local mod_CoManager = require "mod_CoManager"
local mod_LocalStreamer = require "mod_LocalStreamer"
mod_CoManager.init()
local mod_SysInfo = require "mod_SysInfo"
local cosvc_CoManager
------------------
-- **Main entry**
local function Main(args)
	cosvc_CoManager = service.New( "CoManager", "unix:/var/run/CoManager.sock" )
	if cosvc_CoManager then
		_dbg.Info( "CoManager", "CoService of CoManager is created." )
		cosvc_CoManager:CreateCoServerProxy()
		cosvc_CoManager:ExposeObject("/", mod_CoManager )
		cosvc_CoManager:ExposeObject("/local", mod_LocalStreamer )
		cosvc_CoManager:ExposeObject("/sysinfo", mod_SysInfo )
	else
		_dbg.Error( "CoManager", "Fail created CoService!" )
	end
	mod_LocalStreamer.StartLocalModulesCheck()
end

coaf.Start(Main, arg)

_dbg.Info("CoManager", "CoManager is destroying." )
_dbg.Print( "CoManager", "Service terminated." )
    





