local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local cjson = require "cjson"
local pl = require "pl.pretty"
local client = require "coaf.client"

local _M = {
    devices={},
    id_seed= 1
    
}

local function delayedStartDiscovery()
    local proxy = _M.GetProxy( "/discovery", "_CO_DAEMON.coserver" ) 
    if proxy then
        _dbg.Info("DEVICE-MANAGER", "The device name is:", cfg_CoDevice.link_name)
	--     --开启广播
	
		--#Bug fix: if not set correct link_token, other devices can not connect manual target to this.
		proxy.SetLinkToken(cfg_CoDevice.link_token)

        local r2,done2,err2 = proxy.PublishGlobal({name=cfg_CoDevice.link_name,group="edoctor.terminal"})
        if not r2 or not done2 then
            _dbg.Error("DEVICE-MANAGER", "mDNS publish device failed:", done2, err2)
        end

    --     --开启discovery
        local r3,err3 = proxy.StartDiscovery()
        if not r3 then
            _dbg.Error("DEVICE-MANAGER", "Start mDNS discovery failed:", err3)
        end

        --初始化，将手动的数据告诉coaf
        for k,v in pairs(cfg_CoDevice.devices) do
            local r,done,err = proxy.AddManualTarget(tostring(v.id), v.address, v.port, v.link_token)
            if err and err ~= "exist" then
            	_dbg.Error("DEVICE-MANAGER", "Add manual device targets failed:", done, err)
            end
        end
        proxy:Destroy()

    else
        _dbg.Error("DEVICE-MANAGER", "Create proxy link to coserver:/discovery failed.")
    end
end

function _M.init()
    _dbg.Info("DEVICE-MANAGER","Init")
    for k,v in pairs(cfg_CoDevice.devices) do
        if tonumber(_M.id_seed) <= tonumber(v.id) then
            _M.id_seed = tonumber(v.id)
        end
    end
	
	coaf.ScheduleDelayedTask(delayedStartDiscovery, 2) --Delay 2 seconds to wait avahi-daemon ready.

    --cfg_CoDevice:__TRIGGER(true)
end


function _M.GetProxy(objPath, service, loc, timeout, address)
    local proxy
    local cli
    cli = client.New( service, loc, address, timeout or 3 ) --3 seconds timeout.
    if not cli then
        _dbg.Error( "DEVICE-MANAGER", "Fail to create connection to service '" .. (loc or "local") .. "/" .. (service or "nil") )
        return nil
    end
    proxy = cli:CreateProxy(objPath, true) --Exclusive
    if not proxy then
        _dbg.Error( "DEVICE-MANAGER", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
        return nil
    end
    return proxy
end


--手动添加设备时，需要存到数据库中去
function _M.Add(details,id)
    _dbg.Info("DEVICE-MANAGER","Add manaual target:", id)
    details.manual = true
    details.status = "sleep"
    if id then
        details.id = id
    else
        _M.id_seed =_M.id_seed +1
        details.id = tostring(_M.id_seed)
    end
    details.name = details.name or "翼多示教设备"    

    --需要校验下，目前自动发现的数据 中是否存在 details的数据
    for uId,devIp in pairs(_M.devices) do
        if (tonumber(details.id) ~= 1) and (devIp == details.address) then
            _dbg.Error("DEVICE-MANAGER", "Device is already exist in list")
            return false,"该设备已经在发现列表中"
        end
    end
    
    local proxy = _M.GetProxy( "/discovery", "_CO_DAEMON.coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
    if proxy then
        local r,done,err = proxy.AddManualTarget(tostring(details.id), details.address, details.port, details.link_token)
        if not r or not done then
            _dbg.Error("DEVICE-MANAGER", "Add manual target for", id, "error:", done, err)
            return false, "Add manual target failed"
        end
    else
        _dbg.Error("DEVICE-MANAGER", "Create proxy link to coserver:/discovery failed.")
    end
    proxy:Destroy()
    cfg_CoDevice['devices'][details.id] = details
    local r, r2 = cfg_CoDevice:__TRIGGER(true)
    return true
end


--修改 手动添加的哪些设备/平台信息
function _M.Modify(id,details)
    _M.Remove(id)
    details.id = id
    _M.Add(details,id)  
    
    return true
end


--查询本台设备可以连接的所有设备/平台信息
--注意 自动可扫描的设备 增加auto=1字段返回
function _M.GetAllInfos()
    local datas={}
    local linkname,sn = _M.link_sysinfo()
    local k,v
    local proxy = _M.GetProxy( "/discovery", "_CO_DAEMON.coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
    if proxy then
        local r,data,err =  proxy.GetDiscoveryList()
        proxy:Destroy()
        if r and data then
            --名字的话 应该在data.options 里面, 不管设备是否在线，coserver会将我手动添加的信息及自动发现的信息 整合之后告诉我
            for k,tabs in pairs(data) do
                --在线的：
                _M.devices[tabs.id] = tabs.address 
                if tostring(tabs.id) == "1" then
                    tabs.name = "平台"
                    tabs.sn = sn
                    tabs.web_port = cfg_CoDevice.devices[tostring(tabs.id)].web_port
                    tabs.username = cfg_CoDevice.devices[tostring(tabs.id)].username
                    tabs.password = cfg_CoDevice.devices[tostring(tabs.id)].password
                    tabs.department = cfg_CoDevice.devices[tostring(tabs.id)].department

					tabs.link_token = cfg_CoDevice.devices[tostring(tabs.id)].link_token
					tabs.type = "SERVER"
                    table.insert(datas,1,tabs)
                else
                    if tabs.status == "ready" or tabs.status == "online" then 
                        if tabs.manual then
                            tabs.link_token = cfg_CoDevice.devices[tostring(tabs.id)].link_token
                        else
                            tabs.link_token = tabs.options.link_token
                        end
						tabs.name = tabs.options.name
						tabs.type = "TERMINAL"
						if tabs.options.group ~="edoctor.local-module" and tabs.options.group == "edoctor.terminal" then
							table.insert(datas,tabs)
                        end
					else
						--不在线的设备options为空，无法判断group 默认让显示
						if tabs.manual and cfg_CoDevice.devices[tostring(tabs.id)]  then
                            tabs.link_token=cfg_CoDevice.devices[tostring(tabs.id)].link_token
                            tabs.name=cfg_CoDevice.devices[tostring(tabs.id)].name
                            table.insert(datas,tabs)
                        end
                    end
                end
            end
        end

    else
        _dbg.Error("DEVICE-MANAGER", "Create proxy link to coserver:/discovery failed.")
        return false, "system ERROR"
    end
    ---平台信息
    if #datas == 0 then
        table.insert(datas,1,cfg_CoDevice.devices["1"])
    end

    return datas
end


--删除 手动添加的哪些设备/平台信息
function _M.Remove(id)
    for k,v in pairs(cfg_CoDevice.devices) do
        if tostring(k) == tostring(id) then

            local proxy = _M.GetProxy( "/discovery", "_CO_DAEMON.coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
            if proxy then
                local r,done,err = proxy.RemoveManualTarget(tostring(id))
                if not r or not done then
                    _dbg.Error("DEVICE-MANAGER", "Remove manual target of", id, "error:", done, err)
                    return false,"Remove manual target failed"
                end
            else
		        _dbg.Error("DEVICE-MANAGER", "Create proxy link to coserver:/discovery failed.")
                return false,"system ERROR"
            end
            cfg_CoDevice.devices[k] = nil
            _M.devices[k] = nil
        end
    end
    local r, r2 = cfg_CoDevice:__TRIGGER(true)
    return true
end

--查询外部设备上的流服务信息
function _M.GetStreamerInfo(id)

    local data_all = {}
    local data ={
        id=id,name="discover.AllSource"
    }
    local result = {}

    local ip_img = _M.devices[id]
    local ip = id ..".coaf"

    local LOCAL_URL = {
        ["LOCAL-00"] ="rtsp://".. ip..":554/LOCAL-00.coaf",
        ['LOCAL-01'] ="rtsp://".. ip..":554/LOCAL-01.coaf",
        ['LOCAL-02'] ="rtsp://".. ip..":554/LOCAL-02.coaf",
        ['LOCAL-03'] ="rtsp://".. ip..":554/LOCAL-03.coaf"
    }
    local LOCAL_SUB_URL = {
        ["LOCAL-00"] ="rtsp://".. ip..":554/LOCAL-00sub.coaf",
        ['LOCAL-01'] ="rtsp://".. ip..":554/LOCAL-01sub.coaf",
        ['LOCAL-02'] ="rtsp://".. ip..":554/LOCAL-02sub.coaf",
        ['LOCAL-03'] ="rtsp://".. ip..":554/LOCAL-03sub.coaf"
    }
   
    --应该需要调用 指定设备上面的服务
    local proxy = _M.GetProxy("/streams", "codec", id)
    if not proxy then
		    _dbg.Error("DEVICE-MANAGER", "Create proxy link to codec:/streams failed.")
            return false
    else
        local r, streams,err = proxy.GetStreams("local")
        proxy:Destroy()

        if not r or not streams then
            _dbg.Error("DEVICE-MANAGER", "Get local streams information error：", streams, err)
            return false
        end

        local i, item
        for i=1, #streams do
            item = {
                id = streams[i].id,
                name = streams[i].name,
                type = streams[i].type,
                src = streams[i].address,
                status = streams[i].type == "local" and "on" or "off",
                resolution = "分辨率未知",
                manual_url = streams[i].manual_url or "",
                buffer = streams[i].buffer_option or "live:200:200:500:0:0",
                bind_net = streams[i].options and streams[i].options.bind or "",
                trans_mode = streams[i].options and streams[i].options.transport or "tcp",
                video_port = streams[i].options and streams[i].options.rtp_video_port or 0,
                audio_port = streams[i].options and streams[i].options.rtp_audio_port or 0,
                video_codec = streams[i].options and streams[i].options.rtp_video_codec or 0,
                audio_codec = streams[i].options and streams[i].options.rtp_audio_codec or 0,
                bitrate = 0,
                PlatIP = ip_img,
                imgUrl = streams[i].imgUrl,
                platId = id,
                tree_id = id .. "_" .. streams[i].id,
                islocal = false,
            }
            item.manual_url = LOCAL_URL[streams[i].id]
            item.sub_url = LOCAL_SUB_URL[streams[i].id]

            if streams[i].stream_info then
                item.resolution = streams[i].stream_info.resolution or "分辨率未知"
                item.bitrate = streams[i].stream_info.bitrate or 0
            end

            table.insert( result, item )
        end
    end
    
    if #result == 0 then
        data.children = nil
    else
        data.children = result
    end
    table.insert(data_all,data)
    return data_all
end


function _M.link_token() 
    return cfg_CoDevice.link_token
end


function _M.Modify_link_token(link_token) 
    cfg_CoDevice.link_token = link_token
    local proxy = _M.GetProxy( "/discovery", "_CO_DAEMON.coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
    if proxy then
        local r2 = proxy.SetLinkToken(link_token)
        if not r2 then
            _dbg.Error("DEVICE-MANAGER", "Modify link token to:", link_token, "failed!")
            return false
        end
    else
        _dbg.Error("DEVICE-MANAGER", "Create proxy link to coserver:/discovery failed.")
        return false, "system ERROR"
    end
    cfg_CoDevice:__TRIGGER(true)
    return true
end


function _M.link_sysinfo() 
    local sn                       
    local f = io.open("/etc/SERIAL_NO", "r" )
    if not f then               
        sn= "000000000"                    
    else                                             
        local l = f:read("*l")                       
        f:close()                                    
        sn= l or "000000000"               
    end  
    return cfg_CoDevice.link_name,sn
end


function _M.Modify_link_name(link_name) 
    cfg_CoDevice.link_name = link_name
    local proxy = _M.GetProxy( "/discovery", "_CO_DAEMON.coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
    if proxy then
        local r2,done2,err2 = proxy.PublishGlobal({link_token=cfg_CoDevice.link_token,name=cfg_CoDevice.link_name})
        if not r2 or not done2 then
            _dbg.Error("DEVICE-MANAGER", "Modify device name to:", link_name, "error:", done2, err2)
            -- return false
        end
    else
        _dbg.Error("DEVICE-MANAGER", "Create proxy link to coserver:/discovery failed.")
        -- return false,"discovery proxy ERROR!!!"
    end
    cfg_CoDevice:__TRIGGER(true)
    return true
end


--物理显示屏需要展示的信息
function _M.ScreenInfo()
    _dbg.Info("DEVICE-MANAGER","Get screen info")

    for k,v in pairs(cfg_CoDevice.devices) do
        if tostring(k) == "1" then
            local proxy = _M.GetProxy( "/discovery", "_CO_DAEMON.coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
            if proxy then
                local r,data,err =  proxy.GetDiscoveryList()
                proxy:Destroy()
                if r and data then
                    for k,tabs in pairs(data) do
                        if tostring(tabs.id) == "1" then
                            v.status = tabs.status
                        end
                    end
                end
            end

            return v
        end
    end
    return true
end


function _M.GetPlatStreamerInfo(platId,username)
    local proxy = _M.GetProxy( "/link", "Yiduo", "1")
    if proxy then
        local r,info ,err = proxy.GetStreamsList(username)
        return info
    else
        _dbg.Error("DEVICE-MANAGER", "Fail to connect to platform!")
    end
    return {}
end

return _M
