local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

local cjson = require "cjson"
local pl = require "pl.pretty"
local client = require "coaf.client"

local hiperiphery = require "hiperiphery"
local HIMM = hiperiphery.HIMM

local posix_time = require "posix.time"

local _M = {
    local_modules_check_task = nil
}

--查询本机本地编码流的信息
function _M.GetLocalEnInfo()
    local datas = {}
    local mod_CoManager = require "mod_CoManager"
    local proxy = mod_CoManager.GetProxy( "/discovery", "coserver", "local", 3, "unix:/var/run/coaf.coserver.ctrl" )
    if proxy then
        local r,data,err =  proxy.GetDiscoveryList()
        proxy:Destroy()
        if r and data then
            for k,tabs in pairs(data) do
                --在线的：
                if tabs.status == "ready" or tabs.status == "online" then
                    --判断流是否为本地encoder
                    if tabs.options.group =="edoctor.local-module" then  
                        table.insert(datas,tabs)
                    end
                end
            end
        end
    else
	    _dbg.Error("DEVICE-MANAGER", "Create proxy link to codec:/streams failed.")
        return false,"system ERROR"
    end
    return datas
end

local function beep()
	HIMM.write( 0x12200040,0x00 ) --Low, beep.
	coaf.ScheduleDelayedTask(function()
		HIMM.write( 0x12200040,0x10 ) --High, stop beep.
	end, 0.5)
end

local function local_online_check(throbj)
	local done, hardware_ctrl = pcall( require, "system.hardwareVersion" )
	if not done or type(hardware_ctrl) ~= "table" then
		hardware_ctrl = nil
	end

	-- GPIO11-4: Output mode.
    HIMM.write( 0x120f018c, 0x1 )
    HIMM.write( 0x12200400, 0x10 )

	local objects = {
		{ id = "LOCAL-00", reset_reg = 0xab, online = false, last_seen = false, reset_count = 0 },
		{ id = "LOCAL-01", reset_reg = 0xa8, online = false, last_seen = false, reset_count = 0 },
		{ id = "LOCAL-02", reset_reg = 0xa9, online = false, last_seen = false, reset_count = 0 },
		{ id = "LOCAL-03", reset_reg = 0xaa, online = false, last_seen = false, reset_count = 0 }
	}
	
	for i=1, #objects do
		objects[i].last_online_time = posix_time.clock_gettime( posix_time.CLOCK_MONOTONIC )
	end

	_dbg.Info("DEVICE-MANAGER", "Local encoder modules online checking task is running...")
	while not throbj.terminated do
		local items = _M.GetLocalEnInfo()
		local now = posix_time.clock_gettime( posix_time.CLOCK_MONOTONIC )

		for i=1, #objects do
			local fnd = false
			if items then
				for j=1, #items do
					if items[j].id == objects[i].id then
						objects[i].last_seen = true
						objects[i].online = true
						objects[i].last_online_time = now
						objects[i].reset_count = 0
						fnd = true
						break
					end
				end
			end
			
			if not fnd then
				objects[i].online = false

				local delta = (now.tv_sec - objects[i].last_online_time.tv_sec)*1000
				delta = delta + (now.tv_nsec - objects[i].last_online_time.tv_nsec)/1000000

				if delta >= 90000 then --90 seconds (1.5 minutes)
					if objects[i].reset_count < 5 then
						_dbg.Warn("DEVICE-MANAGER", "The local encoder of", objects[i].id, "looks to be not online! Try reset it!")
					elseif objects[i].reset_count == 5 then
						_dbg.Warn("DEVICE-MANAGER", "The local encoder of", objects[i].id, "looks to be not online! Try reset it! (*** last time show this log, reset continuously but no log until it becomes online again)")
					end
					
					if hardware_ctrl then
						local cc = 0
						local last_done
						while cc < 5 do
							if hardware_ctrl.WriteReg( objects[i].reset_reg, {0x00} ) then
								last_done = false
								coaf.Sleep(0.1)
								if hardware_ctrl.WriteReg( objects[i].reset_reg, {0x01} ) then
									if objects[i].last_seen then
										beep()
									end

									last_done = true
									break
								end
							end
							cc = cc + 1
							coaf.Sleep(0.1)
						end
						if not last_done then
							--Try again, since sometimes hardware may conflict.
							hardware_ctrl.WriteReg( objects[i].reset_reg, {0x01} )
						end
					end

					objects[i].reset_count = objects[i].reset_count + 1
					objects[i].last_online_time = {
						tv_sec = now.tv_sec + 60, -- Considering the encoder module reboot requires about 1 minute.
						tv_nsec = now.tv_nsec
					}
				end
			end
		end
		coaf.Sleep(3)
	end
	_dbg.Info("DEVICE-MANAGER", "Local encoder modules online checking task is terminated.")
end

--查询编码码率，分辨率
-- function _M.GetStreamStatus(id )
--     local idr= "00"
--     local data = {}
--     local r
--     local mod_CoManager = require "mod_CoManager"
--     local proxy = mod_CoManager.GetProxy( "/encode", "codec",id )
--     if  proxy then
--         r,data,err = proxy.GetEncodeInfo()
--         proxy:Destroy()
--         if not r or not data then
--             return {}
--         end
--     else
--         return {}
--     end
-- 	return data
-- end

function _M.StartLocalModulesCheck()
	if not _M.local_modules_check_task then
		_M.local_modules_check_task = thread.New( local_online_check )
		if _M.local_modules_check_task then
			_M.local_modules_check_task:Start()
		end
	end
end

return _M
