local DC = require "supertable.declare"
local CONF = require "supertable.core"
--add by kiloview@2020-9-28
local _done, features = pcall(require, 'features')
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end

local _M = {
	config_file = "/data/configs/encoders.conf"
}

--------------------------------------------
--注：
-- encoders配置为一个集合(set)，格式为：
-- {
--   ["LOCAL-00"] = {
--   	video={...},
--   	audio={...}
--   },
--   ["LOCAL-01"] = {...},
--   ...
--   ["OUTPUT1"] = {...},
--   ["OUTPUT1:sub"] = {...}
--   ["OUTPUT2"] = {...}
--   ["OUTPUT2:sub"] = {...}
-- }
--
-- 其中 LOCAL-00/01/02/03 的视频编码配置参数
-- 没有实际意义，因为它们直接存储在Amba模块
-- 上，只有音频是有意义的，用于选择音频源。
--------------------------------------------
local encoders_template = DC.SET(
	DC.OBJ{
		video = DC.OBJ({
			scale = DC.STR("default") : __nosave(true) :
			__get( function(cfg,key,refVal)
				if cfg.scale_width <= 0 or cfg.scale_height <= 0 then
					return "default"
				else
					return cfg.scale_width .. "x" .. cfg.scale_height
				end
			end):

			__set( function(cfg,key,val,oldRef)
				if val:lower() == "default" then
					cfg.scale_width = 0
					cfg.scale_height = 0
				else
					local w,h = val:match("(%d+)[xX](%d+)")
					if w and h then
						cfg.scale_width = tonumber(w)
						cfg.scale_height = tonumber(h)
					end
				end
				return false
			end),

			scale_width = DC.INT(0),
			scale_height = DC.INT(0),
			codec = DC.STR("H264"),
			br_ctrl = DC.STR("cbr"),
			bitrate = DC.INT(4000000),
			profile = DC.STR("high"),
			avclevel = DC.INT(40),
			gop_size = DC.INT(60),
			gop = DC.STR("IP"),

			framerate_mode = DC.STR("half") : __nosave(true) :
			__get( function(cfg,key,refVal)
				if cfg.framerate <= 0 then
					return "default"
				elseif cfg.framerate == 0.5 then
					return "half"
				else
					return "custom"
				end
			end) :
			__set( function(cfg,key,val,oldRef)
				if val == "default" then
					cfg.framerate = 0
				elseif val == "half" then
					cfg.framerate = 0.5
				end
				return false
			end),

			framerate = DC.NUM(0.5),
			quality = DC.INT(80),
			ref_frames = DC.INT(1),
			chrome = DC.INT(0)
		}),

		audio = DC.OBJ({
			source = DC.STR("none")
		}) --Nothing now.
	}
) : __changed( function(conf)
	conf:TAB_SAVE()
end)

--------------------------------------------
local fail_default = {}
if features.FEATURE_LOCAL_STREAM then
	fail_default = {
		["LOCAL-00"] = {},
		["LOCAL-00:sub"] = {},
		["LOCAL-01"] = {},
		["LOCAL-01:sub"] = {},
		["LOCAL-02"] = {},
		["LOCAL-02:sub"] = {},
		["LOCAL-03"] = {},
		["LOCAL-03:sub"] = {},
	
		["OUTPUT1"] = {},
		--子码流
		["OUTPUT1:sub"] = { video = {
			scale_width = 720,
			scale_height = 480,
			bitrate = 2000000
		}},
		["OUTPUT2"] = {},
		--子码流
		["OUTPUT2:sub"] = { video = {
			scale_width = 720,
			scale_height = 480, --405->480
			bitrate = 2000000
		}}
	}
else
	fail_default = {	
		["OUTPUT1"] = {},
		--子码流
		["OUTPUT1:sub"] = { video = {
			scale_width = 720,
			scale_height = 480,
			bitrate = 2000000
		}},
		["OUTPUT2"] = {},
		--子码流
		["OUTPUT2:sub"] = { video = {
			scale_width = 720,
			scale_height = 480, --405->480
			bitrate = 2000000
		}}
	}
end


function _M.Load()
	local conf = CONF.Create({}, encoders_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure
		conf:TAB_BIND(fail_default or {})
	end
	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------

return _M

