local DC = require "supertable.declare"
local CONF = require "supertable.core"
local sys_video_config = require "sys_video_config"

local _M = {
	config_path = "/data/configs"
}

--------------------------------------------

local output_template = DC.OBJ({
	layout = DC.STR("1"),
	resolution = DC.STR("1080P30"),
	cvbs_resolution = DC.STR("PAL"),
	colorspace = DC.STR("AUTO"),
	hdmi_mode = DC.STR("HDMI"),
	hdmi_audio_sampling = DC.INT(48000),

	bind_interfaces = DC.ARR( DC.STR("") ),
	bind_audio_outputs = DC.ARR( DC.STR("") ),
	show_layouts = DC.ARR( DC.STR("1") ),

	embedded_audio = DC.OBJ({
		volume = DC.INT(0),
		disabled = DC.INT(0),
		local_enabled = DC.INT(0)
	}),

	analog_audio = DC.OBJ({
		volume = DC.INT(0),
		disabled = DC.INT(1),
		local_enabled = DC.INT(0)
	}),

	channels = DC.SET( DC.OBJ{
		stream = DC.STR("")
	}),

	border = DC.OBJ({
		bordercolor = DC.INT(0xffffff),
		enabled = DC.INT(0)
	})
}) : __changed( function(conf)
	conf:TAB_SAVE()
end)

--------------------------------------------
local fail_defaults = {
	["sd"] = {
		layout = "1",
		resolution = "PAL",
		cvbs_resolution = "PAL",
		colorspace = "AUTO",
		hdmi_mode = "HDMI",
		bind_interfaces = sys_video_config.outputs.sd0_default_bind.video,
		bind_audio_outputs = sys_video_config.outputs.sd0_default_bind.audio,
		embedded_audio = { volume=0, disabled=1 },
		analog_audio = { volume=0, disabled=0 },
		border = { bordercolor = 0xffffff, enabled = 0}
	},

	["hd0"] = {
		layout = "1",
		resolution = "1080P30",
		cvbs_resolution = "PAL",
		colorspace = "AUTO",
		hdmi_mode = "HDMI",
		bind_interfaces = sys_video_config.outputs.hd0_default_bind.video,
		bind_audio_outputs = sys_video_config.outputs.hd0_default_bind.audio,
		embedded_audio = { volume=0, disabled=0 },
		analog_audio = { volume=0, disabled=0 },
		border = { bordercolor = 0xffffff, enabled = 0}
	},

	["hd1"] = {
		layout = "1",
		resolution = "1080P30",
		cvbs_resolution = "PAL",
		colorspace = "AUTO",
		hdmi_mode = "HDMI",
		bind_interfaces = sys_video_config.outputs.hd1_default_bind.video,
		bind_audio_outputs = sys_video_config.outputs.hd1_default_bind.audio,
		embedded_audio = { volume=0, disabled=0 },
		analog_audio = { volume=0, disabled=1 },
		border = { bordercolor = 0xffffff, enabled = 0}
	},

	["preview"] = {
		layout = "1",
		resolution = "PAL",
		cvbs_resolution = "PAL",
		colorspace = "AUTO",
		hdmi_mode = "",
		embedded_audio = { volume=0, disabled=0 },
		analog_audio = { volume=0, disabled=1 },
		border = { bordercolor = 0xffffff, enabled = 0}
	}
}

function _M.Load(objname)
	local conf = CONF.Create({}, output_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_path .. "/" .. (objname or "output") .. ".conf" } )

	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure
		conf:TAB_BIND(fail_defaults[objname or "hd0"] or {})
		conf:TAB_SAVE()
	end
	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------

return _M


