--[[
    预览模块
--]]
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local output = require "soc.output"
local layout = require "mod_Layout"
local streams = require "mod_Streams"
local cfg_output = require "configs.output_config"
local CO = require "mod_coservice"

local _done, features = pcall(require, "features")
if not _done then
	features = {}
elseif type(features) ~= "table" then
	features = _G
end

--[[ 最大支持32个画面的预览 ]]
local MAX_SUPPORT_WINDOWS = 32

--[[
        预览页面默认的分屏
]]
local layout_map = {
    ["1"] = {
        number = "1",
        position = {
            {
                width = 100,
                height = 100,
                y = 0,
                x = 0
            }
        },
        id = 1,
        enable = 0,
        img_id = 0,
        name = "1"
    },
    ["2"] = {
        number = 2,
        position = {
            {
                height = 50,
                x = 0,
                width = 50,
                y = 25
            },
            {
                height = 50,
                x = 50,
                width = 50,
                y = 25
            }
        },
        img_id = 0,
        id = 2,
        name = "2"
    },
    ["3"] = {
        number = 4,
        position = {
            {
                height = 50,
                x = 0,
                width = 50,
                y = 0
            },
            {
                height = 50,
                x = 50,
                width = 50,
                y = 0
            },
            {
                height = 50,
                x = 0,
                width = 50,
                y = 50
            },
            {
                height = 50,
                x = 50,
                width = 50,
                y = 50
            }
        },
        img_id = 0,
        id = 3,
        name = "4"
    },
    ["4"] = {
        number = 9,
        position = {
            {
                height = 33.33,
                x = 0,
                width = 33.33,
                y = 0
            },
            {
                height = 33.33,
                x = 33.33,
                width = 33.33,
                y = 0
            },
            {
                height = 33.33,
                x = 66.66,
                width = 33.33,
                y = 0
            },
            {
                height = 33.33,
                x = 0,
                width = 33.33,
                y = 33.33
            },
            {
                height = 33.33,
                x = 33.33,
                width = 33.33,
                y = 33.33
            },
            {
                height = 33.33,
                x = 66.66,
                width = 33.33,
                y = 33.33
            },
            {
                height = 33.33,
                x = 0,
                width = 33.33,
                y = 66.66
            },
            {
                height = 33.33,
                x = 33.33,
                width = 33.33,
                y = 66.66
            },
            {
                height = 33.33,
                x = 66.66,
                width = 33.33,
                y = 66.66
            }
        },
        img_id = 0,
        id = 4,
        name = "9"
    },
    ["5"] = {
        number = 16,
        position = {
            {
                height = 25,
                x = 0,
                width = 25,
                y = 0
            },
            {
                height = 25,
                x = 25,
                width = 25,
                y = 0
            },
            {
                height = 25,
                x = 50,
                width = 25,
                y = 0
            },
            {
                height = 25,
                x = 75,
                width = 25,
                y = 0
            },
            {
                height = 25,
                x = 0,
                width = 25,
                y = 25
            },
            {
                height = 25,
                x = 25,
                width = 25,
                y = 25
            },
            {
                height = 25,
                x = 50,
                width = 25,
                y = 25
            },
            {
                height = 25,
                x = 75,
                width = 25,
                y = 25
            },
            {
                height = 25,
                x = 0,
                width = 25,
                y = 50
            },
            {
                height = 25,
                x = 25,
                width = 25,
                y = 50
            },
            {
                height = 25,
                x = 50,
                width = 25,
                y = 50
            },
            {
                height = 25,
                x = 75,
                width = 25,
                y = 50
            },
            {
                height = 25,
                x = 0,
                width = 25,
                y = 75
            },
            {
                height = 25,
                x = 25,
                width = 25,
                y = 75
            },
            {
                height = 25,
                x = 50,
                width = 25,
                y = 75
            },
            {
                height = 25,
                x = 75,
                width = 25,
                y = 75
            }
        },
        img_id = 0,
        id = 5,
        name = "16"
    }
}

local _M = {
    name = nil, --配置文件名字  preview
    config = nil, --配置文件信息，保存在 /data/configs/preview
    stopStream = {}
}


function _M.CreateOutput(output_name, output_device)
    _M.name   = output_name or "preview"
    _M.config = cfg_output.Load(_M.name) --从配置文件里面读取
    if not _M.config then
        _dbg.Error( "PREVIEW", "Fail to load output configure!" )
        _M.config = {
            layout = "1",
            resolution = "PAL",
            cvbs_resolution = "PAL",
            colorspace = "AUTO",
            hdmi_mode = "",
            channels = {},
            analog_audio = {},
            embedded_audio = {},
            __TRIGGER = function()
            end
        }
    end
    --local pretty = require "pl.pretty"                                                       
    --pretty.dump(_M.config.channels)
    _M.UpdateStreamBinding()
end

function _M.GetStatus(no_layout)
	local ret = {
		started = true,
		resolution = _M.config.resolution,
		resolution_name = "",
		layout = _M.config.layout, --当前使用的分屏配置
		layout_number =  #layout_map[_M.config.layout].position, --分屏数量
		bind_interfaces = "",
		hdmi_mode = _M.config.hdmi_mode,
		colorspace = _M.config.colorspace,
		sampling = 48000,
        border = "",
        channels = {}
    }
    local item
    -- 根据使用的分屏配置来初始化 channels
    for i=1, #layout_map[_M.config.layout].position do
        local streamObj
        if _M.config.channels[i] and _M.config.channels[i].stream then
                streamObj = streams.GetStreamObject( _M.config.channels[i].stream )
        end

        item = {
                id = i, --窗口号
                stream = _M.config.channels[i] and _M.config.channels[i].stream or "", -- 当前窗口绑定的stream id
                bind = streamObj and true or false -- 是否绑定stream
        }
        table.insert( ret.channels, item )
    end
	table.sort( ret.channels, function(a,b)
		return a.id < b.id
        end)
        
	return ret
end

function _M.GetSupportLayouts()
	return layout.EnumLayouts()
end

--默认开启主码流拉流 预览子码流
function _M.UpdateStreamBinding()
    for i,val in pairs(_M.config.channels) do
        if type(val) == "table" then
			local streamObj = streams.GetStreamObject( val.stream )
			if streamObj then
				_dbg.Info( "PREVIEW", "streamObj:Start main stream..", _M.name, " streamId ", val.stream)
				streamObj:Start(_M.name, "preview") --Enable stream's output
			end

			local subStreamObj = streams.GetStreamObject( val.stream .. ":sub")
            if subStreamObj then
                _dbg.Info( "PREVIEW", "streamObj:Start preview sub stream..", _M.name, " streamId: ", val.stream)
                subStreamObj:Start(_M.name, "preview") --Enable stream's output
            end
        end
    end
end

--[[
        绑定stream 到 预览窗口
]]
function _M.BindStream( wnd_id, stream_id )
    _dbg.Info( "PREVIEW", "BindStream wnd_id=",wnd_id,"stream_id=",stream_id)
    if wnd_id > 0 and wnd_id <= MAX_SUPPORT_WINDOWS then
        local streamObj
        if not stream_id or stream_id == "" then
			-- 当前窗口绑定了stream, 需要停止
			if _M.config.channels[wnd_id] then
                _M.stopStream[_M.config.channels[wnd_id].stream] = true
				_M.config.channels[wnd_id] = nil --Drop it
			end
        else
            -- 当前窗口未绑定stream
            if  not _M.config.channels[wnd_id] then
                _M.config.channels[wnd_id] = { stream = stream_id } 
            elseif _M.config.channels[wnd_id].stream ~= stream_id then
                --当前窗口绑定了数据流，需要先停止旧数据流，在启动新数据流
                _M.stopStream[_M.config.channels[wnd_id].stream] = true
                _M.config.channels[wnd_id].stream = stream_id
            end
            -- 启动数据流, 主子码流都开启
            streamObj = streams.GetStreamObject(  _M.config.channels[wnd_id].stream )
			if streamObj then
				_dbg.Info( "PREVIEW", "streamObj:Start main stream..", _M.name, " streamId ",  _M.config.channels[wnd_id].stream)
				streamObj:Start(_M.name, "preview") --Enable stream's output
			end

			local subStreamObj = streams.GetStreamObject(  _M.config.channels[wnd_id].stream .. ":sub")
            if subStreamObj then
                _dbg.Info( "PREVIEW", "streamObj:Start preview sub stream..", _M.name, " streamId: ",  _M.config.channels[wnd_id].stream)
                subStreamObj:Start(_M.name, "preview") --Enable stream's output
            end
        end
        --local pretty = require "pl.pretty"
        --pretty.dump(_Preview.config.channels)
        _M.config:__TRIGGER(true) --[[ 更新修改到配置文件 ]]
		return true
	else
        _dbg.Error( "PREVIEW", "Binding stream: windows ID:", wnd_id, "out of range!" )
		return false
	end
end

function _M.RemoveStreamByWindow( wnd_id )
	return _M.BindStream( wnd_id, nil )
end

function _M.RemoveStream( stream_id )
	local k,v
	for k,v in pairs(_M.config.channels) do
		if v.stream == stream_id then
			_M.config.channels[k] = nil
			_M.config:__TRIGGER(true)
		end
    end
    collectgarbage();
	return true
end

function _M.Update()
	local item, flag
	for item, flag in pairs(_M.stopStream) do
		local stop = true
        -- 停止码流的时候需要先查询配置，确定stream没有被使用才可以停止
        for i,val in pairs(_M.config.channels) do
            _dbg.Info( "Preview Stream-id:", item, val.stream)
            if val.stream == item then
                stop = false
                break
            end
        end
        if stop == true then
            local streamObj = streams.GetStreamObject( item )	
            if streamObj then
                _dbg.Info( "Preview Close the stream Stream-id:", item )
                streamObj:Stop(_M.name, "preview") --stop stream's output
            end
            local subStreamObj = streams.GetStreamObject( item .. ":sub" )
            if subStreamObj then
                subStreamObj:Stop(_M.name, "preview") --stop stream's output
            end
        end
	end
	_M.stopStream = {}
end
function _M.GetAllLayouts()
    return layout_map
end

function _M.GetShowLayouts()
    return { "1", "2", "3", "4", "5", "6", "7", "8", "9", }
end

function _M.ChangeShowLayouts(items)
	if type(items) == "table" then
		_M.config.show_layouts = items
		_M.config:__TRIGGER(true)
		return true
	else
		return false
	end
end

function _M.SetOutputLayout (layout_id)
    _M.config.layout = layout_id
    _M.config:__TRIGGER(true)
    return true
end

return _M

