local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
--local mod_snapshot = require "mod.snapshot"
local event = require "coaf.event"
local thread = require "coaf.thread"
local disk_manage = require "mod_diskManage"
local mod_channel = require "mod_Streams"

require "kmCore"


local _M = {
	meta_period = 2,
	snap_period = 2,
	recording = {},
	setting = nil,
	record_event = nil,
	record_event_name = nil,
	select_disk = 1,
	recordSysFile = { 
		{
			name = 'kvmedia',
			id = 1,
			children = {{
							name = 'record_01',
							id = 'record_01'
						},
						{
							name= 'record_02', --子文件夹2
							id = 'record_02'
						}}
		} 
		-- ,
		-- {
		-- 	name = 'data2',
		-- 	id = 2,
		-- 	children = {{
		-- 					name = '子文件夹1',
		-- 					id = 'record_01'
		-- 				},
		-- 				{
		-- 					name= '子文件夹2',
		-- 					id = 'record_02'
		-- 				}}
		-- }
	}
}


---------------------------------------------------
-- Init record module
---------------------------------------------------


local function restart_record()
	local need_restart = {}
	for k,_ in pairs(_M.recording) do
		if _.status and _.status.work == "started" then
			_M.StopChannel(k)
			table.insert(need_restart, k)
			--_M.StartChannel(k)
		end

	end

	local i
	for i=1, #need_restart do
		_M.StartChannel(need_restart[i])

	end
end

local function check_task(thrObj)
	while not thrObj.terminated do
		local should_restart = false
		local diskDetail = disk_manage.get_disk_info( )
		diskDetail_select = diskDetail[_M.select_disk]
		--print("**************specific_status  :  "..diskDetail_select.specific_status .. " disk_manage.disk_status[_M.select_disk].is_record     " .. disk_manage.disk_status[_M.select_disk].is_record)
		if diskDetail_select.specific_status == 5 and disk_manage.disk_status[_M.select_disk].is_record ~= 0 then --Less than 1GB
			if _M.setting.disk_policy == "overwrite" then
				_dbg.Warn( "STREAMER", "Disk #".._M.select_disk.. "is full!  drop oldest record file(s) because disk-policy is overwrite!" )
				if  true then
					--_dbg.Error( "STREAMER", "Disk #", id, "is full and no oldest file can be removed! So force stop recording channel:", channel )
					--record_item.status.errorInfo = "磁盘已满"
					--thrObj.terminated = true
					should_restart = true
				end
			else --Stop record.
				--_dbg.Error( "STREAMER", "Disk #", id, "is full! So force stop recording channel:", channel, " because disk-policy is stopping record!" )
				--record_item.status.errorInfo = "磁盘已满"
				--thrObj.terminated = true
				should_restart = true
				
			end
		end
            
		if should_restart then
			restart_record()
		end	
		

		if not thrObj.terminated then
			coaf.Sleep(1)	--1 second period.
		end
	end

	
end

function _M.Init(cfg, event, event_name)
	_M.setting = cfg
	_M.record_event = event
	_M.record_event_name = event_name

	_M.check_task = thread.New( check_task )
	if _M.check_task then
		_M.check_task:Start()
	end
end


local function saveRecordMetaFile( path, recordFile, record_info)
	if not record_info then return false end

	local filename = path .. "/.META/" .. recordFile .. ".META"
	
	local file = io.open( filename, "w" )
	if file then
		file:write([[
return {
forFile = "]] .. (record_info.file or "") .. [[",
startTime = "]] .. (record_info.startTime or "") .. [[",
stopTime = "]] .. (record_info.stopTime or "") .. [[",
totalDuration = ]] .. (record_info.totalDuration or 0) .. [[,
totalSize = ]] .. (record_info.totalSize or 0) .. [[
}
]])
		file:close()
		mod_channel.UpdateFileListTs(path, recordFile, record_info)
		return true
	else
		return false
	end
end

---------------------------------------------------



local function updateRecordStatus( record_item, channel)
	local info = record_item.recorder:GetSessionStatus( record_item.record_session )
	if info and record_item.status.record_info then
        record_item.status.record_info.file = info.file or ""
		record_item.status.record_info.state = info.state
        if info.state == "preparing" or info.state == "starting" or info.state == "recording" then
            record_item.status.record_info.stopTime = "RECORDING"
        else
            record_item.status.record_info.stopTime = os.date("%Y-%m-%d %H:%M:%S")
        end
        record_item.status.record_info.totalDuration = info.time or 0
        if not info.time or info.time == 0 then --NOTE: Change the start time (to now) if duration is 0
            record_item.status.record_info.startTime = os.date("%Y-%m-%d %H:%M:%S")
        end

        record_item.status.record_info.totalSize = (info.size_kb/1024 and ((info.size_kb/1024)-(info.size_kb/1024)%0.01))  or 0
    end

	--Save meta file.
	if info.file and info.file ~= "" then
		
        saveRecordMetaFile( record_item.record_path, info.file, record_item.status.record_info )
    end
		
end

local function onRecordStarted( record_item, channel)
	--TODO:
end

local function onRecordStopped( record_item, channel)
	updateRecordStatus( record_item, channel)
	--terminateSnapTask( record_item, channel, recordIdx )
end

local function onRecordFileClosed( record_item, channel, file )
	if record_item.status.record_info then
		--record_item.status.record_info.state = "closed"
		--record_item.status.record_info.stopTime = os.date("%Y-%m-%d %H:%M:%S")
		--保证是自己的文件状态数据,延迟关闭录像，可能导致消息到达和数据更新不同步
		if file and file ~= "" and file == record_item.status.record_info.file then
			record_item.status.record_info.state = "closed"
			record_item.status.record_info.stopTime = os.date("%Y-%m-%d %H:%M:%S")
			saveRecordMetaFile( record_item.record_path, file, record_item.status.record_info )
		end
	end
	--record_item.status.stopTime = os.date("%Y-%m-%d %H:%M:%S")
	record_item.status.record_info.stopTime = os.date("%Y-%m-%d %H:%M:%S")
	--terminateSnapTask( record_item, channel, recordIdx, file )
end

local function onRecordFileNew( record_item, channel, file )
	--fix bug 
	record_item.status.record_info.startTime = os.date("%Y-%m-%d %H:%M:%S")
	record_item.status.record_info.stopTime = "RECORDING"
	--startSnapTask( record_item, channel, file )
end

local function onRecordEvent( topic, msg, msgType )

	--local  pl = require "pl.pretty"
	--print("msgtype is "..msgType)
	--print("topic is "..topic)
	--pl.dump(msg)

	if msgType ~= "table" then
		_dbg.Warn( "STREAMER", "Got invalid recording event message of type:", msgType )
		return
	end
	--local channel = topic:match( "%a+:.+$" )
	local channel = topic:match( "LOCAL.*$" )  or topic:match("OUTPUT.*$")
	if not channel  then
		return
	end

	local record_item = _M.recording[channel]
	if not record_item then
		return
	end


	if msg.action == "start" then
		_dbg.Info( "STREAMER", "Record channel:", channel, "is starting")
		onRecordStarted( record_item, channel)
	elseif msg.action == "stop" then
		_dbg.Info( "STREAMER", "Record channel:", channel, "is stopped" )
		onRecordStopped( record_item, channel)
	elseif msg.action == "close-file" then
		_dbg.Info( "STREAMER", "Record channel:", channel, "currect file is closed:", msg.file )
		onRecordFileClosed( record_item, channel, msg.file )
	elseif msg.action == "new-file" then
		_dbg.Info( "STREAMER", "Record channel:", channel,  "new record file:", msg.file )
		onRecordFileNew( record_item, channel, msg.file )
	end
end




function _M.IsChannelPendingStart(channel)
	local record_item = _M.recording[channel]
	if not record_item then
		return false
	else
		return record_item.start
	end
end

function _M.GetChannelRecordStatus(channel)
	local record_item = _M.recording[channel]
	if not record_item then
		return {work = "null"}
	else
		return record_item.status or {work = "null"}
	end
end

local function checkIsRecording(record_item, path, filename)
	if record_item and record_item.status and record_item.status.work == "started" then
		if record_item.record_path == path and record_item.status.record_info.file == filename then
			return record_item.status.record_info
		end
		
		return nil
	else
		return nil
	end
end

function _M.RemoveRecordFiles(channel, disk_id, files)
	if type(files) ~= "table" then
		files = { files }
	end

	local dm = require "mod_diskManage"
	local diskDetail = dm.GetDiskDetail( disk_id )
	if not diskDetail then
		_dbg.Error( "STREAMER", "Remove channel", channel, "record files but specified an invalid disk." )
		return false
	end
	local path = diskDetail.mount .. "/CHN_" .. channel
	local i
	for i=1, #files do
			_dbg.Info( "STREAMER", "Remove channel:", channel, ",disk:",disk_id,"record file:", path .. "/" .. files[i]  )
			os.execute( "/bin/rm -Rf " .. path .. "/" .. files[i] )
			os.execute( "/bin/rm -Rf " .. path .. "/.META/" .. files[i] .. ".META > /dev/null 2>&1" )
			os.execute( "/bin/rm -Rf " .. path .. "/.META/" .. files[i] .. ".SNAP > /dev/null 2>&1" )
	end
	os.execute( "/bin/sync" )
	mod_channel.DeleteFileStreamFileList(path .. "/" , files)
	return true
end

--[[
function _M.GetChannelRecordList(channel)
	if not channel then
		return nil
	end

	channel = tostring(channel)
	channel =  string.gsub(channel, "%d+%.%d+%.%d+%.%d+:%d+/%a+", "127.0.0.1:1935/src_main")

	--local disk_m = require "mod_diskManage"
	local lfs = require "lfs"

	--Is current recording?
	local record_item = _M.recording[channel]

	--local disk = disk_m.GetValidDisks()

	local path = "/media/disk/" .. "CHN_" .. string.sub(string.match(channel,"src_main/.+$" ), 10, -1)
	local disk_list = {
		path = path,
		list = {}
	}

	local name
	local fnd_meta = nil
	local files = {}
	for name in lfs.dir( path ) do
		if name ~= "." and name ~= ".." then
			if name == ".META" or name == ".meta" then
				fnd_meta = name
			else
				local ext = name:match( ".+%.(.*)$" )
				if ext then
					ext = ext:lower()
					if ext == "ts" or ext == "mp4" or ext == "mkv" or ext == "avi" or ext == "mov" then
						table.insert( files, { name = name, type = ext } )
					end
				end
			end
		end
	end

	local j, record_info, meta_info, snap_path, coverfile
	for j=1, #files do
		record_info = checkIsRecording( record_item, path, files[j].name )
		meta_info = record_info
		snap_path = nil
		coverfile = nil
		if not meta_info and fnd_meta then
			local x 
			x, meta_info = pcall(dofile, path .. "/" .. fnd_meta .. "/" .. files[j].name .. ".META")
			if not x then
				_dbg.Error( "STREAMER", "Fail to load meta file for:", files[j].name, ",error:", meta_info )
				meta_info = nil
			end
		end

		--if fnd_meta then
		--	local attr = lfs.attributes( path .. "/" .. fnd_meta .. "/" .. files[j].name .. ".SNAP/00000000.jpg" )
		--	if attr and attr.mode == "file" then
		--		snap_path = fnd_meta .. "/" .. files[j].name .. ".SNAP"
		--		coverfile = "00000000.jpg"
		--	end
		--end

		if not meta_info then
			local fileSize
			local fileTime
			local attr = lfs.attributes( path .. "/" .. files[j].name )
			if attr and attr.mode == "file" then
				fileSize = math.ceil((attr.blocks/1024) * 512)
				fileTime = os.date("%Y-%m-%d %H:%M:%S", attr.change)
			else
				fileSize = -1 --TODO: Aka. unknown
				fileTime = "1900-01-01 00:00:00"
			end
			table.insert( disk_list.list, {
				id = files[j].name, --NOTE here.
				name = files[j].name,
				work = "idle", --Mark is recording or not.
				snap_path = snap_path,
				cover_file = coverfile,
				--disk = disks[i].name,
				--path = path,
				startTime = fileTime,
				stopTime = fileTime,
				totalSize = fileSize,
				totalDuration = -1 --Aka. unknown.
			})
		else
			table.insert( disk_list.list, {
				id = files[j].name,
				name = files[j].name,
				work = record_info and "started" or "idle",
				snap_path = snap_path,
				cover_file = coverfile,
				--disk = disks[i].name,
				--path = path,
				startTime = meta_info.startTime,
				stopTime = meta_info.stopTime,
				totalSize = meta_info.totalSize,
				totalDuration = meta_info.totalDuration
			})
		end
	end

	table.sort( disk_list.list, function(a,b)
		return a.startTime > b.startTime
	end)

	ret = disk_list
	

	return ret
end
]]


function file_exists(path)
	local file = io.open(path, "rb")
	if file then file:close() end
	return file ~= nil
end


function _M.GetChannelRecordList(channel)
	if not channel then
		return nil
	end
	local lfs = require "lfs"

	--Is current recording?
	local record_item = _M.recording[channel]

	--local disks = disk_m.GetValidDisks()
	disks = disk_manage.GetDiskDetail(nil, true)

	local ret = {}

	local i
	for i=1, #disks do
		if disks[i].online and disks[i].status ==  "ready" then		
			local path = disks[i].mount .. "/CHN_" .. channel
			if file_exists(path) then
				local disk_list = {
					path = path,
					list = {}
				}

				local name
				local fnd_meta = nil
				local files = {}
				for name in lfs.dir( path ) do
					if name ~= "." and name ~= ".." then
						if name == ".META" or name == ".meta" then
							fnd_meta = name
						else
							local ext = name:match( ".+%.(.*)$" )
							if ext then
								ext = ext:lower()
								if ext == "ts" or ext == "mp4" or ext == "mkv" or ext == "avi" then
									print("record file is:"..name)
									table.insert( files, { name = name, type = ext } )
								end
							end
						end
					end
				end

				local j, record_info, meta_info, snap_path, coverfile
				for j=1, #files do
					record_info = checkIsRecording( record_item, path, files[j].name )
					meta_info = record_info
					snap_path = nil
					coverfile = nil
					if not meta_info and fnd_meta then
						local x 
						x, meta_info = pcall(dofile, path .. "/" .. fnd_meta .. "/" .. files[j].name .. ".META")
						if not x then
							_dbg.Error( "STREAMER", "Fail to load meta file for:", files[j].name, ",error:", meta_info )
							meta_info = nil
						end
					end

					if fnd_meta then
						local attr = lfs.attributes( path .. "/" .. fnd_meta .. "/" .. files[j].name .. ".SNAP/00000000.jpg" )
						if attr and attr.mode == "file" then
							snap_path = fnd_meta .. "/" .. files[j].name .. ".SNAP"
							coverfile = "00000000.jpg"
						end
					end

					if not meta_info then
						local fileSize
						local fileTime
						local attr = lfs.attributes( path .. "/" .. files[j].name )
						if attr and attr.mode == "file" then
							fileSize = math.ceil((attr.blocks/1024) * 512)
							fileTime = os.date("%Y-%m-%d %H:%M:%S", attr.change)
						else
							fileSize = -1 --TODO: Aka. unknown
							fileTime = "1900-01-01 00:00:00"
						end
						table.insert( disk_list.list, {
							id = files[j].name, --NOTE here.
							name = files[j].name,
							work = "idle", --Mark is recording or not.
							snap_path = snap_path,
							cover_file = coverfile,
							disk = disks[i].name,
							--path = path,
							startTime = fileTime,
							stopTime = fileTime,
							totalSize = fileSize,
							totalDuration = -1 --Aka. unknown.
						})
					else
						table.insert( disk_list.list, {
							id = files[j].name,
							name = files[j].name,
							work = record_info and "started" or "idle",
							snap_path = snap_path,
							cover_file = coverfile,
							disk = disks[i].name,
							--path = path,
							startTime = meta_info.startTime,
							stopTime = meta_info.stopTime,
							totalSize = meta_info.totalSize,
							totalDuration = meta_info.totalDuration
						})
					end
				end

				table.sort( disk_list.list, function(a,b)
					return a.startTime > b.startTime
				end)
				ret[i] = disk_list
			else
				ret[i] = {}
			end
		else
			ret[i] = {}
		end
	end

	return ret
end

--NOTE：获取系统文件目录
function _M.GetSysFileDocument()
	return _M.recordSysFile
end

--NOTE：add hyl测试用于获取系统文件下录像视频，
-- 参数：diskId对于父文件的id，id对于子文件的id
function _M.GetSysFileRecordList(diskId, id)
	diskId = diskId or 1
    local ret = {}
    local disk_list = {
        path = nil,
        list = {}
    }

	local path = "/kvmedia"
	local n
	local flag = false
	-- NOTE 遍历系统录像文件，找到对于文件夹的路径
	for n=1, #_M.recordSysFile do 
		if _M.recordSysFile[n].id == diskId then
			local m
			for m=1, #_M.recordSysFile[n].children do
				if _M.recordSysFile[n].children[m].id == id then
					-- NOTE: 路径格式：/data/record_01
					path = "/".._M.recordSysFile[n].name .. "/" .. _M.recordSysFile[n].children[m].id 
					flag = true
					break;
				end
			end
		end

		if flag then
			break
		end
	end

	-- NOTE 检测目录path：/data/record_01是否存在，不存在就创建目录
	if not file_exists(path) then
		os.execute("mkdir -p" .. " " .. path)
		return ret
	end

    local name
    local fnd_meta = nil
	local files = {}
	
	-- NOTE: 把所有视频文件添加到files里
    for name in lfs.dir( path ) do
        if name ~= "." and name ~= ".." then
            if name == ".META" or name == ".meta" then
                fnd_meta = name
            else
                local ext = name:match( ".+%.(.*)$" )
                if ext then
					ext = ext:lower()
                    if ext == "ts" or ext == "mp4" or ext == "mkv" or ext == "avi" then
                        print("record file is:"..name)
                        table.insert( files, { name = name, type = ext } )
                    end
                end
            end
        end
    end
 
	-- NOTE: 获取视频文件的信息，比如文件名，时间，大小，路径等
	local j
	for j=1, #files do
		local fileSize
		local fileTime
		local fileUrl = nil
		local attr = lfs.attributes( path .. "/" .. files[j].name )
		if attr and attr.mode == "file" then
			fileSize = math.ceil((attr.blocks/1024) * 512)
			fileTime = os.date("%Y-%m-%d %H:%M:%S", attr.change)
			fileUrl = path .. "/" .. files[j].name
		else
			fileSize = -1 --TODO: Aka. unknown
			fileTime = "1900-01-01 00:00:00"
		end
		table.insert( disk_list.list, {
			id = files[j].name, --NOTE here.
			name = files[j].name,
			work = "idle", --Mark is recording or not.
			snap_path = nil,
			-- cover_file = coverfile,
			disk = nil,
			--path = path,
			startTime = fileTime,
			stopTime = fileTime,
			totalSize = fileSize,
			url = fileUrl,
			storeType = 2, --1-磁盘盘，2-文件系统下
			totalDuration = -1 --Aka. unknown.
		})
	end

	-- NOTE: 对视频文件做时间排序
    table.sort( disk_list.list, function(a,b)
        return a.startTime > b.startTime
    end)
    ret[0] = disk_list

    return ret
end




local function removeOldestRecorder( record_item, channel)
	if not channel or not record_item then
		return false
	end

	

	local item = _M.GetChannelRecordList(channel)
	

	if not item then
		return false
	end
	local path = item.path
	local list = item.list
	table.sort( list, function(a,b)
		return a.startTime < b.startTime
	end)

	local i
	for i=1, #list do
		if list[i].work ~= "started" then
			_dbg.Info( "STREAMER", "Remove channel:", channel, "record file:", list[i].name )
			os.execute( "/bin/rm -Rf " .. path .. "/" .. list[i].name )
			os.execute( "/bin/rm -Rf " .. path .. "/.META/" .. list[i].name .. ".META > /dev/null 2>&1" )
			--os.execute( "/bin/rm -Rf " .. path .. "/.META/" .. list[i].name .. ".SNAP > /dev/null 2>&1" )
			os.execute( "/bin/sync" )
			return true
		end
	end
	return false
end 
---------------------------------------------------




local function PrepareRecordPath( channel, disk_mount )
	
	local path = disk_mount .. "/CHN_" .. channel
	local r,status,code = os.execute( "/bin/mkdir -p " .. path )
	if not r or status ~= "exit" or code ~= 0 then
		_dbg.Error( "STREAMER", "Fail to create channel recording path, for channel:", channel, ", path=", path )
		return nil
	end
	os.execute( "/bin/mkdir -p " .. path .. "/.META" ) --For meta files storage
	return path
end





---------------------------------------------------
-- Anything is ready, create a channel's record session(s)
---------------------------------------------------
local function CreateRecord(channel,mediaSrc,record_item,_record_dir)
    local record_dir = _record_dir

    local recordPath = PrepareRecordPath(channel, record_dir)
    if not recordPath then
        _dbg.Error( "STREAMER", "Fail to prepare record file path for channel")
        record_item.status.work = "error"
		record_item.status.zh_errorInfo = "无法写入文件"
		record_item.status.en_errorInfo = "Unable to write file"
        return false
    end
    record_item.recorder = KM_STREAMING.Recorder.createNew( "record-" .. channel )
	if not record_item.recorder then
		_dbg.Error( "STREAMER", "Fail to create record-group for channel:", channel )
		record_item.status.work = "error"
		record_item.status.zh_errorInfo = "无法创建录像"
		record_item.status.en_errorInfo = "Unable to create video"
		return false
	end

	record_item.record_path = nil
	record_item.record_session = nil
	record_item.record_event_topic = nil


    --Video buffer: 4MB
    --Audio buffer: 64KB
    record_item.record_session = record_item.recorder:AddSession( "REC" .. channel, mediaSrc, 4*1024*1024, 64*1024 )
   -- if not record_item.record_session then
	if not record_item.record_session then
        _dbg.Error( "STREAMER", "Fail to create record session of channel:", channel )
		record_item.status.zh_errorInfo = "录像创建失败"  
		record_item.status.en_errorInfo = "Video creation failed" 
   	 else
        local multiple = false
        local maxSize = "0"
        local maxTime = "0"
        if _M.setting.limitation:match( "time" ) then
            if _M.setting.limitation:match( "loop" ) then
                multiple = true
            elseif _M.setting.limitation:match( "multiple" ) then
                multiple = true
            elseif _M.setting.limitation:match( "split" ) then
                multiple = true
            end
            maxTime = tostring((tonumber(_M.setting.limitTime))*60)
        elseif _M.setting.limitation:match( "size" ) then
            if _M.setting.limitation:match( "loop" ) then
                multiple = true
            elseif _M.setting.limitation:match( "multiple" ) then
                multiple = true
            elseif _M.setting.limitation:match( "split" ) then
                multiple = true
            end
			maxSize = tostring((tonumber(_M.setting.limitSize))*1024*1024)
		end
		
		local filePath = recordPath
        record_item.recorder:SetOption(record_item.record_session, {
            path = filePath,
            type = _M.setting.format,
            prefix = "REC_" .. channel,
            multiple = multiple,
            maxSize = maxSize,
            maxTime = maxTime
        })

        --Don't save meta file by module, myself write it.
        --record_item.recorder:SetMetaFile( record_item.record_sessions[i], filePath .. "/.META", { period = _M.meta_period, name = channel } )
			
        if _M.record_event and _M.record_event_name then
            record_item.record_event_topic = _M.record_event:Subscribe( "recording." .. channel, onRecordEvent )
            record_item.recorder:HandleEvent( record_item.record_session, _M.record_event_name, "recording.".. channel )
        end
        record_item.record_path = filePath
	end
	

	if not record_item.record_session then
		record_item.status.work = "error"
		record_item.recorder = nil
		record_item.record_info = {}
		record_item.record_session = nil
		collectgarbage()
		return false
	else
		record_item.status.work = "started"
		
        record_item.status.record_info = {
            startTime = os.date("%Y-%m-%d %H:%M:%S"),
            stopTime = "RECORDING",
            totalDuration = 0,
            totalSize = 0
        }

        record_item.recorder:StartSession( record_item.record_session )
		return true
	end
end



---------------------------------------------------



---------------------------------------------------

local function recordUpdateTask(thrObj, channel)
	local should_stop = false
	while not thrObj.terminated do
		local record_item = _M.recording[channel]
		if record_item then
			updateRecordStatus( record_item, channel )
		end

		if not thrObj.terminated then
			coaf.Sleep(1)	
		end
	end
end

---------------------------------------------------


local function get_record_dir()
	local record_dir = nil
	
	
	local disk_info =  disk_manage.get_disk_info(nil, true)
	
	--if disk_info[_M.select_disk].specific_status == 5 then
	--	restart_record()
	--end
	
	if disk_info[_M.select_disk].specific_status == 1  or  disk_info[_M.select_disk].specific_status == 3 then
			record_dir = disk_info[_M.select_disk].mount 
	end

	if not record_dir then
		_M.select_disk = _M.select_disk%2 + 1
		if disk_info[_M.select_disk].specific_status == 1  or disk_info[_M.select_disk].specific_status == 3 then
			record_dir = disk_info[_M.select_disk].mount
		end

	end

	if not record_dir then
		_dbg.Error( "STREAMER", "No valid disk(s) for record" )
		-- record_item.status.work = "error"
		-- record_item.status.zh_errorInfo = "没有可用磁盘"
		-- record_item.status.en_errorInfo = "No available disk"
		return false
	end

	return record_dir
end

function _M.StartChannel(channel)
	_dbg.Info("to start recording channel: ", channel)
	
	local channelObj = mod_channel.streams[channel]
	if not channelObj then
		_dbg.Error( "STREAMER", "Invalid channel object while starting record channel:", channel )
		return false, "Invalid media object"
	end

	local record_dir = get_record_dir()

	if not record_dir then 
		return false, "not find usable disk "
	end

	local record_item = _M.recording[channel]
	if not record_item then
		record_item = {
			status = {
				work = "idle",
				en_errorInfo = "",
				zh_errorInfo = "",
				mode = ""
			}
		}
		_M.recording[channel] = record_item
	end

	record_item.start = true

	--If recorder is exist, just ignore it. TODO: make this policy be better.
	if record_item.recorder then
		return true
	end

	record_item.status.record_info = {}
	record_item.status.en_errorInfo = ""
	record_item.status.zh_errorInfo = ""

	local MediaSource = channelObj:GetMediaSource()

	if not channelObj or not MediaSource then
		_dbg.Error( "STREAMER", "Invalid media source while starting record channel:", channel )
		record_item.status.work = "error"
		record_item.status.zh_errorInfo = "无效的媒体源"
		record_item.status.en_errorInfo = "Invalid media source"
		return false, "Invalid media source"
	end

    if not CreateRecord( channel, MediaSource, record_item, record_dir) then
		return false, " internal error"
	end
	record_item.update_task = thread.New( recordUpdateTask )
	if record_item.update_task then
		record_item.update_task:Start( channel )
	end


	disk_manage.disk_status[_M.select_disk].is_record = disk_manage.disk_status[_M.select_disk].is_record + 1 
	disk_manage.GetDiskDetail(_M.select_disk, true)
	record_item.status.use_disk = _M.select_disk

	--Save the snap_url, for starting snap task later (when start record event triggered)
	--record_item.snap_url = channelObj.snap_url
	
	return true
end

---------------------------------------------------

function RemoveChannel(channel)
	_dbg.Info("RECORD", "to stop record channel : "..channel)
	local record_item = _M.recording[channel]
	if not record_item then
		return true
	end
	
	--if record_item.snap_task then
	--	record_item.snap_task:Destroy()
	--	record_item.snap_task = nil
	--end

	if record_item.update_task then
		record_item.update_task:Terminate()
		record_item.update_task = nil
	end
	
	if not record_item.recorder then
		return true
	end

	if record_item.record_session then
		
		record_item.recorder:StopSession( record_item.record_session)
		--record_item.recorder:StopAllSessions()
		
        if _M.record_event and record_item.record_event_topic and record_item.record_event_topic then
            _M.record_event:Unsubscribe( record_item.record_event_topic )
        end
		
		record_item.record_session = nil
		record_item.record_event_topic = nil
	end

	
    record_item.status.record_info.state = "stopped"

    if not record_item.status.record_info.stopTime or
    record_item.status.record_info.stopTime == "" or
    record_item.status.record_info.stopTime == "RECORDING" then
        record_item.status.record_info.stopTime = os.date("%Y-%m-%d %H:%M:%S")
    end

    if record_item.status.record_info.file then
        saveRecordMetaFile( record_item.record_path, record_item.status.record_info.file, record_item.status.record_info )
    end
	

	if record_item.status.work ~= "error" then
		record_item.status.work = "idle"
	end

	record_item.record_path = nil
	record_item.recorder = nil
	collectgarbage()
	
	disk_manage.disk_status[_M.select_disk].is_record = disk_manage.disk_status[_M.select_disk].is_record - 1
	return true
end


---------------------------------------------------

function _M.StopChannel(channel)
	
	local record_item = _M.recording[channel]
	if not record_item then
		return true
	end


	--Mark not start required.
	record_item.start = false
	RemoveChannel(channel)
	 
	disk_manage.GetDiskDetail(_M.select_disk, true)
	return true

end



---------------------------------------------------
function _M.StartRecord()

	-- local is_recording, cur_id = _M.get_record_status()
	-- if is_recording then
	-- 	return true
	-- end
	
	local channels = mod_channel.streams 
	
	local k,_
	for k,_ in pairs(channels) do
		--if _.type == "local" or _.type == "output" then
		if _.origin == "local" or _.origin == "output" then
			if (_.sourceType and _.sourceType == "main") or not _.sourceType then
				if not _M.StartChannel(k) then
					_M.StopChannel(k)
					_dbg.Error("STREAMER", "start record error  channel : " ..k)
					return false, "unknow error"
				end
			end
		end
        
	end

	return true
end
---------------------------------------------------
function _M.StopRecord()
	local mod_channel = require "mod_Streams"
	local channels = mod_channel.streams--Hacking
	
	local k,_
    for k,_ in pairs(channels) do
        
            _M.StopChannel(k)
        
	end
	return true
end
---------------------------------------------------

function _M.SetRecordOptions( opts )
	if type(opts) == "table" then
		local k,v
		for k,v in pairs(opts) do
			_M.setting[k] = v
			_dbg.Info(".............", k, v)
		end

		if _M.setting["disk_number"]   then
			_M.set_record_disk(_M.setting["disk_number"])
		end

		_M.setting:__TRIGGER(true)
		return true
	else
		return false, "Invalid record options"
	end
end

function _M.GetRecordOptions()
	if _M.setting then
		_M.setting["disk_number"] = _M.select_disk

		local  pl = require "pl.pretty"
		pl.dump(_M.setting)

		return _M.setting
	else
		return {}
	end
end


function _M.get_record_status()
	local is_recording = false
	local recording_disk_id = _M.select_disk
	for i=1, #disk_manage.disk_status do
		if disk_manage.disk_status[i].is_record ~= 0 then
			is_recording = true
			recording_disk_id = i
		end
	end

	local nodisk = true
	local diskDetail = disk_manage.get_disk_info()
	local k,v
	for k,v in pairs(diskDetail) do
		if v.online then
			nodisk = false
			break
		end
	end

	if nodisk then 
		recording_disk_id = 0
	end	
	return is_recording, recording_disk_id
end

function _M.set_record_disk(_disk_id)
	if type(_disk_id) ~= "number" then
		_disk_id = tonumber(_disk_id)
	end

	if not _disk_id or (_disk_id ~=1 and _disk_id ~=2) then
	
		return false, "set record disk error"
	end
	
	 local r1, r2 = _M.get_record_status()
	 if r1 and _M.select_disk ~= _disk_id then
		return false, "recording now!!!! can not modify record disk"
	 end

	_M.select_disk = _disk_id
	return true

end


function _M.get_all_record_status()

	local result = {}
	local channels = mod_channel.streams 
	local pl = require "pl.pretty"
	pl.dump(channels) 

	local k,_
	for k,_ in pairs(channels) do
		--if _.type == "local" or _.type == "output" then
		if _.origin == "local" or _.origin == "output" then
			if (_.sourceType and _.sourceType == "main") or not _.sourceType then
				result[k] = _M.GetChannelRecordStatus(k)
			end
		end
        
	end
	
	pl.dump(result)

	return result
end


function get_record_bitrate()
	local channels = mod_channel.streams 
	local total_bitrate = 0
	local k,_
	for k,_ in pairs(channels) do
		if _.origin == "local" or _.origin == "output" then
			if _M.recording[k] and _M.recording[k].status and _M.recording[k].status.work == "started" then
				local stream_info = _:GetStreamStatus()
				if stream_info and stream_info.bitrate then
					total_bitrate = total_bitrate + stream_info.bitrate
				end

			end
		end
        
	end

	return total_bitrate
end

function get_disk_total_free()
	local disk_info = disk_manage.GetDiskDetail(nil, false)
	--print((disk_info[1] and disk_info[1].free or 0).."**********"..(disk_info[2] and disk_info[2].free or 0))
	return ((disk_info[1] and disk_info[1].free or 0)  + (disk_info[2] and disk_info[2].free or 0))

end


function _M.get_record_time_future()
	local t_h=0
	local t_d=0
	local disk_free = get_disk_total_free()
	local total_record_bitrate = get_record_bitrate()
	--print("disk_free is ".. disk_free .. "and total_record_bitrate is ".. total_record_bitrate )
	if disk_free>=2 and   total_record_bitrate~=0 then
	    t_h = math.ceil((((disk_free-2)*1024*1024)/(total_record_bitrate/8)/3600))
		t_d = math.floor(t_h/24)
		--print("t_d --------- : "..t_d)
		if t_d <=99 and t_d >=1 then
			return t_d.."天"
		elseif t_d > 99 then
			return ">"..t_d .. "天"
		elseif t_d < 1 then
			return t_h.."小时"
		end

		

	else
		return false
	end
end

return _M
