local _dbg = require "coaf.debug"

local _M = {
	sipEp = nil,
	mod_Sip = nil
}

local function on_incomingCall( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Warn( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid call account while incoming call!" )
	elseif acc_obj.is_gb28181 then
		local y, i
		if args.body then
			y = args.body:match("\ny=([0-9]+)")
			i = args.body:match("\ni=(.-)[\r\n]")
		end
		local Subject = args.headers["Subject"] or "",

		acc_obj:OnIncomingCall( call_id, args, y, Subject, i )
	else
		_dbg.Info( "SIP", "************", pcall(function()
		acc_obj:OnIncomingCall( call_id, args )
		end))
	end

	if acker then
		acker:Post()
	end
end

local function on_callState( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Warn( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid call account while call state:", args.state )
	else
	--state == DISCONNCTD
	--state == CALLING
	--state == EARLY
	--state == CONNECTING
		acc_obj:OnCallState( call_id, args.state )
	end
	
	if acker then
		acker:Post()
	end
end

local function on_mediaBroken( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Warn( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid call account while media broken." )
	else
		acc_obj:OnMediaBroken( call_id )
	end
	
	if acker then
		acker:Post()
	end
end

local function on_registerStarted( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Warn( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid call account while register started state!" )
	else
		if args and args.renew == 1 then
			acc_obj:OnRegisterState( "reg", "registering", 0 )
		else
			acc_obj:OnRegisterState( "unreg", "unregistering", 0 )
		end
	end
	
	if acker then
		acker:Post()
	end
end

local function on_registerState( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Warn( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid call account while get register state" )
	elseif args then
		if args.renew == 1 then
			if not args.success then
				acc_obj:OnRegisterState( "reg", "error", args.code or 0 )
			else
				acc_obj:OnRegisterState( "reg", "registered", 200 )
			end
		else
			if not args.success then
				acc_obj:OnRegisterState( "unreg", "error", args.code or 0 )
			else
				acc_obj:OnRegisterState( "unreg", "off", 200 )
			end
		end
	end
	
	if acker then
		acker:Post()
	end
end

local function on_mediaState( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_message( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )

	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Error( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid account while get MESSAGE" )
	elseif acc_obj.is_gb28181 then
		if args.body and args.mime_type and args.mime_type == "Application/MANSCDP+xml" then
			local query,cmd,sn = args.body:match("<(Query)>.*<CmdType>(.-)</CmdType>.*<SN>(.-)</SN>")
			if query then
				if cmd == "Catalog" then
					local devId = args.body:match("<DeviceID>(.-)</DeviceID>" )
					acc_obj:OnGB28181Query( devId, sn )
				elseif cmd == "DeviceInfo" then
					local devId = args.body:match("<DeviceID>(.-)</DeviceID>" )
					acc_obj:OnGB28181DeviceInfo( devId, sn )
				elseif cmd == "DeviceStatus" then
					local devId = args.body:match("<DeviceID>(.-)</DeviceID>" )
					acc_obj:OnGB28181DeviceStatus( devId, sn )
				elseif cmd == "ConfigDownload" then
					local devId = args.body:match("<DeviceID>(.-)</DeviceID>" )
					acc_obj:OnGB28181ConfigDownload( devId, sn )
				end
			end
		end
	end

	if acker then
		acker:Post()
	end
end

local function on_initVideoStream( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_initVideoSubsystem( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_initAudioSubsystem( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_startVideoSubsystem( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_startAudioSubsystem( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_destroyVideoSubsystem( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_destroyAudioSubsystem( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_updateVideoStream( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	--_dbg.Info( "SIP", "Remote SDP:", args.remote_sdp )
	--_dbg.Info( "SIP", "Local SDP:", args.local_sdp )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Error( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid account while update video stream!" )
	else
		acc_obj:OnUpdateVideoStream( call_id, args.tx.video_port, args.tx.audio_port )
	end
	if acker then
		-- TODO: To modify SDP fields:
		if acc_obj.OnFixupSDP then
			acc_obj:OnFixupSDP( acker )
		end
		--[[
		if acc_obj.is_gb28181 then
			acker:PutMessage( "o:username", acc_obj.deviceId )
			acker:PutMessage( "o:sessionid", "0" )
			acker:PutMessage( "o:version", "0" )
			acker:PutMessage( "s:", "Play" )

			acker:PutMessage( "drop:b", "b" )
			acker:PutMessage( "drop:x-nat", "X-nat" )

			acker:PutMessage( "video-drop:c", "c" )
			acker:PutMessage( "audio-drop:c", "c" )

			--TODO: Drop 'rtcp'?
			if  acc_obj.SSRC then
				acker:PutMessage( "video:y", acc_obj.SSRC )
			end
		end
		]]
		acker:Post()
	end
end

local function on_stopVideoStream( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Error( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid account while stop video stream!" )
	else
		acc_obj:OnStopVideoStream( acc_id, call_id )
	end
	if acker then
		acker:Post()
	end
end

local function on_updateAudioStream( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_stopAudioStream( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	local acc_obj = _M.mod_Sip.GetAccountObject( acc_id )
	if not acc_obj then
		_dbg.Error( "SIP", "Account:", acc_id, "Call:", call_id, "Invalid account while stop audio stream!" )
	else
		acc_obj:OnStopAudioStream( acc_id, call_id )
	end
	if acker then
		acker:Post()
	end
end

local function on_getVideoCodecAttrs( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

local function on_getAudioCodecAttrs( evId, acc_id, call_id, args, acker )
	_dbg.Info( "SIP", "On:", evId, "Account:", acc_id, "Call:", call_id )
	if acker then
		acker:Post()
	end
end

-------------------------------------------------------------------------
function _M.Init(sipEp)
	_M.sipEp = sipEp
	local mod_Sip = require "mod_Sip"
	_M.mod_Sip = mod_Sip
	sipEp:on("incomingCall", on_incomingCall)
	sipEp:on("callState", on_callState)
	sipEp:on("mediaState", on_mediaState)
	sipEp:on("messageRequest", on_message)
	sipEp:on("registerStarted", on_registerStarted)
	sipEp:on("registerState", on_registerState)
	sipEp:on("mediaBroken", on_mediaBroken)
	sipEp:onMedia("initVideoStream",on_initVideoStream)
	sipEp:onMedia("initVideoSubsystem", on_initVideoSubsystem)
	sipEp:onMedia("initAudioSubsystem", on_initAudioSubsystem)
	sipEp:onMedia("startVideoSubsystem", on_startVideoSubsystem)
	sipEp:onMedia("startAudioSubsystem", on_startAudioSubsystem)
	sipEp:onMedia("destroyVideoSubsystem", on_destroyVideoSubsystem)
	sipEp:onMedia("destroyAudioSubsystem", on_destroyAudioSubsystem)
	sipEp:onMedia("updateVideoStream", on_updateVideoStream)
	sipEp:onMedia("updateAudioStream", on_updateAudioStream)
	sipEp:onMedia("stopVideoStream", on_stopVideoStream)
	sipEp:onMedia("stopAudioStream", on_stopAudioStream)
	sipEp:onMedia("getVideoCodecAttrs", on_getVideoCodecAttrs)
	sipEp:onMedia("getAudioCodecAttrs", on_getAudioCodecAttrs)
end

return _M

