local sys_config = require "sys_config"
local mod_Decoder = require "mod_Decoder"
local mod_GenStream = require "mod_GeneralStream"
local mod_Streamer = require "mod_Streamer"
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

require "kmCore"

local _M = {
}

local stream_meta = {}

function stream_meta:GetSipType()
	if self.call_ctx and self.call_ctx.dir == "out" then
		return "sip_out"
	else
		return "sip_in"
	end
end

function stream_meta:GetIcons()
	if not self.call_ctx then
		return "" --No icons, if invalid.
	elseif self.call_ctx.status == "idle" then
		if self.call_ctx.dir == "out" then
			return "aws"
		else
			return "" --TODO:
		end
	elseif self.call_ctx.status == "ring" then
		if self.call_ctx.dir == "in" then
			return "asw,hup"
		else
			return "hup"
		end
	elseif self.call_ctx.status == "calling" then
		return "hup"
	else
		return ""
	end
end

function stream_meta:GetShowAddress()
	if not self.call_ctx or not self.call_ctx.peer or self.call_ctx.peer == "" then
		return "LISTEN"
	else
		local peer = self.call_ctx.peer
		local tail = peer:match("sip[s]*:(.*)$")
		if not tail then
			tail = peer
		end
		local uid, addr = tail:match("(.-)@(.*)$")
		if uid then
			local pas
			addr, pas = uid:match("(.+):(.*)$")
			if addr then
				return addr
			else
				return uid
			end
		else
			return tail
		end
	end
end

function stream_meta:SpecialInit()
	--TODO:
	return true
end

function stream_meta:SpecialDestroy()
	--TODO:
end

function stream_meta:EnableStream()
	_dbg.Info( "SIP_STREAM", self.id, ": *** Enable streaming..." )
	if self.delay_used_call_info or not self.stream_ms then
		local sipSource
		if self.acc_id and self.call_id then
			sipSource = self.sipEp:GetMediaSource( self.acc_id, self.call_id )
		end

		if not sipSource and self.delay_used_call_info then
			_dbg.Info( "SIP_STREAM", self.id, ": No valid call media source while enable stream, use delayed call media source continously..." )
			self:SelectSource()
			return true
		end

		--Use new media source, clear delay used call info.
		self.delay_used_call_info = nil
		self.stream_ms = sipSource
		--TODO: Create ps demuxer for GB28181.
		_dbg.Info( "SIP_STREAM", self.id, ": Got and bind SIP call media source of:", sipSource )
		self.syncSource:BindMediaSource( sipSource, true ) --Since previous is just only network source, I can reuse the crossbuffer
		mod_Streamer:UpdateStreams( self.id ) --Force update source stream for streamer, because the media source may be nil.
		self:SelectSource() --Select source with default configured.
	end
	return true
end

function stream_meta:DisableStream(delay_use)
	_dbg.Info( "SIP_STREAM", self.id, ": *** Disable streaming, with delay use flag:", delay_use )
	if self.stream_ms then
		if delay_use then
			--Save current call info (account id and call id), mark it is using current media source continously,
			--need clear later.
			self.delay_used_call_info = {
				acc_id = self.acc_id,
				call_id = self.call_id
			}
		else
			self.delay_used_call_info = nil
			--Mark enabled first, Since streamer:UpdateStreams will get media source later.
			self.stream_ms = nil
			self.syncSource:BindMediaSource( nil )
			mod_Streamer:UpdateStreams( self.id ) --Force update source stream for streamer, because the media source has be loss.
			local mod_Sip = require "mod_Sip"
			mod_Sip.UpdateAllSourceBindings() --Force update SIP source bindings when media source has be loss.
		end
	else
		self.delay_used_call_info = nil
	end
	return true
end

function stream_meta:ClearMediaSourceUsing( acc_id, call_id )
	if self.delay_used_call_info and self.delay_used_call_info.acc_id == acc_id and self.delay_used_call_info.call_id == call_id then
		self:DisableStream()
		return true
	else
		return false
	end
end

function stream_meta:ChangeCallID(acc_id, call_id)
	if acc_id ~= self.acc_id or call_id ~= self.call_id then
		local en = self.stream_ms 
		if en then
			self:DisableStream(true) --Delay use
		end
		self.acc_id = acc_id
		self.call_id = call_id
		if en then
			self:EnableStream()
		end
	end
end

function stream_meta:SelectSource( source_id )
	local change = false
	if source_id and source_id ~= self.source_id then
		_dbg.Info( "SIP_STREAM", self.id, "Changed source:", source_id )
		self.source_id = source_id
		change = true
	elseif not source_id then
		change = true
	end
	if self.stream_ms and change then
		if self.acc_id and self.call_id then
			if self.source_id == "" or self.source_id == "none" or self.source_id == "NONE" then
				_dbg.Info( "SIP_STREAM", self.id, "No source specified so remove SIP source binding" )
				self.sipEp:BindMediaSource( self.acc_id, self.call_id, nil )
			else
				local mod_Streams = require "mod_Streams"
				local stream_obj = mod_Streams.GetStreamObject( self.source_id )
				if not stream_obj then
					_dbg.Info( "SIP_STREAM", self.id, "Source without valid MeidaSource so remove SIP source binding" )
					self.sipEp:BindMediaSource( self.acc_id, self.call_id, nil )
				else
					_dbg.Info( "SIP_STREAM", self.id, "Bind new media source" )
					self.sipEp:BindMediaSource( self.acc_id, self.call_id, stream_obj:GetMediaSource() )
				end
			end
		end
	end
end

function stream_meta:GetValidSources()
end

--此处需要更新

function stream_meta:ChangeSource(url,options)
	--TODO:
end

function stream_meta:SpecialStart(group, action)
	--TODO:
	self:EnableStream()
end

function stream_meta:SpecialStop(group, action)
	--TODO:
	self:DisableStream()
end

function stream_meta:GetStreamStatus(group)
	local with_act, acts = self:CheckStartActions()
	local output_start = false
	local stream_start = false
	if with_act then
		local k,v
		for k,v in pairs(acts) do
			if k == "output" then
				output_start = true
				stream_start = true
				break
			else
				stream_start = true
			end
		end
	end

	local ret = {
		output_start = output_start,
		stream_start = stream_start
	}

	local call_state = "ERROR"
	local live_status = "off"
	if self.call_ctx then
		if self.call_ctx.status == "idle" then
			call_state = "IDLE"
		elseif self.call_ctx.status == "ring" then
			call_state = "RINGING..."
			live_status = "connecting"
		elseif self.call_ctx.status == "hangup" then
			call_state = "HANGING..."
		elseif self.call_ctx.status == "answer" then
			call_state = "ANSWERING..."
			live_status = "connecting"
		elseif self.call_ctx.status == "calling" then
			call_state = "IN CALL"
			live_status = "connecting"
		else
			call_state = "CALL ERROR"
			live_status = "error"
		end
	end

	if not self.stream_ms then
		ret.live = live_status
		ret.resolution = call_state --TODO:
		ret.bitrate = 0
		ret.video = "NONE"
		ret.audio = "NONE"
	else
		--TODO: Get stream details.
		local status = self.sipEp:GetStatus(self.acc_id, self.call_id)
		if status and status.video then
			ret.live = "on"
			if status.video.frameRate == 0 then
				status.video.frameRate = "auto"
			end
			ret.resolution = '' .. (status.video.width or "0") .. "x" .. (status.video.height or "0") .. "@" .. (status.video.frameRate or "0") .. "Hz"
			ret.bitrate = status.video.bitrate or 0
			ret.video = status.video.codec or "H264"
			ret.audio = "g711"
		else
			ret.live = "on"
			ret.resolution = call_state --TODO:
			ret.bitrate = 0
			ret.video = "H264"
			ret.audio = "Default"
		end
	end
	return ret
end

function stream_meta:GetMediaSource()
	if not self.sipEp or not self.acc_id or not self.call_id then
		return nil
	elseif not self.stream_ms then
		return nil
	else
		return self.stream_ms
		--return self.sipEp:GetMediaSource( self.acc_id, self.call_id )
	end
end

stream_meta = mod_GenStream.ConstructStreamMeta( stream_meta )

-----------------------------------------------------

function _M.CreateStream(sipEp, id, name, call_ctx, acc_id, call_id)
	local min_group, decoder_group = mod_Decoder.AllocateDecoderGroup()
	local stream_obj = {
		type = "sip",
		sipEp = sipEp,
		id = id,
		call_ctx = call_ctx,
		decoder_group_id = min_group,
		decoderGroup = decoder_group,
		name = name,
		acc_Id = acc_id,
		call_id = call_id,
		decoder_channel = nil,
		start_actions = {},
		audio_outputs = {},
		audio_enabled = {},
		vpss_using = {},
		current_audio_intfs = {},
		stream_openned = false,
		buffer_option = "live:200:200:500:0:0",
		no_manual_url = true,
		source_id = nil,
		delay_used_call_info = nil,
		origin = "manual"
	}
	setmetatable( stream_obj, stream_meta )
	stream_obj:Init()
	return stream_obj
end

return _M

