require "himpp"
local _dbg = require "coaf.debug"

local SYS = HiMPP.SYS
local VB = HiMPP.VB
local VPSS = HiMPP.VPSS
local VI = HiMPP.VI
local RGN = HiMPP.RGN

local _M = {
	-- You can change this for VPSS alocation.
	VPSS_ID_BASE = 0,
	MAX_CHANNELS = 2,
	--
	vi_dev = nil,
	vi_channel = nil,
	
	vi_attr = {
		stCapRect = {0,0,1920,1080},
		stDestSize = {1920,1080},
		enCapSel = "BOTH",
		enScanMode = "PROGRESSIVE",
		enPixFormat = "YUV_SEMIPLANAR_420",
		bMirror = false,
		bFlip = false,
		s32SrcFrameRate = 60,
		s32DstFrameRate = 60
	},

	channels = {}
}

--NOTE: SYS.SetMemConf(...) to configure channel using which DDR.

-- Init Encoder module.
function _M.Init()
	if not VI then
		_dbg.Warn( "CAPTURE", "This chipset does not support video capture feature!" )
		return
	end

	if _M.vi_dev then
		_dbg.Warn( "CAPTURE", "The capture module is already initialized." )
		return
	end

	_M.vi_dev = VI.GetDevice(0)
	if not _M.vi_dev then
		_dbg.Error( "CAPTURE", "Fail to get VI input device 0" )
		return
	end

	_M.vi_channel = _M.vi_dev:GetChannel(0)
	--Disable them first.
	_M.vi_channel:Disable()
	_M.vi_dev:Disable()

	local r, errmsg
	r, errmsg = _M.vi_dev:ConfigMode( "BT1120" )
	if not r then
		_dbg.Error( "CAPTURE", "Fail to configure VI device to BT.1120 mode, with error:", errmsg )
		_M.vi_dev = nil
		return
	end

	r, errmsg = _M.vi_channel:SetAttr( _M.vi_attr )
	if not r then
		_dbg.Error( "CAPTURE", "Fail to set VI channel attributes:", errmsg )
	end

	--TODO: test only
	--_M.testOsd = KM_OSDMODULE.Hisi.Sdk.createNew(0)
end

-- Cleanup encoder module.
function _M.Cleanup()
	if not _M.vi_dev then
		return
	end

	local i
	for i=1, _M.MAX_CHANNELS do
		if _M.channels[i] then
			_M.channels[i]:Destroy()
		end
	end
	_M.channels = {}

	if _M.vi_channel then
		_M.vi_channel:Destroy()
		_M.vi_channel = nil
	end

	_M.vi_dev:Disable()
	_M.vi_dev = nil
end

-- Set source video format (width, height, frame-rate), each one can be
-- nil, that means ignore it.

function _M.SetSourceVideo( w, h, fr )
	local changed = false
	if w and w ~= _M.vi_attr.stDestSize[1] then
		changed = true
		_M.vi_attr.stDestSize[1] = w
		_M.vi_attr.stCapRect[3] = w
	end
	if h and h ~= _M.vi_attr.stDestSize[2] then
		changed = true
		_M.vi_attr.stDestSize[2] = h
		_M.vi_attr.stCapRect[4] = h
	end
	if fr and fr ~= _M.vi_attr.s32SrcFrameRate then
		changed = true
		_M.vi_attr.s32SrcFrameRate = fr
		_M.vi_attr.s32DstFrameRate = fr
	end
	if changed and _M.vi_channel then
		_M.vi_channel:Disable()
		local r, errmsg = _M.vi_channel:SetAttr( _M.vi_attr )
		_M.vi_channel:Enable()
		if not r then
			_dbg.Error( "CAPTURE", "Fail to set VI channel attributes:", errmsg )
		end

		local i
		for i=1, _M.MAX_CHANNELS do
			if _M.channels[i] then
				_M.channels[i]:UpdateVPSS()
			end
		end
	end
end

-------------------------------------------------------
-- Channel meta methods.
-------------------------------------------------------
local channel_meta = {}

-- Init encoding channel.
function channel_meta:Init()
	if self.initialized then
		return
	end

	if not _M.vi_channel then
		_dbg.Error( "CAPTURE", "Invalid VI device! Have you initialized it?" )
		return
	end

	VPSS.ForceFreeGroup( _M.VPSS_ID_BASE + self.id - 1 )

	-- 1st, Create a VPSS goup, for resizer, also for Overlay.
	self.vpssGroup = VPSS.GetGroup(_M.VPSS_ID_BASE + self.id - 1)
	if not self.vpssGroup then
		_dbg.Error( "CAPTURE", "Fail to create VPSS group, for video capture channel ID=" .. self.id )
		return
	end

	local r, errmsg
	r, errmsg = self.vpssGroup:Create({
		u32MaxW = 1920,--_M.vi_attr.stDestSize[1],
		u32MaxH = 1088,--_M.vi_attr.stDestSize[2],
		enPixFmt = "YUV_SEMIPLANAR_420",
		--bNrEn = true,
		enDieMode = "NODIE"
	})
	if not r then
		_dbg.Error( "CAPTURE", "Fail to initialize VPSS group, for video capture channel ID=" .. self.id .. ", error:", errmsg )
		self.vpssGroup = nil
		return
	end

	-- 2nd, Bind VI channel to VPSS group.
	r, errmsg = SYS.Bind( _M.vi_channel, self.vpssGroup )
	if not r then
		_dbg.Error( "CAPTURE", "Fail to bind VI channel to VPSS group, for capture channel ID=" .. self.id .. ", reason:", errmsg )
		self.vpssGroup:Destroy()
		self.vpssGroup = nil
		return
	end

	r = _M.vi_dev:Enable()	--Enable VI first.
	if r ~= 0 then
		_dbg.Error( "CAPTURE", "Fail to enable VI device, with error code:", string.format("%08X",r) )
	end

	--TODO: test only
	--[[
	self.region = RGN.GetRegion( self.id-1 )
	
	_dbg.Print( "CAPTURE", "Create Region:",
	self.region:Create({
		type = "OVERLAYEX",
		attr = {
			enPixelFmt = "RGB_1555",
			u32BgColor = 0,
			stSize = {1280,128}
		}
	})
	)

	_dbg.Print( "CAPTURE", "SetAttachField:",
	self.region:SetAttachField("FRAME")
	)
 	print("*****************xgw")	
	if _M.testOsd then
		_M.testOsd:StartOSD()
		--_M.testOsd:SetOverlay( self.id-1, {x=0,y=0,font=0,size=48,outline=2,text="Region叠加测试"} )
		_M.testOsd:SetOverlay( self.id-1, {x=0,y=0,font=0,size=48,outline=2,time=1} )

	end
	]]

	self.initialized = true
end

-- Destroy channel.
function channel_meta:Destroy()
	if not self.initialized then
		return
	end

	--TODO: test only
	--[[
	if self.region then
		_M.testOsd:ClearOSDParams( self.id-1 )
		self.region:Destroy()
		self.region = nil
	end
	]]

	self.vpssGroup:Stop()
	SYS.UnBind( _M.vi_channel, self.vpssGroup )
	self.vpssGroup:Destroy()
	
	self.vpssGroup = nil
	
	self.initialized = false
end

-- Start channel encoding.
function channel_meta:Start()
	if not self.initialized then
		self:Init()
	end
	if self.initialized then
		local r
		r = _M.vi_channel:Enable()
		if r ~= 0 then
			_dbg.Error( "CAPTURE", "Fail to enable VI input channel for capture(ID=" .. self.id .. "), error:", string.format("%08X",r) )
		end
		r = self.vpssGroup:Start()
		if r ~= 0 then
			_dbg.Error( "CAPTURE", "Fail to start VPSS group for capture(ID=" .. self.id .. "), error:", string.format("%08X",r) )
		end
	end
end

-- Stop channel encoding.
function channel_meta:Stop()
	if self.initialized then
		self.vpssGroup:Stop()
	end
end

-- Get current encoding channel used VPSS group
-- so you can use it to bind to output, ... etc.
function channel_meta:GetVPSSGroup()
	return self.vpssGroup
end

-- Update the VPSS parameters.
function channel_meta:UpdateVPSS()
--[[
	if self.vpssGroup then
		local attr = self.vpssGroup:GetAttr()
		if not attr then
			_dbg.Warn( "CAPTURE", "Fail to get VPSS group attributes, using default attributes" )
			attr = {
				enPixFmt = "YUV_SEMIPLANAR_420",
				--bNrEn = true,
				enDieMode = "NODIE"
			}
		end
		attr.u32MaxW = _M.vi_attr.stDestSize[1]
		attr.u32MaxH = _M.vi_attr.stDestSize[2]

		local r, errmsg = self.vpssGroup:SetAttr(attr)
		if not r then
			_dbg.Error( "CAPTURE", "Fail to set capture channel " .. self.id .. " VPSS attributes, reason: ", errmsg )
		end
	end
]]
end

channel_meta.__index = channel_meta

--------------------------------------------------------------

-- Create an capture channel.
-- Return an capture channel object.
function _M.CreateChannel(chanId)
	chanId = chanId or 1
	if chanId < 1 or chanId > _M.MAX_CHANNELS then
		_dbg.Error( "CAPTURE", "Invalid channel ID passed. It's only support 1-" .. _M.MAX_CHANNELS .. " channels." )
		return nil
	end

	if _M.channels[chanId] then
		return _M.channels[chanId]
	end

	local chanObj = {
		id = chanId,
		vpssGroup = nil
	}

	local r = setmetatable( chanObj, channel_meta )
	_M.channels[ chanId ] = r
	return r
end

------------------------

return _M

