local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"

require "himpp"
local HDMI = HiMPP.HDMI

local _M = {
}

----------------------------------------------
-- HDMI meta.
----------------------------------------------

local hdmi_meta = {}

local function hdmi_detection_task(thrObj, hdmi_obj)
	local vo_drvs = require "video.vo_drivers"

	_dbg.Info( "HDMI", "HDMI auto detection task is running..." )
	local loss_edid_counter = 0
	local loss_conn_counter = 1 --So first time, can print the connection info.
	local action_counter = 0
	while not thrObj.terminated do
		if action_counter > 0 then
			action_counter = action_counter + 1
			if action_counter >= 20 then --2seconds per time.
				action_counter = 0
			end
			coaf.Sleep(0.1)
		else
			action_counter = 1
			hdmi_obj.buffered_edid = hdmi_obj.hdmi_if:ForceGetEDID()
			if not hdmi_obj.buffered_edid then
				loss_edid_counter = loss_edid_counter + 1
			else
				hdmi_obj.capabilty = hdmi_obj.hdmi_if:GetSinkCapability()
				if not hdmi_obj.capabilty then
					loss_edid_counter = loss_edid_counter + 1
				else
					if loss_edid_counter > 0 then
						_dbg.Info( "HDMI", "HDMI sink is ready! (Got EDID by auto detection)" )
					end
					loss_edid_counter = 0
					hdmi_obj.sink_loss_or_error = false
				end
			end

			if loss_edid_counter >= 3 then  --EDID lost
				hdmi_obj.sink_loss_or_error = true
				if loss_edid_counter == 3 then
					_dbg.Info( "HDMI", "HDMI sink is loss or error! (Fail get EDID by auto detection)" )
				end
			elseif hdmi_obj.capabilty then
				if not hdmi_obj.capabilty.bConnected then
					loss_conn_counter = loss_conn_counter + 1
					if loss_conn_counter == 1 then
						_dbg.Info( "HDMI", "HDMI sink is disconnected! (Connection loss by auto detection)" )
					end
				else
					if loss_conn_counter > 0 then
						loss_conn_counter = 0
						_dbg.Info( "HDMI", "HDMI sink is connected! (By auto detection)" )
					end
					local work_mode = "DVI"
					local rgb_yc_mode = "RGB444"
					local best_format = nil
					if hdmi_obj.capabilty.bSupportHdmi then
						work_mode = "HDMI"
						if hdmi_obj.capabilty.bSupportYCbCr then
							if vo_drvs.support_hdmi_ycbcr422 then
								if hdmi_obj.capabilty.bSupportDeepColorYCBCR444 then
									rgb_yc_mode = "YCBCR444"
								else
									rgb_yc_mode = "YCBCR422"
								end
							else
								rgb_yc_mode = "YCBCR444"
							end
						end
					end
					best_format = hdmi_obj.capabilty.enNativeVideoFormat

					if hdmi_obj.detect_working_mode ~= work_mode or
					   hdmi_obj.auto_rgb_yc_mode ~= rgb_yc_mode or
					   (best_format and hdmi_obj.best_format ~= best_format) then
					   _dbg.Info( "HDMI", "The auto detection format info is changed, update the HDMI port configuration!" )
					   hdmi_obj.detect_working_mode = work_mode
					   hdmi_obj.auto_rgb_yc_mode = rgb_yc_mode
					   hdmi_obj.best_format = best_format
					   hdmi_obj:_UpdateHDMIAttr()
					end
				end
			end
		end --1 seconds do a detection, but sleep only 0.1s.
	end
	_dbg.Info( "HDMI", "HDMI auto detection task is terminated!" )
end

function hdmi_meta:Init()
	self.hdmi_if = HDMI.GetInterface()
	if not self.hdmi_if then
		_dbg.Error( "HDMI", "Fail to get HDMI interface object!" )
	else
		local r = self.hdmi_if:Open()	--Must open it first.
		_dbg.Info( "HDMI", "Open HDMI interface with result:", string.format("%08x",r))
		if r ~= 0 then
			_dbg.Error( "HDMI", "Fail to open HDMI interface!" )
			self.hdmi_if = nil
			return
		end
	end
end

function hdmi_meta:Destroy()
	--TODO: Destroy.
end

--This is *MUST REQUIRED* for soc/output module.
function hdmi_meta:Enable()
	if not self.hdmi_if then
		_dbg.Error( "HDMI", "Enable: HDMI interface is not initialized!" )
		return
	end
	if not self.enabled then
		self.hdmi_if:Stop()
		self.enabled = true
		self:_UpdateHDMIAttr()
	end
end

--This is *MUST REQUIRED* for soc/output module.
function hdmi_meta:Disable()
	if not self.hdmi_if then
		_dbg.Error( "HDMI", "Disable: HDMI interface is not initialized!" )
		return
	end
	self.enabled = false
	self.hdmi_if:Stop()
end

--This is *MUST REQUIRED* for soc/output module.
function hdmi_meta:UpdateResolution( res, syncDetail, t1001 )
	if self.resolution ~= res then
		self.resolution = res
		self:_UpdateHDMIAttr()
	end
end

function hdmi_meta:_UpdateHDMIAttr()
	if not self.hdmi_if or not self.enabled then
		return
	end

	local vidMode = "RGB444"
	if not self.rgb_yc_mode or self.rgb_yc_mode == "AUTO" then
		if self.auto_detect_task and self.auto_rgb_yc_mode then
			vidMode = self.auto_rgb_yc_mode
		end
	else
		vidMode = self.rgb_yc_mode
	end

	local resolution = self.resolution
	if resolution == "AUTO" or resolution == "auto" then
		resolution = self.best_format or "720P60"
	end

	--TODO: May this correct? 4K@60Hz format, force it to be YCBCR420?
	
	--if resolution == "2560x1440@60" or resolution == "2560x1600@60" or resolution == "3840x2160@50" or resolution == "3840x2160@60" then
	--	local vo_drvs = require "video.vo_drivers"
	--	if vo_drvs.support_hdmi_ycbcr420 then
	--		vidMode = "YCBCR420"
	--	end
	--end

	local depColor = 24
	--[[
	if vidMode == "YCBCR422" or vidMode == "YCBCR420" then
		depColor = 16
	end
	]]

	local work_mode = "DVI"
	if not self.detect_working_mode or not self.auto_detect_task then
		work_mode = self.fail_default_mode
	else
		work_mode = self.detect_working_mode
	end

	_dbg.Info( "HDMI", "Set/change the HDMI attributes: mode=", work_mode, "resolution=", resolution, "colorspace=", vidMode, "depColor=", depColor, "audioSampling=", self.audio_sampling );

	local attr = {
		bEnableHdmi = work_mode == "HDMI",
		bEnableVideo = true,
		enVideoFmt = resolution,
		enVidOutMode = vidMode,
		enDeepColorMode = depColor,
		bEnableAudio = work_mode == "HDMI" and self.enabled_audio or false,
		enSoundIntf = "I2S",
		bIsMultiChannel = false, --TODO: here
		enSampleRate = self.audio_sampling,
		enBitDepth = self.audio_sampleWidth,
		bEnableAviInfoFrame = work_mode == "HDMI",
		bEnableAudInfoFrame = work_mode == "HDMI",
		bEnableSpdInfoFrame = false,
		bDebugFlag = false,
		bHDCPEnable = false, --TODO:
		b3DEnable = false, --TODO:
		enDefaultMode = work_mode,
		bAuthMode = self.enabled_authMode
	}

	self.hdmi_if:Stop()

	_dbg.Info( "HDMI", "Set attribute result:",
	self.hdmi_if:SetAttr( attr )
	)

	self.hdmi_if:Start()
end

function hdmi_meta:EnableAutoDetection(en)
	if not en then
		if self.auto_detect_task then
			self.auto_detect_task:Terminate()
			self.auto_detect_task = nil
		end
	else
		if self.hdmi_if and not self.auto_detect_task then
			self.auto_detect_task = thread.New( hdmi_detection_task )
			if self.auto_detect_task then
				self.detect_working_mode = nil
				self.auto_rgb_yc_mode = nil
				self.best_format = nil
				self.auto_detect_task:Start(self)
			end
		end
	end
end

function hdmi_meta:EnableAudio(en)
	if self.enabled_audio ~= en then
		self.enabled_audio = en
		self:_UpdateHDMIAttr()
	end
end

function hdmi_meta:SetAudioFormat(sampling, channel, sampleWidth )
	if self.audio_sampling ~= sampling or
	   self.audio_channels ~= channel or
	   self.audio_sampleWidth ~= sampleWidth then
	   self.audio_sampling = sampling
	   self.audio_channels = channel
	   self.audio_sampleWidth = sampleWidth
	   if self.enabled_audio then
		   self:_UpdateHDMIAttr()
	   end
	end
end

function hdmi_meta:SetVideoColorOptions(rgb_yc_mode)
	local vo_drvs = require "video.vo_drivers"
	if rgb_yc_mode == "YCBCR422" and not vo_drvs.support_hdmi_ycbcr422 then
		rgb_yc_mode = "YCBCR444"
	elseif rgb_yc_mode == "YCBCR420" and not vo_drvs.support_hdmi_ycbcr420 then
		if vo_drvs.support_hdmi_ycbcr422 then
			rgb_yc_mode = "YCBCR422"
		else
			rgb_yc_mode = "YCBCR444"
		end
	end

	if self.rgb_yc_mode ~= rgb_yc_mode then
		self.rgb_yc_mode = rgb_yc_mode
		self:_UpdateHDMIAttr()
	end
end

function hdmi_meta:EnableHDCP(en)
	self.enabled_hdcp = en
	--TODO: Now nothing to do.
end

function hdmi_meta:Enable3D(en)
	self.enabled_3d = en
	--TODO: Now nothing to do.
end

function hdmi_meta:SetAuthMode(en)
	if self.enabled_authMode ~= en then
		self.enabled_authMode = en
		self:_UpdateHDMIAttr()
	end
end

function hdmi_meta:SetFailDefaultMode(mode)
	if self.fail_default_mode ~= mode then
		self.fail_default_mode = mode
		self:_UpdateHDMIAttr()
	end
end

function hdmi_meta:GetEDID()
	if self.hdmi_if then
		if self.buffered_edid then --If auto detection task is running and buffered EDID, return thus.
			return self.buffered_edid
		else
			return self.hdmi_if:ForceGetEDID()
		end
	else
		return nil
	end
end

hdmi_meta.__index = hdmi_meta

----------------------------------------------

local default_hdmi = nil

function _M.Init()
	local tryIf = HDMI.GetInterface()
	if tryIf then
		tryIf:Close()
		tryIf = nil
		collectgarbage()
	end
	HDMI.DeInit()
	HDMI.Init()
	if not default_hdmi then
		default_hdmi = {
			interface_type = "HDMI",
			support_4k = true, --TODO:
			enabled = false,
			resolution = "1080P30",
			syncDetail = nil,
			t1001 = false,
			enabled_audio = true,
			enabled_hdcp = false,
			enabled_3d = false,
			enabled_authMode = false,
			rgb_yc_mode = "AUTO", --Auto detect.
			detect_working_mode = nil,
			fail_default_mode = "HDMI",
			audio_sampling = 48000,
			audio_channels = 2,
			audio_sampleWidth = 16,
			auto_detect_task = nil
		}

		setmetatable( default_hdmi, hdmi_meta )
		default_hdmi:Init()
	end
end

--Now only support one instance.
function _M.GetInstance()
	return default_hdmi
end

return _M

