-- Downloader dir mapping, for different products, you can
-- replace this file to your mapping.

local client = require "coaf.client"
local _dbg = require "coaf.debug"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)

local proxy_Cli
local proxy_Cli_coserver

local function getSysctrlProxy()
	local proxy
	if not proxy_Cli then
		proxy_Cli = client.New( "systemctrl", "local" )
		if not proxy_Cli then
			_dbg.Error( "DOWNLOAD", "Fail to create systemctrl client")
			return nil
		end
	end

	proxy = proxy_Cli:CreateProxy("/")
	if not proxy then
		_dbg.Error( "DOWNLOAD", "Fail to create systemctrl proxy /" )
		return nil
	end
	return proxy
end

local function getCoserverProxy()
	local proxy
	if not proxy_Cli_coserver then
		proxy_Cli_coserver = client.New( "_CO_DAEMON.coserver", "local" )
		if not proxy_Cli_coserver then
			_dbg.Error( "DOWNLOAD", "Fail to create coserver client")
			return nil
		end
	end

	proxy = proxy_Cli_coserver:CreateProxy("/discovery")
	if not proxy then
		_dbg.Error( "DOWNLOAD", "Fail to create coserver proxy /discovery" )
		return nil
	end
	return proxy
end

local function getMainAddress()
	local proxy = getCoserverProxy()
	if not proxy then
		return nil
	end

	local done,list = proxy.GetDiscoveryList()
	proxy:Destroy()

	if not done or type(list) ~= "table" then
		_dbg.Error( "DOWNLOAD", "Get PLATFORM location address failed!" )
		return nil
	end

	local i,v
	for i,v in ipairs(list) do
		if v.id == "PLATFORM" then
			local port = 80
			local http_type = "http"
			if type(v.options) == "table" then
				if v.options["http_port"] then
					port = v.options["http_port"]
				elseif v.options["https_port"] then
					http_type = "https"
					port = v.options["https_port"]
				end
			end
			if v.address then
				return v.address .. ":" .. port, http_type
			else
				break
			end
		end
	end
	_dbg.Error( "DOWNLOAD", "No valid PLATFORM location address found!" )
	return nil
end

local _M = {

["firmware"] = { 
	location = "/tmp",
	-- You can add a function named 'filterUrl' to modify URL before download:
	filterUrl = function(url)
		if not url then
			return nil
		else
			local flag = url:match("^(.*)://.*")
			if flag then
				return url
			else
				local address, http_type = getMainAddress()
				if not address then
					return nil
				else
					return (http_type or "http") .. "://" .. address .. url
				end
			end
		end
	end,
	--
	-- You can add a function named 'beforeProcess' to prepare downloading:
	beforeProcess = function(dir,file,url)
		os.execute( "/bin/mkdir -p " .. dir )
		os.execute( "/bin/rm -Rf " .. file )
		os.execute( "/bin/rm -Rf /tmp/.*" ) --Clear the exist temporary files.
		return true
	end,
	--
	-- You can add a functin named 'afterProcess' to process downloaded file:
	afterProcess = function(status,dir,file)
		local sysctrl = getSysctrlProxy()
		if not sysctrl then
			_dbg.Error( "DOWNLOAD", "Fail to get systemctrl proxy!" )
			return false, "Fail to get systemctrl proxy"
		else
			local r,reason,err = sysctrl.ProcessFirmware( dir .. "/" .. file )
			os.execute( "/bin/rm -Rf " )
			if not r or not reason then
				sysctrl:Destroy()
				return false, reason
			else
				-- sysctrl.Reboot(3) -- **DONT** After 3 seconds later, reboot.
				sysctrl:Destroy()
				return true
			end
		end
	end
},

["font"] = { 
	location = "/usr/share/fonts",
	filterUrl = function(url)
		if not url then
			return nil
		else
			local flag = url:match("^(.*)://.*")
			if flag then
				return url
			else
				local address, http_type = getMainAddress()
				if not address then
					return nil
				else
					return (http_type or "http") .. "://" .. address .. url
				end
			end
		end
	end
}

}

return _M

