
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"
local CO = require "mod_coservice"
local pt = require "pl.pretty"
local mo_icon = require "mod_IconData"

_M ={

}

local function FormatNum (num)
    if num <= 0 then
        return 0
    else
        local t1, t2 = math.modf(num)
        ---小数如果为0，则去掉
        if t2 > 0 then
            return num
        else
            return t1
        end
    end
end

function _M.getEncoderInfo(id)
	local data={
		resolution = "----",
		bitrate = "----",
		codec = "----",
		status = "----"
	}
	-- 暂时只获取00主码流的状态信息 01 子码流 02 MJPEG
	local proxy = CO.GetProxy( "/encoder/00", "codec", "LOCAL-0"..(id-1) or 0 ) 
    if not proxy then
        _dbg.Error( "GETINFO", "无法连接Amba模块！" )
		return data
	else
		local r, ret, encoderinfo = proxy:GetVideoProperties()
		proxy:Destroy()
		if r and encoderinfo and ret == 6 then
			data.resolution = encoderinfo["video.width"] .. "x" .. encoderinfo["video.height"] .. "@" .. encoderinfo["video.framerate"]
			data.bitrate = FormatNum(encoderinfo["video.bitrate"]/1000000) .. "Mbps" or "----"
			data.codec = encoderinfo["video.codec"] or "----"
        else
            _dbg.Error( "GETINFO", "获amba编码信息失败" )
			return data
		end
	end
	return data
end

function _M.getOutputEncoderInfo(id)
	local data={
		resolution = "----",
		bitrate = "----",
		codec = "----",
		status = "----"
	}
	local proxy = CO.GetProxy( "/encoder", "codec" ) 
    if not proxy then
        _dbg.Error( "GETINFO", "无法连接输出编码模块！" )
		return data
	else
		local r, encoderinfo = proxy.GetEncoderStatus("OUTPUT"..(id))
		proxy:Destroy()
		if r and encoderinfo then
			if (encoderinfo.encoder) then
				data.resolution = '' .. encoderinfo.encoder.encodingOpt.attr.u32PicWidth .. "x" .. encoderinfo.encoder.encodingOpt.attr.u32PicHeight
				data.resolution = data.resolution .. "@" .. encoderinfo.encoder.bitrateOpts["CBR"].attr.fr32DstFrmRate .. "Hz"
				data.bitrate = FormatNum(encoderinfo.encoder.bitrateOpts[ encoderinfo.encoder.currentBrMode or "CBR" ].attr.u32BitRate / 1000) .."Mbps" 
				data.codec = "h264"..encoderinfo.encoder.currentBrMode
			end
        else
            _dbg.Error( "GETINFO", "获取输出编码信息失败" )
			return data
		end
	end
	return data
end

-- pt.dump(_M.getOutputEncoderInfo(1))
-- pt.dump(_M.getOutputEncoderInfo(2))


local function getDevices()
	local proxy = CO.GetProxy( "/device", "codec" )
    if not proxy then
        _dbg.Error( "GETINFO", "无法连接Codec模块！" )
		return nil
	else
		local r, devices = proxy.GetDeviceIDList()
		proxy:Destroy()
		if r and devices then
			return devices
        else
            _dbg.Error( "GETINFO", "获amba输入源信息失败" )
			return nil
		end
	end
end

local function getAVINinfo(id)
	local data={
		format = "----",
		signal = false,
		source = "----"
	}

	local proxy = CO.GetProxy( "/avio", "codec", id or "LOCAL-00" )
    if not proxy then
        _dbg.Error( "GETINFO", "无法连接Amba模块！" )
		return data
	else
		local r, avinfo = proxy.GetVideoFormat()
		proxy:Destroy()
		if r and avinfo then
			data.format = avinfo.format or "----"
			data.signal = avinfo.signal or false
			data.source = avinfo.source or "----"
        else
            _dbg.Error( "GETINFO", "获amba输入源信息失败" )
			return data
		end
	end
	return data
end

function _M.getVideoInInfo()
	local ret = {}
	local info
	local k

	local devices = getDevices()
	if devices then
		for k=1,#devices do
			info = getAVINinfo(devices[k])
			if info then
				ret[tostring(k)] = info
			end
		end
	end
	return ret		
end


function _M.getEthInfo()
    local proxy = CO.GetProxy( "/", "networkmanager" )
    local cfg_proxy = CO.GetProxy( "/config/nm", "networkmanager" )
    if not proxy or not cfg_proxy then
        _dbg.Error( "GETINFO","无法连接网络管理服务" )
        return nil
    end

    local r
    local eth0, eth1
    r, eth0 = proxy.GetLinkDetail("eth0")
    if not r then
        _dbg.Error( "GETINFO", "无法获取网卡1的配置" )
        return nil
    end

    r, eth1 = proxy.GetLinkDetail("eth1")
    if not r then
        _dbg.Error( "GETINFO", "无法获取网卡2的配置" )
        return nil
    end

    proxy:Destroy()

    local aliases = cfg_proxy._P("aliases")
    cfg_proxy:Destroy()

    local alias_eth0 = aliases and aliases.eth0 or {}
    local alias_eth1 = aliases and aliases.eth1 or {}

    -- _dbg.Info("GETINFO", "eth0 stats="..eth0.status .. " ip="..eth0.address) 
    -- _dbg.Info("GETINFO", "eth1 stats="..eth1.status .. " ip="..eth1.address) 
    return {
        eth0_status = eth0.status,
        eth0_ip = eth0.address,
        eth1_status = eth1.status,
        eth1_ip = eth1.address
    }
end

-- _M.getaddress()

function _M.getOutputInfo()
	local proxy = CO.GetProxy( "/output", "codec" )
	if not proxy then
		_dbg.Error( "GETINFO", "无法连接系统服务" )
		return nil
	else
		local r, status = proxy.GetStatus()
		proxy:Destroy()

		if not r or not status then
			_dbg.Error( "GETINFO", "无法获取输出状态" )
			return nil
		end

		local result = {
			layout_id = status.layout or "4",
			output = status.started and "on" or "off",
			audio_digital_gain = 8, --TODO:
			audio_analog_gain = 8,
			resolution = status.resolution or "1080P30",
			sampling = 48000, --TODO:
			layout = {}
		}

		if status.channels then
			proxy = CO.GetProxy( "/streams", "codec" )
			if not proxy then
				_dbg.Error( "OUTPUT", "Warning: Fail to get codec:/streams proxy so could not get stream details!" )
			end
			local i, item
			for i=1, #status.channels do
				item = {
					id = status.channels[i].id,
					stream_id = status.channels[i].stream,
					status = status.channels[i].bind and "on" or "off",
					stream_name = "未知",
					volume = "off",
					src = "UNKNOWN",
					bitrate = 0,
					buffer = "live:200:200:500:0:0"
				}

				if proxy then
					local detail
					r, detail = proxy.GetStreamDetail( status.channels[i].stream )
					if r and detail then
						item.buffer = detail.buffer_option or item.buffer_option
						item.stream_name = detail.name
						item.type = detail.type --Even no used.
						item.src = detail.address or "UNKNOWN"
						item.volume = detail.audio_enabled and "on" or "off"
						if detail.stream_info then
							item.bitrate = detail.stream_info.bitrate or 0
							if detail.stream_info.live == "connected" then
								if detail.stream_info.stream_start and detail.stream_info.output_start then
									item.status = "view_pull"
								elseif detail.stream_info.stream_start then
									item.status = "pull"
								elseif detail.stream_info.output_start then
									item.status = "view"
								end
							else
								if detail.stream_info.live ~= "closed" and detail.stream_info.live ~= "" then
									item.status = detail.stream_info.live
								end
							end
						end
					end
				end

				table.insert( result.layout, item )
			end

			if proxy then
				proxy:Destroy()
			end
		end

		return result
	end
end

function _M.getPlatform()
	local data = {
		address = "------",
		port = 0000,
		status = false
	}

	local proxy = CO.GetProxy( "/", "CoManager" )
    if not proxy then
    	_dbg.Error( "无法连接CoManager服务！" )
    else
		local r,ret = proxy.ScreenInfo()
		proxy:Destroy()
		if not r or not ret then
			_dbg.Error( "查询平台信息失败！" )
		else
			if ret then
				data.address = ret.address or "------"
				data.port = ret.port or 14000

				if(ret.status == "ready" or ret.status == "online" ) then
					data.status = true
				end
			else
				_dbg.Error( "GETINFO", "无法获取平台地址" )
				return nil
			end
		end
	end	
	return data
end

local function getCodec(red, args)
	local proxy = CO.GetProxy( "/config/codec", "codec" )
	if not proxy then
		_dbg.Error( "GETINFO","无法连接编码服务" )
		return nil
	end

	local configs = proxy._P( "mainStream" )
	proxy:Destroy()

	if not configs then
		_dbg.Error( "GETINFO","无法获取视频编码配置参数" )
		return nil
	end

	local retData={}

	local i
	for i=1, #configs do
		if not args or not args.id or tonumber(args.id) == i then
			local item = {}
			item.name = "暂不支持"
			item.audioSrc = configs[i].audio.source
			item.scale_mode = configs[i].video.scale_mode
			item.grey = configs[i].video.grey
			item.profile = configs[i].video.profile
			item.brCtrl = configs[i].video.brCtrl
			item.bitrate = configs[i].video.bitrate
			item.framerate = configs[i].video.framerate
			item.gop = configs[i].video.gop
			item.refCount = configs[i].video.refCount
			table.insert( retData, item )
		end
	end

	return retData
end


local function getSource()
	local proxy = CO.GetProxy( "/streams", "codec" )
	if not proxy then
		_dbg.Error( "GETINFO", "无法连接系统服务" )
		return nil
	else
		local r, streams = proxy.GetStreams()
		proxy:Destroy()

		if not r or not streams then
			_dbg.Error( "GETINFO", "无法获取媒体源列表" .. (streams or "ERROR") )
			return nil
		end

		local result = {
		}

		local i, item
		for i=1, #streams do
			if not args or not args.id or args.id == streams[i].id then
				item = {
					id = streams[i].id,
					name = streams[i].name,
					type = streams[i].type,
					src = streams[i].address,
					status = streams[i].type == "manual" and "off" or "on",
					resolution = "分辨率未知",
					name_position = "left_top", --TODO
					time_position = "left_top",
					manual_url = streams[i].manual_url or "",
					buffer = streams[i].buffer_option or "live:200:200:500:0:0",
					bitrate = 0
				}

				if streams[i].stream_info then
					if streams[i].stream_info.live == "connected" then
						if streams[i].stream_info.stream_start and streams[i].stream_info.output_start then
							item.status = "view_pull"
						elseif streams[i].stream_info.stream_start then
							item.status = "pull"
						elseif streams[i].stream_info.output_start then
							item.status = "view"
						else
							item.status = "on"
						end
					else
						if (streams[i].type == "manual" and streams[i].stream_info.live and streams[i].stream_info.live ~= "") or (streams[i].stream_info.live ~= "closed" and streams[i].stream_info.live ~= "") then
							item.status = streams[i].stream_info.live
						end
					end

					item.resolution = streams[i].stream_info.resolution or "分辨率未知"
					item.bitrate = streams[i].stream_info.bitrate or 0
				end

				table.insert( result, item )
			end
		end

		return result
	end
end


function _M.eth()
	result = {
		eth0stat=mo_icon.error,
		eth1stat=mo_icon.error,
		eth0address="-.-.-.-",
		eth1address="-.-.-.-", 
	}

	local netinfo = _M.getEthInfo()
	if (netinfo) then
		if (netinfo.eth0_status=="up") then
			eth0stat = mo_icon.ok
		end
		if (netinfo.eth1_status=="up") then
			eth1stat = mo_icon.ok
		end

		if (netinfo.eth0_ip) then
			eth0address = netinfo.eth0_ip
		end

		if (netinfo.eth1_ip) then
			eth1address = netinfo.eth1_ip
		end
	end
	return result
end



function _M.getSATA( diskid )
	if (not diskid) then 
		diskid = 1
	end

	result = {
		stat=0,
		disksize=""
	}

    local proxy = CO.GetProxy( "/diskmanage", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data,err = proxy.get_disk_info()
		proxy:Destroy()
		if not r or not Data then
			_dbg.Error( "OT", "查询磁盘状态失败！err:"..err )
		else
			-- pt.dump(Data)
			if ( Data[diskid].online ) then
				if(Data[diskid].specific_status)then
					result.stat = Data[diskid].specific_status
				end				
				if ( Data[diskid].used and Data[diskid].total ) then
					result.disksize=Data[diskid].used .. "G/"..Data[diskid].total .. "G"
				end
			end
		end
	end	
	return result
end


function _M.initDisk( diskid )
	if (not diskid) then 
		diskid = 1
	end

	result = false

    local proxy = CO.GetProxy( "/diskmanage", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data,err = proxy.InitDisk(diskid)
		proxy:Destroy()
		if not r or not Data then
			_dbg.Error( "OT", "初始化失败！err:"..err )
		else
			if Data then
				result=Data
			end
		end
	end	
	return result
end




function _M.RemoveDisk( diskid )
	if (not diskid) then 
		diskid = 1
	end

	result = false

    local proxy = CO.GetProxy( "/diskmanage", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data,err = proxy.remove_disk(diskid)
		proxy:Destroy()
		if not r or not Data then
			_dbg.Error( "OT", "弹出硬盘失败！err:"..err )
		else
			if Data then
				result=Data
			end
		end
	end	
	return result
end


function _M.GetRecordStatus()
	result = {
		status = false,
		disk = 0 --无磁盘
	}

    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data1,Data2 = proxy.get_record_status()
		proxy:Destroy()
		-- pt.dump(Data1)
		-- pt.dump(Data2)
		if not r then
			_dbg.Error( "OT", "获取录像状态失败！" )
		else
			result.status = Data1
			if Data2 then
				result.disk = Data2
			end			
		end
	end	
	return result
end

function _M.GetRecordTimeFuture()
	result = nil

    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data = proxy.get_record_time_future()
		proxy:Destroy()
		-- pt.dump(Data1)
		-- pt.dump(Data2)
		if not r or not Data then
			_dbg.Error( "OT", "获取录像时长失败！" )
		else
			result = Data
		end
	end	
	return result
end


function _M.StartRecord()
	local result = false

    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data,err = proxy.StartRecord()
		proxy:Destroy()
		if not r then
			result = false
			_dbg.Error( "OT", "录像服务连接失败！" )
		else
			if Data then
				result = Data 
			end
		end
	end	
	return result
end

function _M.StopRecord()
	local result = false
    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data,err = proxy.StopRecord()
		proxy:Destroy()
		if not r then
			result = false
			_dbg.Error( "OT", "停止录像失败！err:", err )
		else
			if Data then
				result = Data
			end
		end
	end	
	return result
end

function _M.Set_Record_disk( diskid )
	if (not diskid) then 
		diskid = 1
	end

	local result = false

    local proxy = CO.GetProxy( "/record", "codec" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data,err = proxy.set_record_disk(diskid)
		proxy:Destroy()
		_dbg.Error( "OT","设置录像",Data)
		if not r or not Data then
			_dbg.Error( "OT",  "设置录像硬盘失败！")
		else
			if Data then
				result=Data
			end
		end
	end	
	return result
end

function _M.GetManageInfo()
	result = {
		info = "--.--.--.--:----",
		status = "off"
	}

    local proxy = CO.GetProxy( "/", "CoManager" )
    if not proxy then
    	_dbg.Error( "OT", "无法连接系统服务！" )
    else
		local r,Data,err = proxy.ScreenInfo()
		proxy:Destroy()
		-- pt.dump(Data)
		if not r or not Data then
			_dbg.Error( "OT", "设置录像硬盘失败！err:"..err )
		else
			if Data then
				result.info=Data.MASTER_ADDR..":"..Data.MASTER_PORT
				result.status=Data.status
			end
		end
	end	
	return result
end




function _M.GetNetworkInfo()
	local proxy = CO.GetProxy( "/", "networkmanager" )
	local cfg_proxy = CO.GetProxy( "/config/nm", "networkmanager" )
	if not proxy or not cfg_proxy then
		_dbg.Error( "OT", "无法连接网络管理服务" )
		return true
	end

	local r
	local eth0, eth1
	r, eth0 = proxy.GetLinkDetail("eth0")
	if not r then
		_dbg.Error( "OT", "无法获取网卡1的配置" )
		return nil
	end

	r, eth1 = proxy.GetLinkDetail("eth1")
	if not r then
		_dbg.Error( "OT", "无法获取网卡2的配置" )
		return nil
	end

	proxy:Destroy()

	local aliases = cfg_proxy._P("aliases")
	cfg_proxy:Destroy()

	local alias_eth0 = aliases and aliases.eth0 or {}
	local alias_eth1 = aliases and aliases.eth1 or {}
	local retData={
		{
			device = "eth0",
			state = eth0.status,
			ip = eth0.address,
			mask = eth0.netmask,
			mac = eth0.mac,
			gw = eth0.gw,
			dynamic = eth0.method == "dhcp" and "y" or "n",
			dns = eth0.dns and eth0.dns[1] or "",

			ip2 = alias_eth0.address or "0.0.0.0",
			mask2 = alias_eth0.netmask or "255.255.255.0",
			gw2 = alias_eth0.gw or "",

			ip3 = alias_eth0.address2 or "0.0.0.0",
			mask3 = alias_eth0.netmask2 or "255.255.255.0",
			gw3 = alias_eth0.gw2 or "",
			enable = alias_eth0.enable and 1 or 0,
		},
		{
			device = "eth1",
			state = eth1.status,
			ip = eth1.address,
			mask = eth1.netmask,
			mac = eth1.mac,
			gw = eth1.gw,
			dynamic = eth1.method == "dhcp" and "y" or "n",
			dns = eth1.dns and eth1.dns[1] or "",

			ip2 = alias_eth1.address or "0.0.0.0",
			mask2 = alias_eth1.netmask or "255.255.255.0",
			gw2 = alias_eth1.gw or "",

			ip3 = alias_eth1.address2 or "0.0.0.0",
			mask3 = alias_eth1.netmask2 or "255.255.255.0",
			gw3 = alias_eth1.gw2 or "",
			enable = alias_eth1.enable and 1 or 0,
		}
	}
	return retData
end


function _M.RestoreIP2()
	local networkinfo = _M.GetNetworkInfo()
	if networkinfo then
		local i,ethinfo
		local sets = {}
		local ip2=""
		for i=1,#networkinfo do
			ethinfo = networkinfo[i]
		
			local ip2
			if (ethinfo.device == "eth0") then
				ip2 = "192.168.1.168"
			elseif (ethinfo.device == "eth1") then
				ip2 = "192.168.2.168"
			end
		
			sets["ethernets." .. ethinfo.device .. ".address"] = ethinfo.ip
			sets["ethernets." .. ethinfo.device .. ".netmask"] = ethinfo.mask
			sets["ethernets." .. ethinfo.device .. ".mac"] = ethinfo.mac
			sets["ethernets." .. ethinfo.device .. ".gw"] = ethinfo.gw
			sets["ethernets." .. ethinfo.device .. ".method"] = ethinfo.dynamic and (ethinfo.dynamic == "y" and "dhcp" or "static") or nil
			sets["ethernets." .. ethinfo.device .. ".dns[1]" ] = ethinfo.dns
		
			sets["aliases." .. ethinfo.device .. ".enable"] = 1
		
			sets["aliases." .. ethinfo.device .. ".address" ] = ip2
			sets["aliases." .. ethinfo.device .. ".netmask" ] = "255.255.255.0"
			sets["aliases." .. ethinfo.device .. ".gw" ] = ""
			sets["aliases." .. ethinfo.device .. ".address2" ] = ethinfo.ip3
			sets["aliases." .. ethinfo.device .. ".netmask2" ] = ethinfo.mask3
			sets["aliases." .. ethinfo.device .. ".gw2" ] = ethinfo.gw3
		end

		local proxy = CO.GetProxy( "/config/nm", "networkmanager" )
		if not proxy then
			_dbg.Error("OT", "访问网络模块异常" )
			return false,"Failed"
		end

		local r,rr = proxy:P_SET(sets)
		if not r or not rr then
			_dbg.Error("OT", "修改网卡配置参数失败" )
			return false,"Failed"
		end

		return true,"Successed"
	else
		return false,"Failed"
	end	
end

function _M.SetPub( enable )
	local streamernum = 8	
	local devices = getDevices()
	if devices then
		streamernum = #devices * 2
	end	

	local proxy = CO.GetProxy( "/streamer", "codec" )
	if not proxy then
		_dbg.Error("OT", "无法连接系统服务" )
		return false,"无法连接系统服务"
	end
	local r,pub = proxy.GetPublishPoints()
	if not r or not pub then
		proxy:Destroy()
		_dbg.Error("OT", "获取发布点失败" )
		return false,"获取发布点失败"
	else
		local k,v
		for k,v in pairs(pub) do
			if tonumber(v.id) >  streamernum then
				proxy:ChangeObject( v.objPath )
				local r, streams = proxy:GetStreams()
				if not r or not streams then
					proxy:Destroy()
					_dbg.Error("OT", "获取流服务失败" )
					return false,"获取流服务失败"
				else
					local i
					for i=1, #streams do
						if enable then
							proxy:EnableStream( streams[i].id )
						else
							proxy:DisableStream( streams[i].id )
						end
					end
				end
			end
		end
	end
	proxy:Destroy()
	return true
end

function _M.GetPub()
	local streamernum = 8	
	-- local devices = getDevices()
	-- if devices then
	-- 	streamernum = #devices * 2
	-- end
		
	local count = 0
	local enable_count = 0

	local proxy = CO.GetProxy( "/streamer", "codec" )
	if not proxy then
		_dbg.Error("OT", "无法连接系统服务" )
		return 0,0
	end
	local r,pub = proxy.GetPublishPoints()
	if not r or not pub then
		proxy:Destroy()
		_dbg.Error("OT", "获取发布点失败" )
		return 0,0
	else
		local k,v
		for k,v in pairs(pub) do
			if tonumber(v.id) >  streamernum then
				proxy:ChangeObject( v.objPath )
				local r, streams = proxy:GetStreams()
				if not r or not streams then
					proxy:Destroy()
					_dbg.Error("OT", "获取流服务失败" )
					return 0,0
				else
					local i
					for i=1, #streams do
						count = count +1 
						if streams[i].enable then
							enable_count = enable_count +1 
						end
					end
				end
			end
		end
	end
	proxy:Destroy()
	return enable_count, count
end

-- pt.dump(_M.getSATA(2))
-- pt.dump(_M.GetManageInfo())
-- pt.dump(_M.getSATA(2))

-- local a,b
-- a,b = _M.RestoreIP2()
-- print (a,b)

return _M

