#!/usr/bin/env lua
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"
local pt = require "pl.pretty"
local thread = require "coaf.thread"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local event = require "coaf.event"
local mo_Oled = require "mod_Oled"

local mo_TouchPad = require "mod_TouchPad"
local mo_Pages = require "mod_Pages"
local mo_OPage = require "mod_OledPage"
local mo_IconPage = require "mod_OledIconPage"
local mo_info = require "mod_GetInfo"


local NOEVENTBACKTIME = 15
local listpage = nil
local pagenow = nil
local nopresscount = 0

local checkreccount = 0
local recodingpage = false

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

local flag_oled_updating

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/oled_touch.log")
	if not logF or not logF:init() then
		_dbg.Error("OT", "Fail to open log file: /var/log/oled_touch.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "oledtouch_service" )

if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "oledtouch" )
	stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )


_G.g_event = event.createNew( "oledtouch" )

-- ** CoService **
local cosvc_OledTouch
------------------

local level =1
local list = {}
local index = 1
local upList = {}
local upList_index = {}
local interactionpage = nil

local iconwindowindex = 1
local iconselectindex = 1

local BeepCount = 0

function showPage()
	if not list then
		_dbg.Error("OT", "list empty !!!")
		return
	end

	local page = list[index]

	if page.type and page.type =="iconpage" then
		mo_Oled.OLED_Clear()
		pagenow = mo_IconPage.createNew(page.icons,iconwindowindex,iconselectindex)
	else
		pagenow = mo_OPage.createNew(page.pagedata)
	end
    pagenow:DrawPage()
end

local function DrawLogo()
	list = mo_Pages.LIST
	index = 1

	local recordinfo = mo_info.GetRecordStatus()
	if (recordinfo and recordinfo.status) then
		pagenow = mo_OPage.createNew(mo_Pages.RECSTAT_LOGO)
		pagenow:DrawPage()
		recodingpage = true
	else
		local page = list[index]
		pagenow = mo_OPage.createNew(page.pagedata)	
		pagenow:DrawPage()
	end
end

function pageChange( arg )
	if #list == 1 then
		return
	end

    -- 上下翻页
	if (arg == "down") then
        index = index + 1
    end

    if (arg == "up") then
		index = index - 1
	end

	if index > #list then 
		index = 1
	end

	if index < 1 then 
		index = #list
	end
    showPage()
end


function Enter()
    if (list[index].subpages) then 
        table.insert(upList,list)
		table.insert(upList_index,index)  
		
        level = level + 1  
        list = list[index].subpages			
		index = 1
        showPage()
	end
	
	if (list[index].interaction) then 
		_dbg.Info("ListPage", "have interaction!!!")	
		table.insert(upList,list)
		table.insert(upList_index,index)  
		interactionpage = list[index]
		
        pagenow = mo_OPage.createNew(list[index].interaction.pagedata)	
    	pagenow:DrawPage() 	
        level = level + 1  				
	end
end



function Back()
	if pagenow.oppage == true then
		_dbg.Info("ListPage", "interactionpage back")
		pagenow = mo_OPage.createNew(interactionpage.interaction.pagedata)	
		pagenow:DrawPage()
		return
	end
	 

    if level > 1 then       
        list = upList[level-1]
        index = upList_index[level-1]  
        table.remove( upList, level-1 )       
		table.remove( upList_index, level-1 )
		
		if (list == mo_Pages.LIST and index == 1) then
			DrawLogo()
		else
			showPage()
		end
		level = level - 1
	end
	
	-- icon index 恢复默认1
	if level == 1 then
		iconwindowindex = 1
		iconselectindex = 1
	end
end

local function keypress( thrObj, kp )
	if not flag_oled_updating then
		flag_oled_updating = true
		-- local keydown = false
		if (kp[1]) then
			-- keydown = true
			if pagenow.type and pagenow.type == "iconpage" then
				pagenow:Pagedown()
			else
				if (pagenow.f1) then
					if type(pagenow.f1) == "table" then
						pagenow  = mo_OPage.createNew(pagenow.f1)	
						pagenow:DrawPage()
					end
					if type(pagenow.f2) == "function" then
						pagenow:f1()
					end
				else
					pageChange("up")
				end
			end
		end

		if (kp[2])  then 
			-- keydown = true
			if pagenow.type and pagenow.type == "iconpage" then
				pagenow:Pageup()
			else
				if (pagenow.f2) then
					if type(pagenow.f2) == "table" then
						pagenow  = mo_OPage.createNew(pagenow.f2)	
						pagenow:DrawPage()
					end
					if type(pagenow.f2) == "function" then
						pagenow:f2()
					end
				else
					pageChange("down")
				end
			end
		end

		if (kp[3]) then 
			-- keydown = true
			if pagenow.type and pagenow.type == "iconpage" then
				if (pagenow:Getpages()) then 
					table.insert(upList,list)
					table.insert(upList_index,index)  				
					level = level + 1  
					list = pagenow:Getpages()		
					index = 1
					iconwindowindex,iconselectindex = pagenow:Getindex()	
					showPage()
				end
			else
				if (pagenow.f3) then
					if type(pagenow.f3) == "table" then
						pagenow  = mo_OPage.createNew(pagenow.f3)	
						pagenow:DrawPage()
					end
					if type(pagenow.f3) == "function" then
						pagenow:f3()
					end

				else
					Enter()
				end
			end
		end

		if (kp[4])  then 
			-- keydown = true
			if (pagenow.f4) then
				if type(pagenow.f4) == "table" then
					pagenow  = mo_OPage.createNew(pagenow.f4)	
					pagenow:DrawPage()
				end
				if type(pagenow.f4) == "function" then
					pagenow:f4()
				end
			else
				Back()
			end
		end

		flag_oled_updating = false
	end
end

local function for10ms_DetectPress()

	if ( pagenow == nil) then
		return
	end

	local kp = mo_TouchPad.DetectPress()
	-- local keydown = false

	if kp[1] or kp[2] or kp[3] or kp[4] then
		mo_TouchPad.Beep()
		local task = thread.New(keypress)
		if task then
			task:Start(kp)
		end
		nopresscount = 0
	end

end




local function for1s_BackDisplayLogo()
	nopresscount = nopresscount + 1
	if( nopresscount > NOEVENTBACKTIME ) then	--15秒无操作返回显示Logo
		if list ~= mo_Pages.LIST or index~=1 then 
			DrawLogo()
		end
		nopresscount = 0
		iconwindowindex = 1
		iconselectindex = 1
	end

	checkreccount = checkreccount + 1
	if( checkreccount > 10 ) then				--10秒检查一次录像状态
		local recordinfo = mo_info.GetRecordStatus()		

		if (list == mo_Pages.LIST and index == 1 ) then
			if (recordinfo and recordinfo.status ) then
				if(type(pagenow.getinfo) == "function" and recodingpage) then
					pagenow:getinfo(pagenow)
					mo_Oled.OLED_ClearAera(4,85,128)
					mo_Oled.OLED_ClearAera(5,85,128)
					mo_Oled.OLED_ClearAera(6,69,128)
					mo_Oled.OLED_ClearAera(7,69,128)
					pagenow:DrawText()
				else
					pagenow = mo_OPage.createNew(mo_Pages.RECSTAT_LOGO)	
					pagenow:DrawPage()
					recodingpage = true
				end				
			else
				local page = list[index]
				pagenow = mo_OPage.createNew(page.pagedata)
				pagenow:DrawPage()
				recodingpage = false
			end
		end		
		checkreccount = 0
	end
end


-- ** Timers **
local for10msTimer
local for1sTimer

-- **Main entry**
local function Main(args)
	

	mo_Oled.OLED_Init()
	mo_Oled.OLED_Clear()
	mo_Oled.OLED_Display_On()

	list = mo_Pages.LIST
	index = 1

	-- list = mo_Pages.LIST
	-- index = 1 
	-- local page = list[index]
    -- pagenow = mo_OPage.createNew(page.pagedata)	
	-- pagenow:DrawPage()
	DrawLogo()

	mo_TouchPad.init()

	for10msTimer = coaf.NewTimer( for10ms_DetectPress, 0.01, 0.01 )
	if for10msTimer then
		for10msTimer:start()
	end


	for1sTimer = coaf.NewTimer( for1s_BackDisplayLogo, 1, 1 )
	if for1sTimer then
		for1sTimer:start()
	end
end

coaf.Start(Main, arg)


if for10msTimer then
	for10msTimer:stop()
end

if for1sTimer then
	for1sTimer:stop()
end

mo_Oled.OLED_Clear()
mo_Oled.OLED_Display_Off()

event.free( "oledtouch" )

_dbg.Print( "OT", "Service terminated." )

