local coaf = require 'coaf.core'
local _dbg = require 'coaf.debug'
local client = require 'coaf.client'
require 'himpp'
require 'osd2'
local mo_OSD_adap = require 'mod_OSD_adapter'
local CO = require 'mod_coservice'
local pt = require 'pl.pretty'
local vscreen = require 'textScreen'

local _M = {
    -- hdmi = nil,
    -- sdi = nil,
    fbosds = {},
    picInfo = nil,
    vs = vscreen.New()
}

_M.vs:Load()

----CoAF-----------------
local function GetHomePicInfos()
    local proxy = CO.GetProxy('/relation', 'picManager')
    if not proxy then
        _dbg.Error('IMGO', '无法连接图片服务！')
        return nil
    else
        local r, Data = proxy.GetHomePicInfos()
        proxy:Destroy()
        if not r or not Data then
            _dbg.Error('IMGO', '获取图片信息失败！')
            return nil
        else
            return Data
        end
    end
    return nil
end

local function GetScreenResolution(id)
    local proxy = CO.GetProxy('/output/hd' .. (id - 1), 'codec')
    if not proxy then
        _dbg.Error('IMGO', 'Connect output service error', id)
        return nil, nil
    else
        local r, width, height = proxy.GetResolution()
        proxy:Destroy()
        if (not r) or (not width) or (not height) then
            _dbg.Error('IMGO', '无法获取输出状态！')
            return nil, nil
        else
            return width, height
        end
    end
    return nil, nil
end

local function GetLayoutInfo(id)
    local proxy = CO.GetProxy('/output/hd' .. (id - 1), 'codec')
    if not proxy then
        _dbg.Error('IMGO', 'Connect output service error', id)
        return nil
    else
        local r, Data = proxy.GetOutputLayout()
        proxy:Destroy()
        if not r or not Data then
            _dbg.Error('IMGO', '无法获取Layout状态！')
            return nil
        else
            return Data
        end
    end
    return nil
end

local function convertImg(src, dst)
    local proxy = CO.GetProxy('/pics', 'picManager')
    if not proxy then
        _dbg.Error('IMGO', 'Connect picManager service error')
        return false
    else
        local r, Data = proxy.ConvertPic(src, dst)
        proxy:Destroy()
        if not r or not Data then
            _dbg.Error('IMGO', '转换图片失败！')
            return false
        else
            return Data
        end
    end
    return false
end

local function getimgSize(imgpath)
    local proxy = CO.GetProxy('/pics', 'picManager')
    if not proxy then
        _dbg.Error('IMGO', 'Connect picManager service error')
        return 0, 0
    else
        local r, size_w, size_h = proxy.GetPicRate(imgpath)
        proxy:Destroy()
        if not r or not size_w or not size_h then
            _dbg.Error('IMGO', '获取图片尺寸失败！')
            return 0, 0
        else
            return size_w, size_h
        end
    end
    return 0, 0
end
----CoAF END--------------




local function isArrayTable(t)
    if type(t) ~= 'table' then
        return false
    end

    local n = #t
    for i, v in pairs(t) do
        if type(i) ~= 'number' then
            return false
        end

        if i > n then
            return false
        end
    end

    return true
end

local pagemeta = {
    items = {}
}

function pagemeta:Init()
    if not self.width or not self.height then
        local width, height = GetScreenResolution(self.id)
        if not width or not height then
            _dbg.Error('IMGO', 'GetScreenResolution err', self.id)
            return
        else
            _dbg.Info('IMGO', 'GetScreenResolution ok', width, height)
        end

        self.width = width
        self.height = height
    end
    -- self.items = {} --文字OSD
    if not self.fb then
        local fb = mo_OSD_adap.createNew(
            'fb',
            {
                device = self.device,
                width = self.width,
                height = self.height
            }
        )
        if not fb then
            _dbg.Error('IMGO', 'Fail to create FB osd instance for channel !', self.id, self.width, self.height)
            return
        else
            _dbg.Info('IMGO', 'Created FB osd instance for channel !', self.width, self.height)
        end
        self.fb = fb

        -- pt.dump(self)
        -- pt.dump(_M.picInfo)
        if _M.picInfo then
            self:InitOSD(_M.picInfo, self.id)
        end
    end
end

function pagemeta:InitOSD(picinfo)
    for i, pic in pairs(picinfo) do
        if pic.output == self.id then
            if pic.type == 'osd' and pic.url then
                _dbg.Info('OSD', self.id, ' osd=' .. pic.url)
                self:setLogo(pic.url, pic.position)
            end

            _dbg.Info('OSD', 'enable', pic.type, pic.enable)
            self:enable(pic.type, pic.enable)
        end
    end
    self:drawOSD()
end

function pagemeta:updateLogoPos()
    if ((not self.logo_h) or (not self.logo_w)) then
        _dbg.Error('IMGO', 'logo size error')
        return
    end

    self.logo_x = 0
    self.logo_y = 0

    if (self.logpos == 'left_top') then
        self.logo_x = 16
        self.logo_y = 16
    end

    if (self.logpos == 'left_bottom') then
        self.logo_x = 16
        self.logo_y = self.height - self.logo_h - 16
    end

    if (self.logpos == 'right_top') then
        self.logo_x = self.width - self.logo_w - 16
        self.logo_y = 16
    end

    if (self.logpos == 'right_bottom') then
        self.logo_x = self.width - self.logo_w - 16
        self.logo_y = self.height - self.logo_h - 16
    end

    if self.logo_w == self.width and self.logo_h == self.height then
        self.logo_x = 0
        self.logo_y = 0
    end
end

function pagemeta:setLogo(imgpath, pos)
    if (imgpath == '') then --unselect bg
        self.logo = nil
        return true
    end

    _dbg.Info('IMGO', 'setLogo=' .. imgpath)
    self.logpos = pos

    --转换格式
    if not convertImg('/data/storage/img/' .. imgpath, '/tmp/' .. self.outputid .. 'logo.rgba') then
        _dbg.Error('IMGO', 'convert Logo error')
        return false
    end

    self.logo = '/tmp/' .. self.outputid .. 'logo.rgba'
    --获取图片尺寸
    self.logo_w, self.logo_h = getimgSize(imgpath)

    if (self.logo_w and self.logo_h) then
        -- if (self.logo_w > self.screenw or self.logo_h > self.screenh ) then
        --     _dbg.Error('IMGO', 'img w h=', self.logo_w, self.logo_h, " too big!!!")
        --     return false
        -- end
        _dbg.Info('IMGO', 'img w h=', self.logo_w, self.logo_h)
        self:updateLogoPos()
    else
        _dbg.Info('IMGO', '获取图片尺寸失败')
    end

    -- self:drawOSD()
    return true
end

function pagemeta:setHole(layout)
    _dbg.Info('IMGO', 'setHole layout=......')
    -- pt.dump(layout)
    if (isArrayTable(layout)) then
        self.holes = layout
        return true
    else
        return false
    end
end

function pagemeta:UpdateTextArea(areaId, glbCfg, itemCfg)
    if self.fb then
        self.fb:DropArea(areaId)
        if glbCfg.enabled == 0 then
            return
        end
        local dyn_startline, dyn_lines
        local text
        local osd_type = itemCfg.type or itemCfg.osd_type
        if not osd_type or osd_type == 'none' or osd_type == '' then
            self.items[areaId] = nil
            self.fb:DropArea(areaId)
            return
        elseif osd_type == 'time' then
            text = '%Y/%m/%d %H:%M:%S'
        elseif osd_type == 'date' then
            text = '%Y/%m/%d'
        elseif osd_type == 'clock' then
            text = '%H:%M:%S'
        else
            if osd_type == 'dynamic' then
                dyn_startline, dyn_lines, text = self.vs:GetScreen()
            else
                text = itemCfg.text
            end
        end
        -- print("UpdateTextArea", areaId, osd_type, text)
        self.fb:ChangeAreaType(areaId, osd_type)

        if not text or text == '' then
            self.fb:EnableArea(areaId, false)
            self.items[areaId] = nil
            return
        else
            self.fb:EnableArea(areaId, true)
        end

        local screenW = glbCfg.gridW or 40
        local screenH = glbCfg.gridH or 25

        --Don't be too small
        if screenW < 16 then
            screenW = 16
        elseif screenW > 80 then
            screenW = 80
        end

        if screenH < 9 then
            screenH = 9
        elseif screenH > 45 then
            screenH = 45
        end

        local screenGridW, screenGridH
        screenGridW = math.floor( self.width/screenW )
        screenGridH = math.floor( self.height/screenH )
        local thisAreaW, thisAreaH
        local xOffset, yOffset
        local imageFile
        local font, fontSize
        --Now, check/calculate font size.
        if dyn_startline then
            xOffset = math.floor( glbCfg.gridShiftX * screenGridW )
            yOffset = math.floor( glbCfg.gridShiftY * screenGridH + (dyn_startline-1) * screenGridH )
        else
            xOffset = math.floor( glbCfg.gridShiftX * screenGridW )
            yOffset = math.floor( glbCfg.gridShiftY * screenGridH )
        end
        fontSize = screenGridW
        if screenGridW > screenGridH then
            fontSize = screenGridH
        end

        if itemCfg.outline and tonumber(itemCfg.outline) then
            screenGridH = screenGridH + 2 * tonumber(itemCfg.outline)
        end
        
        -- font = _M.FontMap[fntname] or _M.DefaultFontIndex or     0
        font = 0
        -- pt.dump({ font = font, size=fontSize, bold=itemCfg.bold or 0, italic=itemCfg.italic or 0, outline=itemCfg.outline , screenGridH=screenGridH})
        self.fb:SetAreaFont( areaId, { font = font, size=fontSize, bold=itemCfg.bold or 0, italic=itemCfg.italic or 0, outline=itemCfg.outline } )
        self.fb:SetAreaFontColor( areaId, itemCfg.color )
        self.fb:SetAreaOutlineColor( areaId, itemCfg.outlineColor )
        self.fb:SetAreaLineHeight( areaId, screenGridH )
        self.fb:SetAreaText( areaId, text )
        --Calculate lines and area height, according to screen grid height (but not text rendering size)
        local lineCnt = 0
        local flag
        for flag in text:gmatch("(\n)") do
            lineCnt = lineCnt + 1
        end
        thisAreaW, thisAreaH = self.fb:TellAreaSize( areaId )
        thisAreaH = (lineCnt+1)*screenGridH

        --Should return back restricted size.
        -- print("ChangeAreaSize", areaId, thisAreaW, thisAreaH)
        thisAreaW, thisAreaH = self.fb:ChangeAreaSize( areaId, thisAreaW, thisAreaH )
    
        local x, y
        if itemCfg.position == "left-top" then
            x = xOffset
            y = yOffset
        elseif itemCfg.position == "top-center" then
            x = math.floor((self.width - thisAreaW)/2)
            y = yOffset
        elseif itemCfg.position == "right-top" then
            x = self.width - thisAreaW - xOffset
            y = yOffset
        elseif itemCfg.position == "left-bottom" then
            x = xOffset
            y = self.height - yOffset - thisAreaH
        elseif itemCfg.position == "bottom-center" then
            x = math.floor((self.width - thisAreaW)/2)
            y = self.height - yOffset - thisAreaH
        elseif itemCfg.position == "right-bottom" then
            x = self.width - thisAreaW - xOffset
            y = self.height - yOffset - thisAreaH
        elseif itemCfg.position == "center" then
            x = math.floor((self.width - thisAreaW)/2)
            y = math.floor((self.height - thisAreaH)/2)
        else
            local posX = itemCfg.x or 0 --itemCfg.customX or 0
            local posY = itemCfg.y or 0 --itemCfg.customY or 0
            --x = xOffset + posX          --* screenGridW
            --y = yOffset + posY          --* screenGridH
            -- x = posX
            -- y = posY
            x = xOffset + posX * screenGridW
            y = yOffset + posY * screenGridH
        end

        -- print("SetAreaPositionType", areaId, itemCfg.position)
        self.fb:SetAreaPositionType( areaId, itemCfg.position )

        -- print("SetAreaPosition", areaId, x, y )
        self.fb:SetAreaPosition( areaId, x, y )
    end
end

function pagemeta:drawText(itemId)
    local glbCfg = {
        enabled = _G.osdcfg.output[self.id].enabled,
        gridW = _G.osdcfg.output[self.id].screen_w,
        gridH = _G.osdcfg.output[self.id].screen_h,
        gridShiftX = _G.osdcfg.output[self.id].shiftX,
        gridShiftY = _G.osdcfg.output[self.id].shiftY
    }
    local outputId
    local i
    -- pt.dump(_G.osdcfg.output[self.id])
    -- print("drawText self.id=",self.id)
    for i = 0, 3 do
        if not itemId or itemId == i then
            local itemCfg = _G.osdcfg.output[self.id].items[i]
            if itemCfg then
                self:UpdateTextArea(i + 3, glbCfg, itemCfg)
            end
        end
    end
    if self.fb then
        self.fb:Update()
    end
end

function pagemeta:ChangeSize(width, height)
    self.width = width
    self.height = height

    if not self.fb then
        self:Init()
    end

    if (self.fb) then
        local i
        for i = 2, 6 do
            self.fb:DropArea(i)
        end
        self.fb:Update()

        _dbg.Info('IMGO', 'ChangeSize', width, height)
        self.fb:ChangeSize(width, height)
        self:updateLogoPos()
        self:drawOSD()
    end
end

function pagemeta:enable(type, flag)
    _dbg.Info('IMGO', 'enable type=', type, flag)

    if (flag == 1 or flag == '1' or flag == true) then
        flag = true
    else
        flag = false
    end

    if (self.fb) then
        if (type == 'osd') then
            self.logoenable = flag
        end

        -- self:drawOSD()
        _dbg.Info('IMGO', 'enable end')
    end
    return true
end

local function float_equal(lhs, rhs, epsilon)
    epsilon = epsilon or 1E-12

    local abs = math.abs
    local abslhs = abs(lhs)
    local absrhs = abs(rhs)
    local diff = abs(abslhs - absrhs)

    if (lhs == rhs) then
        return true
    elseif (lhs == 0 or rhs == 0 or diff < 1E-12) then
        return diff < (epsilon * 1E-12)
    else
        -- return diff / (abslhs + absrhs) < epsilon;
        return abs(lhs - rhs) <= epsilon
    end
end

-- print(float_equal(1,0.3333,0.0001))
-- print(float_equal(0.33340,0.3333,0.0001))
-- print(float_equal(0.33339,0.3333,0.0001))
-- print(float_equal(0.66660,0.6666,0.0001))
-- print(float_equal(0.66669,0.6666,0.0001))

function pagemeta:drawOSD()
    if not self.fb then
        return
    end
    _dbg.Info('IMGO', self.id .. ' drawOSD')
    _dbg.Info('IMGO', 'fb device=' .. self.device)

    if (self.logo) then
        _dbg.Info('IMGO', 'DrawOSD Logo~~~~')
        self.fb:ChangeAreaType(2, 'image')
        _dbg.Info('IMGO', 'logoenable', self.logoenable)
        self.fb:EnableArea(2, self.logoenable)
        self.fb:ChangeAreaSize(2, self.logo_w, self.logo_h)
        self.fb:SetAreaPosition(2, self.logo_x, self.logo_y)
        self.fb:SetBackGroundFlag(2, false)
        self.fb:LoadFile(2, {filename = self.logo, w = self.logo_w, h = self.logo_h})
    else
        _dbg.Info('IMGO', 'disable Logo')
        self.fb:EnableArea(2, false)
    end

    self:drawText()
    self.fb:Update()
end

function pagemeta:Destroy()
    if self.fb then
        self.fb:Reset()
        self.fb = nil
    end
    collectgarbage()
end

pagemeta.__index = pagemeta
---------------------------------------------

function _M.createNew(output)
    local osdObj = {}
    osdObj.id = output
    if (output == 1) then
        osdObj.device = '/dev/fb0'
        osdObj.outputid = 'hd0'
    elseif (output == 2) then
        osdObj.device = '/dev/fb1'
        osdObj.outputid = 'hd1'
    end

    if (osdObj.device == nil) then
        return nil
    end

    setmetatable(osdObj, pagemeta)
    osdObj:Init()
    return osdObj
end

function _M.Init()
    _M.picInfo = GetHomePicInfos()
    if not _M.picInfo then
        _dbg.Error('IMGO', 'Fail to Get HomePic Info!')
        _M.picInfo = {}
    end

    local id, layoutinfo
    for id = 1, 2 do
        local osd = _M.createNew(id)
        if osd then
            _M.fbosds['osd'..id] = osd
        end
    end
end

function _M.enableImg(output, type, flag)
    _dbg.Info('IMGO', 'output=' .. output .. ' type=' .. type .. ' flag=' .. flag)
    local enable
    if flag == 0 or flag == '0' then
        enable = false
    else
        enable = true
    end

    local osdid = "osd"..output
    _dbg.Info('IMGO', osdid ..' enable type=' .. type .. ' flag=', enable)
    local osd = _M.fbosds[osdid]
    if osd then
        if (osd:enable(type, enable)) then
            osd:drawOSD()
            return true
        else
            return false
        end
    else
        return false
    end
end

function _M.setLogo(output, img, pos)
    local osdid = "osd"..output
    local osd = _M.fbosds[osdid]
    if osd then
        if (osd:setLogo(img, pos)) then
            osd:drawOSD()
            return true
        else
            return false
        end
    else
        return false
    end
end

function _M.SetHole(output, layoutinfo)
    local osdid
    if (output == 'hd0') then
        osdid = "osd1"
    else
        osdid = "osd2"
    end
    local osd = _M.fbosds[osdid]
    if osd then
        if (osd:setHole(layoutinfo)) then
            _dbg.Info('IMGO', osdid, 'setHole ok  drawOSD')
            osd:drawOSD()
            return true
        else
            return false
        end
    else
        return false
    end
end

function _M.drawOSD()
    for id = 1, 2 do
        local osd = _M.fbosds["osd"..id]
        if osd then
            osd:drawOSD()
        end
    end
    return true
end

function _M.Destroy()
    for id = 1, 2 do
        local osd = _M.fbosds["osd"..id]
        if osd then
            osd:Destroy()
            _M.fbosds["osd"..id] = nil
        end
    end
    return true
end

function _M.ChangeResolution(output, width, height)
    -- print("ChangeResolution", output, width, height)
    local outputid
    if output == "hd0" then
        outputid = "osd1"
    elseif output == "hd1" then
        outputid = "osd2"
    end
    local osd = _M.fbosds[outputid]
    if not osd then
        _dbg.Error('IMGO', outputid, 'osd object is nil!')
        return false
    end
    osd:ChangeSize(width, height)
    return true
end

function _M.GetOsd(idr)
    local outputId = tonumber(idr) or 1
    if outputId < 1 or outputId > 2 then
        return nil, 'Invalid arguments'
    end

    local ret = {}
    local id
    for id = 0, 3 do
        table.insert(
            ret,
            {
                id = id + 1,
                x = _G.osdcfg.output[outputId].items[id].x,
                y = _G.osdcfg.output[outputId].items[id].y,
                bold = _G.osdcfg.output[outputId].items[id].bold,
                ['type'] = _G.osdcfg.output[outputId].items[id].osd_type,
                --font = _G.osdcfg.output[outputId].items[id].font,
                position = _G.osdcfg.output[outputId].items[id].position,
                text = _G.osdcfg.output[outputId].items[id].text,
                color = _G.osdcfg.output[outputId].items[id].color,
                outline = _G.osdcfg.output[outputId].items[id].outline,
                outlineColor = _G.osdcfg.output[outputId].items[id].outlineColor
            }
        )
    end
    return ret
end

function _M.SetOsd(idr, args)
    local outputId = tonumber(idr) or 1
    if type(args) ~= 'table' or outputId < 1 or outputId > 2 then
        return false, 'Invalid arguments'
    end
    local id = tonumber(args.id)
    if not id or id < 1 or id > 4 then
        return false, 'Invalid OSD item id'
    end
    id = id - 1

    if args.x then
        _G.osdcfg.output[outputId].items[id].x = tonumber(args.x)
    end

    if args.y then
        _G.osdcfg.output[outputId].items[id].y = tonumber(args.y)
    end

    if args.bold then
        _G.osdcfg.output[outputId].items[id].bold = tonumber(args.bold)
    end

    if args.type then
        _G.osdcfg.output[outputId].items[id].osd_type = args.type
    end
    --if args.font then
    --_G.osdcfg.output[outputId].items[id].font = args.font
    --end

    if args.position then
        _G.osdcfg.output[outputId].items[id].position = args.position
    end

    if args.text then
        _G.osdcfg.output[outputId].items[id].text = args.text
    end

    if args.color then
        _G.osdcfg.output[outputId].items[id].color = tonumber(args.color)
    end

    if args.outline then
        _G.osdcfg.output[outputId].items[id].outline = tonumber(args.outline)
    end

    if args.outlineColor then
        _G.osdcfg.output[outputId].items[id].outlineColor = tonumber(args.outlineColor)
    end

    --args.size
    --args.imgname

    _G.osdcfg:__TRIGGER(true)
    return true
end

function _M.UpdateTextOSD()
    local  _i, osd
    for _i, osd in pairs(_M.fbosds) do
        osd:drawText()
    end
    return true
end


function _M.GetOsdSetting( idr )
    local outputId = tonumber(idr) or 1
    if outputId < 1 or outputId > 2 then
        return false, 'Invalid arguments'
    end

    return {
        width = _G.osdcfg.output[outputId].screen_w,
        height = _G.osdcfg.output[outputId].screen_h,
        x = _G.osdcfg.output[outputId].shiftX,
        y = _G.osdcfg.output[outputId].shiftY
    }
end


function _M.SetOsdSetting( idr, args )
    local outputId = tonumber(idr) or 1
    if type(args) ~= "table" or outputId < 1 or outputId > 2 then
            return false, "Invalid arguments"
    end
    if args.width then
        _G.osdcfg.output[outputId].screen_w = tonumber(args.width)
    end
    if args.height then
        _G.osdcfg.output[outputId].screen_h = tonumber(args.height)
    end
    if args.x then
        _G.osdcfg.output[outputId].shiftX = tonumber(args.x)
    end
    if args.y then
        _G.osdcfg.output[outputId].shiftY = tonumber(args.y)
    end
    _G.osdcfg:__TRIGGER(true)
    return true
end

function _M.Check()
    for id = 1, 2 do
        local osd = _M.fbosds["osd"..id]
        if osd then
            if not osd.fb then
                osd:Init()
            end
        end
    end
end

return _M
