local _M = {}


function _M.New()
--============ New VScreen ==============
local vs = {
	vScreen = {},
	filename = "/data/configs/txt-buffer.conf"
}

function vs:Load(filename)
	filename = filename or "/data/configs/txt-buffer.conf"
	self.filename = filename
	local f = io.open( filename, "r" )
	if f then
		local cfgtxt = f:read("*a")
		f:close()
		if cfgtxt then
			local func, err = load( "return " .. cfgtxt )
			if func then
				local done,cnt = pcall(func)
				if done and cnt then
					self.vScreen = cnt
				end
			end
		end
	end
end

function vs:Save()
	local pretty = require "pl.pretty"
	local f = io.open( self.filename, "w" )
	if f then
		f:write( pretty.write( self.vScreen ) )
		f:close()
	end
end

function vs:Clear()
	self.vScreen = {}
	self:Save()
end

function vs:AddText( line, pos, str )
	if line < 1 or line > 64 then
		print( "ERROR: Invalid line number ", line )
		return
	end

	local i
	for i=#self.vScreen + 1, line-1 do
		self.vScreen[i] = {}
	end

	if not self.vScreen[line] then
		self.vScreen[line] = {}
	end

	local curLine = self.vScreen[line]

	for i=#curLine + 1, pos-1 do
		curLine[i] = ' '
	end

	local curChar
	local b
	i=1
	while i <= #str do
		b = str:byte(i)
		if (b&0x80) == 0 then
			curLine[pos] = (b==0 and ' ' or string.char(b))
			pos = pos + 1
			i = i + 1
		else
			curChar = string.char(b)
			local n=1
			i = i+1
			while (b&(0x80>>n)) ~= 0 and i <= #str do
				curChar = curChar .. string.char( str:byte(i) )
				i = i+1
				n = n+1
			end
			curLine[pos] = curChar
			pos = pos + 1
		end
	end

	self:Save()
end

function vs:GetScreen()
	local startLine, lines
	local text = ''
	local i, j
	local thisLine

	for i=1, #self.vScreen do
		thisLine = self.vScreen[i]
		if #thisLine > 0 and not startLine then
			startLine = i
			lines = 0
		end

		if startLine then
			lines = lines + 1
			if text ~= '' then
				text = text .. "\n"
			end

			for j=1, #thisLine do
				text = text .. thisLine[j]
			end
		end
	end
	return startLine, lines, text
end

function vs:GetScreenLines()
	local startLine, lines
	local text = {}
	local i, j
	local thisLine

	for i=1, #self.vScreen do
		thisLine = self.vScreen[i]
		if #thisLine > 0 and not startLine then
			startLine = i
			lines = 0
		end

		local lineText = ''
		if startLine then
			lines = lines + 1
			for j=1, #thisLine do
				lineText = lineText .. thisLine[j]
			end
		end
		table.insert( text, lineText )
	end
	return startLine, lines, text
end

return vs
--======== End New VScreen ===========
end

return _M

