local _dbg = require "coaf.debug"
local cjson = require "cjson"
local pl = require "pl.pretty"


local _M = {
    id_seed = 0,
    PICPATH = "/data/storage/img/"
}

function _M.init()
    _dbg.Info("==pic======init")
    _M.id_seed = 0
    local k,v
    for k,v in pairs(cfg_pics ) do
        if v.id > _M.id_seed then
                _M.id_seed = v.id
        end
    end
end


local function IdToUrl(id)
    --根据图片id，获取图片url
    for k,v in pairs(cfg_pics) do
        if tonumber(v.id) == tonumber(id) then
            return v.url
        end
    end
    return nil
end

function _M.AddPic(details)
    local Cfg = cfg_pics
    _M.id_seed = _M.id_seed + 1 
    details.id = _M.id_seed 

    for k,v in pairs(Cfg) do
        if v.name == details.name then
            _dbg.Error("picManager", "该图片名字已经存在")
            return false, "该图片名字已经存在"
        end
        if v.url == details.url then
            _dbg.Error("picManager", "该图片已经存在于设备")
            return false, "该图片已经存在于设备"
        end
    end

    ---获取图片的 宽高
    local src_file = _M.PICPATH .. details.url

    local size_w, size_h 
    local f = io.popen( "/usr/bin/identify '" .. src_file .. "'")
    if f then
    	local line = f:read("*l")
    	f:close()
        if line then
            --利用identify 判断图片的格式 是否PNG
            -- local rr = line:match("PNG")
            -- if not rr then
            --     ---删除设备上面的图片
            --     _dbg.Error("图片的格式 不是PNG")
            --     local r, code = os.execute("rm -rf " .. src_file)
            --     return false, "上传文件必须是png文件"
            -- end

    		size_w, size_h = line:match(".-([0-9]+)[xX]([0-9]+)")
    		size_w = tonumber(size_w)
    		size_h = tonumber(size_h)
            if not size_w or not size_h then
                _dbg.Error("无法获取图片的宽高")
    			return false,"无法获取图片的宽高"
    		end
        else
    		return false
    	end
    else
        _dbg.Error("执行identify file 报错")
    	return false
    end
    details.size_w = size_w
    details.size_h = size_h

    -- 上传图片文件后自动转换成rgba文件
    -- local dest_file = _M.PICPATH .. details.url .. ".rgba"
    -- if _M.ConvertPic(src_file, dest_file) then
    --     details.rgba = details.url .. ".rgba"
    -- else
    --     _dbg.Error("picManager","ConvertPic error!")
    -- end

    table.insert(Cfg,details)
    local r, r2 = cfg_pics:__TRIGGER(true)
    return true
end


local function CallOsd(output,typ,url,position)
    ---通知 重新处理背景图或者迭加图
    local mod_relationImg = require "mod_relationImg"
    local proxy
    if typ == "bg" then
        local outputid
        if output == "1" or output == 1 then
            outputid = "hd0"
        elseif output == "2" or output == 2 then
            outputid = "hd1"
		end
		if cfg_imgRelation["output"..output][typ].enable and (cfg_imgRelation["output"..output][typ].enable == 0 or cfg_imgRelation["output"..output][typ].enable == "0" ) then
            return true           
        end
        proxy = mod_relationImg.GetProxy( "/output/"..outputid, "codec" )
    elseif typ == "osd" then
        proxy = mod_relationImg.GetProxy( "/osd", "OSD" )
    end
    if not proxy then
        _dbg.Error("picManager", "无法连接OSD服务")
		return false, "无法连接OSD服务"
    else
        local r, done,err
        if typ == "bg" then
            r, done, err = proxy.setBackGround(url)
        elseif typ == "osd" then
            r, done, err = proxy.setLogo(tonumber(output),url,position)
        end
        proxy:Destroy()
        if not r or not done then
            _dbg.Error("picManager", "设置 输出的叠加/背景图片失败!!!!err:",err)
            return false,"设置 输出的叠加/背景图片失败"
        end
    end
    _dbg.Info("通知 重新处理背景图或者迭加图 ok")
    return true
end

function _M.ModifyPic(tab, norun)
    if not norun then
        norun = false
    end
    local url
    if tonumber(tab.enable)==0 or tonumber(tab.id) == 0 then
        url = ""
    else
        url = IdToUrl(tonumber(tab.id))
    end
    local output = tab.output
    local type = tab.type

    if not norun then
        ---通知osd服务
        -- print("output~~~~~~~~~", output)
        local r,err = CallOsd(output,type,url,tab.position)
        if not r then
            return false,err
        end
    end
    
    cfg_imgRelation["output"..output][type].url = url
    cfg_imgRelation["output"..output][type].position = tab.position
    cfg_imgRelation:__TRIGGER(true)
    return true
end

--图片管理 页面查询使用
function _M.GetPics(type)
    local datas = {}
    if type then
        for k,v in pairs(cfg_pics) do
            if v.type == type then
                if type == "bg"  then
                    table.insert(datas,v)
                elseif type == "osd" then
                    table.insert(datas,v)
                end
            end
        end
    else
        datas = cfg_pics
    end
    return datas
end

function _M.GetPicByid(id)
    if not id then
        return nil
    end
    local k,v
    for k,v in pairs(cfg_pics) do
        if v.id == id then
            return v
        end
    end
    _dbg.Info("picManager", "GetPicByid no found!")
    return nil
end

-- type,可选参数osd/bg
--根据输出的1，2 及类型来查看 所有图片
--被选中的图片则enable=1
function _M.GetOutputPics(output,type)
    local pic
    local position 
    if tonumber(output) == 1 then
        if type == "bg" then
            pic=cfg_imgRelation.output1.bg.url
        elseif type == "osd" then
            pic=cfg_imgRelation.output1.osd.url

            position=cfg_imgRelation.output1.osd.position
        end
    else
        if type == "bg" then
            pic=cfg_imgRelation.output2.bg.url
        elseif type == "osd" then
            pic=cfg_imgRelation.output2.osd.url
            position=cfg_imgRelation.output2.osd.position
        end
    end
    local data = {}
    for k,pictabs in pairs(cfg_pics) do
        pictabs.enable = 0
        if pictabs.type == type then
            if pictabs.url == pic  then
                pictabs.enable=1
            end
            table.insert(data,pictabs)
        end
    end

    return data,position
end


--首页的输出 对应的图片地址
function _M.GetBGPic(output)
    local pic = ""
    if tonumber(output) == 1 then
        pic=cfg_imgRelation.output1.bg.url
    else
        pic=cfg_imgRelation.output2.bg.url
    end
    return pic
end


function _M.RemovePic(id)
    local Cfg = cfg_pics
    local img_name

    for k,v in pairs(Cfg) do
        if tonumber(v.id) == tonumber(id) then
            img_name=v.url
            local mod_relationImg = require "mod_relationImg"
            local used_enable = mod_relationImg.CheckPicUsed(img_name)
            if used_enable == 1 then
                _dbg.Error("图片被叠加，正在使用,不能删除!",img_name)
                return false , "图片已经被使用,不能删除!"
            end
            table.remove(Cfg,k)
        end
    end
    cfg_pics:__TRIGGER(true)
    img_name = _M.PICPATH .. img_name

    local r, code = os.execute("/bin/rm -rf " .. img_name)
    -- local r, code = os.execute("/bin/rm -rf " .. img_name..".rgba")
    
    return true
end

--根据分辨率 对图片进行处理 --bg
function _M.ResizePic(rate,picName,newfile)
    _dbg.Info("convert -quality 100 -resize '".. rate .. "!' " .._M.PICPATH..  picName .."  " .. newfile )
    local r, code = os.execute("convert -quality 100 -resize '".. rate .. "!' " .._M.PICPATH..  picName .."  " .. newfile )
    if not r then
        return false,code
    end
    return true
end


--图片转RGBA
function _M.ConvertPic(picName,newfile)
    _dbg.Info("convert  " .. picName .. " -depth 8 " .. "  " .. newfile)
    local r, code = os.execute("convert  " .. picName .. " -depth 8" .. " " .. newfile)
    if not r then
        return false,code
    end
    return true
end

--获取指定图片的大小
function _M.GetPicRate(src_file)
    local datas = {}
    for k,v in pairs(cfg_pics) do
        if v.url == src_file then
            return v.size_w,v.size_h
        end
    end
    return nil,nil,"找不到这张图片"
end

function _M.RemoveAllPic()
    local Cfg = cfg_pics
    for k,v in pairs(Cfg) do      
        _M.RemovePic(tonumber(v.id))
    end
    return true
end

return _M
