local DC = require "supertable.declare"
local CONF = require "supertable.core"

----------------------------
local _M = {
	config_location = "/data/configs",
	config_file = "roles.conf"
}

----------------------------

local users_template = DC.SET(
    DC.OBJ({
        users = DC.INT(0),
        role = DC.INT(0),
        systime = DC.INT(0),
        firmware = DC.INT(0),
        reset = DC.INT(0),
        reboot = DC.INT(0),
        restore = DC.INT(0),
        network = DC.INT(0),
        platform = DC.INT(0),
        authorization = DC.INT(0),
        setDeviceName = DC.INT(0),
        sourceManage = DC.INT(0),
        setSource = DC.INT(0),
        terminal = DC.INT(0),
        pattern = DC.INT(0),
        streamService = DC.INT(0),
        streamServiceControl = DC.INT(0),
        publishing = DC.INT(0),
        sip = DC.INT(0),
        outputFormate = DC.INT(0),
        coding = DC.INT(0),
        overlay = DC.INT(0),
        adjustment = DC.INT(0),
        recordControl = DC.INT(0),
        recordFile = DC.INT(0),
        pictures = DC.INT(0),
    })
): __changed( function(conf)
	conf:TAB_SAVE()
end)

----------------------------

local failDefault = {
    admin = {
        users = 1, role = 1, systime = 1, firmware = 1,
        reset = 1, reboot = 1, restore = 1, network = 1,
        platform = 1, authorization = 1, setDeviceName = 1, sourceManage = 1,
        setSource = 1, terminal = 1, pattern = 1, streamService = 1,
        streamServiceControl = 1, publishing = 1, sip = 1, outputFormate = 1,
        coding = 1, overlay = 1, adjustment = 1, recordControl = 1,
        recordFile = 1, pictures = 1, id = 1, alias = "admin",
    },
    user = {
        users = 1, role = 1, systime = 1, firmware = 1,
        reset = 1, reboot = 1, restore = 1, network = 1,
        platform = 1, authorization = 1, setDeviceName = 1, sourceManage = 1,
        setSource = 1, terminal = 1, pattern = 1, streamService = 1,
        streamServiceControl = 1, publishing = 1, sip = 1, outputFormate = 1,
        coding = 1, overlay = 1, adjustment = 1, recordControl = 1,
        recordFile = 1, pictures = 1, id = 2, alias = "user",
    }
}

-------------------------------------------------------

function _M.Load()
	local conf = CONF.Create({}, users_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_location .. "/" .. _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure 
		conf:TAB_BIND(failDefault)
	end
	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------

return _M


