local _dbg = require "coaf.debug"
local DC = require "supertable.declare"
local CONF = require "supertable.core"

----------------------------
local _M = {
	config_file = "/data/configs/sysctrl_timer.conf",
	cron_file = "/var/spool/cron/crontabs/root",
	timer_config = nil
}

----------------------------

local timer_template = DC.OBJ{
			enable = DC.BOOL(false),
			policy = DC.STR("day"),
			date = DC.OBJ{
					year =   DC.INT("0"),
					month =  DC.INT("0"),
					day =    DC.INT("0"),
					hour =   DC.INT("0"),
					minute = DC.INT("0"),
					second = DC.INT("0")
			},
			n_hour = DC.OBJ{
					hour =   DC.INT("0"),
					minute = DC.INT("0"),
					second = DC.INT("0")
				},
			n_minute = DC.OBJ{
					minute = DC.INT("0"),
					second = DC.INT("0")
				}
} : __changed( function(conf)
	print(conf:TAB_SAVE())
end)

local failDefault = {
	enable = false,
	policy = "day",
	date = {
		year = 0,
		month = 0,
		day = 0,
		hour = 0,
		minute = 0,
		second = 0
	},
	n_hour = {
		hour = 0,
		minute = 0,
		second = 0
	},
	n_minute = {
		minute = 0,
		second = 0
	}
}

-------------------------------------------------------
function _M.Load()
	local conf = CONF.Create({}, timer_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure
		conf:TAB_BIND(failDefault)
		conf:TAB_SAVE()
	end
	conf:TAB_RESET_CHANGE()
	return conf
end
-------------------------------------------------------

local function update(old, new)
    for i, v in pairs(old) do
        if type(new[i]) == "table" and type(v) == "table" then
            update(v, new[i])
        elseif type(v) ~= "table" and new[i] ~= nil then
            old[i] = new[i]
        end
    end
end

---------------------------------------------------------------
function _M.GetTimerInfo()
	return _M.timer_config
end
----------------------------------------------------
-- crontab 格式
-- *  *  *  *  *   cmd
--分 时 日 月 星期 CMD
-----------------------------------------------------
function _M.SetTimerInfo(cfg, cmd)
	if not cfg or not cfg.policy or cfg.policy == "" then
		return false
	end

	update(_M.timer_config, cfg)

	-- local f = io.open(_M.cron_file, "r")
	-- if  f then
	-- 	f:close()
	-- 	os.execute("/bin/rm  -rf " .. _M.cron_file)
	-- end

	os.execute("/usr/bin/crontab  -r ")

	if cfg.enable then
		local cron_cmds = nil
		if cfg.policy == "day" then
			cron_cmds = tostring(cfg.date.minute) .. " " .. tostring(cfg.date.hour) .. " *" ..  " *" .. " *" .. "    /sbin/reboot"
		elseif cfg.policy == "hour" then
			if cfg.n_hour.hour > 0 then
				cron_cmds = "0" .. " */" .. tostring(cfg.n_hour.hour) .. " *" ..  " *" .. " *" .. "    /sbin/reboot"
			end
		elseif cfg.policy == "minute" then
			if cfg.n_minute.minute > 0 then
				cron_cmds = "*/" ..tostring(cfg.n_minute.minute) ..  " *" .. " *" ..  " *" .. " *" .. "    /sbin/reboot"
			end
		end

		if cron_cmds then
		 	local r,code = os.execute("/bin/echo " .. "'" .. cron_cmds .. "'" .. "  >  " ..  _M.cron_file)
			_dbg.Info("res:",r,code)
		end
	end

	_M.timer_config:__TRIGGER(true)
	return true
end

function _M.Init()
	_dbg.Info("SYSCTRL Timer  Init")
	_M.timer_config = _M.Load()

	_M.SetTimerInfo(_M.timer_config, "/sbin/reboot" )
end

return _M