#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"
local service = require "coaf.service"

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"

local roll = require "coaf.log.rollfile"

local event = require "coaf.event"

local preset = require "presetManager"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

------------------------------------------------

local withLog = false
_G.globalCfgLocation = "/usr/share/lua/5.3"
local _x=1
while _x <= #arg do
	if arg[_x] == "--log" then
		withLog = true
	elseif arg[_x] == "--global-loc" then
		_x = _x + 1
		if _x <= #arg then
			_G.globalCfgLocation = arg[_x]
		end
	end
	_x = _x+1
end

-- First, initialize the log file.
if withLog then
	os.execute( "/bin/mkdir -p /var/log/sysctrl" )
	local logF = roll.Create("/var/log/sysctrl/sysctrl.log")
	if not logF or not logF:init() then
		_dbg.Error("SYSCTRL", "Fail to open log file: /var/log/sysctrler/sysctrler.log, so the log messages should be output to console.")
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
end


------------------------------------------------

_G.g_service = nil
_G.g_action = nil
_G.g_actionDelay = nil

coaf.Init( "systemctrl" )
-- Initialize service.
service.InitSocket()
client.InitSocket( socket, adapter )

local sysctrlService = require "sysctrl_base"
local datetimeService = require "sysctrl_datetime"
local authService = require "sysctrl_auth"
local rolesAuth = require "user_roles"
local sysctrlTimer = require "sysctrl_timer"

---------------------------------------------------------

local function Main(args)
	_G.g_event = event.createNew( "systemctrl", true ) -- Don't start slave thread, since we never use it.
	sysctrlService.Init()
	datetimeService.Init()
	authService.Init()
	rolesAuth.Init()
	sysctrlTimer.Init()

	g_service = service.New( "systemctrl", "unix:/var/run/systemctrl.sock" )
	if g_service then
		g_service:CreateCoServerProxy()
		g_service:ExposeObject("/", sysctrlService )
		g_service:ExposeObject("/datetime", datetimeService )
		g_service:ExposeObject("/auth", authService )
		g_service:ExposeObject("/roles", rolesAuth)
		g_service:ExposeObject("/preset", preset)
		g_service:ExposeObject("/sysctrlTimer", sysctrlTimer)
	else
		_dbg.Error( "SYSCTRL", "Fail to create CoService for systemctrl" )
	end
end

coaf.Start(Main,arg)

_dbg.Print( "SYSCTRL", "systemctrl service is terminated." )

if _G.g_action == "reboot" then
	_dbg.Warn( "SYSCTRL", "Reboot the device " .. (_G.g_actionDelay and ( "after " .. _G.g_actionDelay .. " seconds") or "now") )
	if _G.g_actionDelay then
		coaf.Sleep(_G.g_actionDelay)
	end
	os.execute( "/sbin/reboot" )
end

