#!/usr/bin/env lua
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

_G.TIME_TICK = 0

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	-- ** initialize the log file **
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/ui.log")
	if not logF or not logF:init() then
		_dbg.Error("UI", "Fail to open log file: /var/log/ui.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
	-- ** log **
end

coaf.Init( "ui_module" )

service.InitSocket()
client.InitSocket( socket, adapter )

local LED = require "ui_leds"
local SW = require "ui_switchs"


-- ** CoService **
local cosvc_ui
local ui_service = require "ui_service"

------------------

local counter = 0


local function for125ms_PeriodCheck()
	_G.TIME_TICK = _G.TIME_TICK + 0.125
	LED.Step()
	SW.Step()
end

-- ** Timers **
local for125msTimer

-- **Main entry**
local function Main(args)
	ui_service.Init()
	cosvc_ui = service.New( "ui", "unix:/var/run/ui.sock" )
	if cosvc_ui then
		_dbg.Info( "UI", "CoService of 'ui' is created." )
		cosvc_ui:CreateCoServerProxy()
		cosvc_ui:ExposeObject( "/", ui_service )
	else
		_dbg.Error( "UI", "Fail created CoService!" )
	end

	LED.SetLightStatus( "run", LED.BLINK )

	for125msTimer = coaf.NewTimer( for125ms_PeriodCheck, 1, 0.125 )
	if for125msTimer then
		for125msTimer:start()
	end
end

coaf.Start(Main, arg)

if for125msTimer then
	for125msTimer:stop()
end

_dbg.Print( "UI", "Service terminated." )

