local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local hiperiphery = require "hiperiphery"
local HIMM = hiperiphery.HIMM

local _M = {
	sws = {}
}

function _M.Init()
end

-- **SW options**
-- {
-- 		muxreg = ...,
-- 		muxval = ...,
-- 		pingroup = ...,
-- 		pin = ...,
-- 		pol = 0/1,
-- 		holdTimes = N(*125ms)
-- }
function _M.CreateSW(name, swOpts)
	if _M.sws[name] then
		return true
	end
	if not swOpts or not swOpts.pingroup or not swOpts.pin then
		return false
	end
	if swOpts.muxreg then
		HIMM.write( swOpts.muxreg, swOpts.muxval or 0 )
	end
	local val = HIMM.read( swOpts.pingroup + 0x400 )
	val = val & ~(1<<swOpts.pin)
	HIMM.write( swOpts.pingroup + 0x400, val )
	local pol = swOpts.pol or 0

	local sw = {
		pin = swOpts.pin,
		readReg = swOpts.pingroup + (1<<(swOpts.pin+2)),
		compareVal = pol << swOpts.pin,
		holdTimes = swOpts.holdTimes or 24,	--3 seconds (24*125ms)
		callback = swOpts.callback,
		callbackObj = swOpts.callbackObj,
		holdCnt = 0
	}

	sw.lastValue = HIMM.read( sw.readReg )
	_M.sws[name] = sw
	return true
end

function _M.RemoveSW(name)
	_M.sws[name] = nil
end

function _M.ChangeSWPolarity(name, pol)
	local sw = _M.sws[name]
	if sw then
		pol = pol or 0
		sw.compareVal = pol << sw.pin
		sw.holdCnt = 0
		sw.lastValue = nil
	end
end

function _M.Step()
	local n,sw
	local val
	for n,sw in pairs(_M.sws) do
		val = HIMM.read( sw.readReg )
		if val ~= sw.lastValue then
			sw.lastValue = val
			sw.holdCnt = 0
		elseif val == sw.compareVal then
			sw.holdCnt = sw.holdCnt + 1
		end

		if val == sw.compareVal then
			if sw.holdCnt >= sw.holdTimes then
				sw.holdCnt = 0
				if sw.callback then
					if sw.callbackObj then
						sw.callback( sw.callbackObj, n )
					else
						sw.callback( n )
					end
				end
			end
		end
	end
end

return _M

