local coaf = require "coaf.core"
local thread = require "coaf.thread"
local _dbg = require "coaf.debug"
local run_daemon = require "coaf.run_daemon"
local TOOLS = require "system.cmdtools"
local hwclock = require "system/hwclock"

local _M = {
	running_pid = nil,
	running_task = nil,
	running_servers = nil
}

local function waitTask(thrObj)
	_dbg.Info( "SYSCTRL", "NTP client is running..." )
	local r,flag = coaf.WaitPID(_M.running_pid)
	if not r then
		local st, signal = pcall(require, "posix.signal")
		if st and signal then
			signal.kill( _M.running_pid )
		end
	end
	_dbg.Info( "SYSCTRL", "NTP client is terminated." )
	_M.running_task = nil
	_M.running_pid = nil
end

function _M.NtpSync ( servers , mod_time )
	run_daemon.Init()
	local args = { "-nq" }
	local i
	for i=1, #servers do
		table.insert( args, "-p" )
		table.insert( args, servers[i] )
	end

	local synctask = run_daemon.Create({
        name = "ntpd sync",
        run = TOOLS.NTP_CLI,
		args = args,
		on_terminate = function ()
			hwclock.Store()
			if mod_time and mod_time.SyncAmba and type(mod_time.SyncAmba)=="function" then
				mod_time.SyncAmba()
			end
		end
    })

	if synctask then 
		synctask:Start()
	end
end

function _M.Start( servers , mod_time )
	if type(servers) ~= "table" then
		return false
	end

	if mod_time then
		_M.NtpSync(servers,mod_time)
	else
		_M.NtpSync(servers)
	end

	if _M.running_task then
		local sameArgs = true
		if not _M.running_servers then
			sameArgs = false
		else
			local i
			for i=1, #servers do
				if _M.running_servers[i] ~= servers[i] then
					sameArgs = false
					break
				end
			end
		end
	
		if sameArgs then
			return true
		end

		_M.Stop()
	end

	_M.running_pid = coaf.Fork( function()
		local st, posix_unistd = pcall(require,"posix.unistd")
		if not st or not posix_unistd then
			os.exit(1)
		else
			local args
			if type(TOOLS.NTP_CLI_ARGS) == "function" then
				args = TOOLS.NTP_CLI_ARGS(servers)
			elseif type(TOOLS.NTP_CLI_ARGS) == "table" then
				args = TOOLS.NTP_CLI_ARGS
			else
				args = servers
			end
			posix_unistd.execp( TOOLS.NTP_CLI, args )
			os.exit(0)
		end
	end)

	if _M.running_pid then
		_M.running_task = thread.New(waitTask)
		_M.running_task:Start()
		return true
	else
		return false
	end
end

function _M.Stop()
	if _M.running_task then
		_M.running_task:Terminate()
	end
end

return _M

