-- Upgrade pre-check
CHECK_DEVICE_TYPE = "E_DOCTOR_S"
CHECK_OEM_TYPE = nil
CHECK_VERSION_MATCH = true
VERSION_MATCH_ONLY = nil
VERSION_MATCH_LT = "0.7.2212"
VERSION_MATCH_GT = "0"
NEW_VERSION="0.7.2212"
---- Up before this are arguments auto generated by upgrading scripts ---

local client = require "coaf.client"
local _dbg = require "coaf.debug"
local adapter = require "coaf.cli_adapter.simple"

client.InitSocket(nil, adapter)

--------------------------------------
local function getDownloaderProxies()
	local proxy
	local cli
	cli = client.New( "_CO_DAEMON.coserver", "local" )
	if not cli then
		_dbg.Error( "UPGRADE.CO", "Fail to create coserver connection client")
		return nil
	end
	proxy = cli:CreateProxy("/discovery", true)
	if not proxy then
		_dbg.Error( "UPGRADE.CO", "Fail to create _CO_DAEMON.coserver:/discovery proxy" )
		cli:Destroy()
		return nil
	end
	local done, all_locations = proxy.GetDiscoveryList()
	proxy:Destroy()
	cli:Destroy()
	proxy = nil
	cli = nil

	if not done or type(all_locations) ~= "table" then
		_dbg.Error( "UPGRADE.CO", "Fail to get discovered list:", all_locations )
		return nil
	end

	local localMods = {}
	local i,item
	for i, item in pairs(all_locations) do
		if item and item.id and item.options and item.options["group"] == "edoctor.local-module" then
			table.insert( localMods, item.id )
		end
	end

	local rslt = {}
	for i, item in ipairs(localMods) do
		cli = client.New( "downloader", item )
		if not cli then
			_dbg.Error( "UPGRADE.CO", "Fail to create downloader client for:", item)
		else
			proxy = cli:CreateProxy( "/", true )
			if proxy then
				table.insert( rslt, proxy )
			else
				cli:Destroy()
				cli = nil
				_dbg.Error( "UPGRADE.CO", "Fail to create downloader proxy for:", item)
			end
		end
	end

	return rslt
end

local function sleep(n)
	if ngx and ngx.sleep then
		ngx.sleep(n)
	else
		local r, coaf = pcall( require, "coaf.core" )
		if r and coaf then
			coaf.Sleep(n)
		else
			local socket
			r, socket = pcall( require, "socket" )
			if r and socket then
				socket.sleep(n)
			else
				os.execute( "/bin/usleep " .. math.ceil(n*1000000) )
			end
		end
	end
end

local function UpgradePreCheck( tmpLoc )
	if CHECK_DEVICE_TYPE or CHECK_OEM_TYPE then
		local done, features = pcall(require,"features")
		if not done or type(features)~="table" or not features.DEVICE_TYPE then
			--return false, "device-type-error"
			--Since this is a very old version, ignore the later verifications.
			return true, "done"
		end
		if CHECK_DEVICE_TYPE and CHECK_DEVICE_TYPE ~= features.DEVICE_TYPE then
			return false, "device-type-error"
		end
		if CHECK_OEM_TYPE and CHECK_OEM_TYPE ~= features.PRODUCT_ID then
			return false, "device-type-error"
		end
	end

	if CHECK_VERSION_MATCH then
		local done, _r = pcall(require,"version")
		if not done or not SOFTWARE_VERSION then
			return false, "device-type-error"
		end
		if VERSION_MATCH_ONLY then
			if SOFTWARE_VERSION ~= VERSION_MATCH_ONLY and SOFTWARE_VERSION ~= NEW_VERSION then
				return false, "version-wrong"
			end
		--Only matched numeric '.????' suffix.
		elseif SOFTWARE_VERSION:match("%.%d?%d?%d?%d?$") then
			if SOFTWARE_VERSION < VERSION_MATCH_GT then
				return false, "bin-base-version-is-higher"
			elseif SOFTWARE_VERSION > VERSION_MATCH_LT then
				return false, "bin-version-is-old"
			end
		end
	end

--	os.execute("/bin/rm -rf /data/configs/relationImg.conf")
--	os.execute("/bin/rm -rf /data/configs/pics.conf")

	local copack_f = io.open( tmpLoc .. "/_COPACK/co-upgrade.bin", "r" )
	local copack_upgrade=false
	if copack_f then
		copack_f:close()
		copack_upgrade = true
	end

	if copack_upgrade then
		local dl = getDownloaderProxies()
		if not dl then
			return false, "precheck.co-error"
		end

		os.execute( "/bin/mkdir -p /tmp/co-package" )
		os.execute( "/bin/mv -f " .. tmpLoc .. "/_COPACK/co-upgrade.bin /tmp/co-package/co-upgrade.bin" )

		local n
		local did, done, err, status

		local busy = {}
		local errors = {}

		for n=1, #dl do
			did, done, err = dl[n].StartDownload( "firmware", "/co-package/co-upgrade.bin", "co-upgrade.bin" )
			if not did or not done then
				_dbg.Error( "UPGRADE.CO", "Start download error for #", n, ", message is:", err )
				errors[n] = err or 'ERROR'
				dl[n]:Destroy()
				dl[n] = nil
			else
				busy[n] = true
			end
		end

		while #busy > 0 do
			for n=1, #dl do
				if dl[n] then
					did, status, err = dl[n].GetDownloadStatus( "firmware", "co-upgrade.bin" )
					if did and status and status.status == "finished" then
					--Try again to confirm that.
						sleep(0.2)
						did, status, err = dl[n].GetDownloadStatus( "firmware", "co-upgrade.bin" )
					end

					if did and status then
						if status.status == "done" or status.status == "finished" then
							busy[n] = nil
							dl[n]:Destroy()
							dl[n] = nil
						elseif status.status ~= "pending" and status.status ~= "downloading" then
							busy[n] = nil
							errors[n] = status.status
							dl[n]:Destroy()
							dl[n] = nil
							_dbg.Error( "UPGRADE.CO", "Download error for #", n, ", status:", status.status, status.message )
						end
					else
						busy[n] = nil
						errors[n] = err or "ERROR"
						dl[n]:Destroy()
						dl[n] = nil
						_dbg.Error( "UPGRADE.CO", "Download fail, status:", did, status, err )
					end
				end
			end

			if #busy > 0 then
				sleep(0.5)
			end
		end

		if #errors > 0 then
			return false, "precheck.co-download"
		end
	end
	return true, "done"
end

return UpgradePreCheck

